@@ -155,7 +155,7 @@ static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, const char *con
 	/* Free allocated memory */
 	free(buff);
 
-	if (s->ConnectionType==0) {
+	if (s->ConnectionType == 0) {
 		return ERR_UNKNOWNCONNECTIONTYPESTRING;
 	}
 
@@ -224,8 +224,11 @@ static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, const char *con
 #ifdef GSM_ENABLE_BLUEOBEX
 	GSM_RegisterConnection(s, GCT_BLUEOBEX,   &BlueToothDevice,&OBEXProtocol);
 #endif
-	if (s->Device.Functions==NULL || s->Protocol.Functions==NULL)
-			return ERR_SOURCENOTAVAILABLE;
+	if (s->Device.Functions == NULL || s->Protocol.Functions == NULL) {
+		smprintf(s, "Connection %s is know but was disabled on compile time\n", connection);
+		return ERR_DISABLED;
+	}
+
 	return ERR_NONE;
 }
 
@@ -291,6 +294,15 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 			return ERR_NONE;
 		}
 #endif
+
+		/* With GNAPgen and auto model we can work with unknown models too */
+#ifdef GSM_ENABLE_GNAPGEN
+		if (s->ConnectionType == GCT_BLUEGNAPBUS || s->ConnectionType == GCT_IRDAGNAPBUS) {
+			smprintf(s,"[Module           - \"%s\"]\n",GNAPGENPhone.models);
+			s->Phone.Functions = &GNAPGENPhone;
+			return ERR_NONE;
+		}
+#endif
 		if (model->model[0] == 0) return ERR_UNKNOWNMODELSTRING;
 	}
 	s->Phone.Functions=NULL;
@@ -461,6 +473,9 @@ GSM_Error GSM_TryGetModel(GSM_StateMachine *s)
 
 		error=s->Phone.Functions->GetModel(s);
 		if (error != ERR_NONE) return error;
+
+		error=s->Phone.Functions->Terminate(s);
+		if (error != ERR_NONE) return error;
 	}
 	return ERR_NONE;
 }
@@ -509,11 +524,13 @@ GSM_Error GSM_InitConnection(GSM_StateMachine *s, int ReplyNum)
 
 		s->di 				  = di;
 		s->di.use_global 		  = s->CurrentConfig->UseGlobalDebugFile;
-		GSM_SetDebugLevel(s->CurrentConfig->DebugLevel, &s->di);
-		error = GSM_SetDebugFile(s->CurrentConfig->DebugFile, &s->di);
-		if (error != ERR_NONE) {
-			GSM_LogError(s, "Init:GSM_SetDebugFile" , error);
-			return error;
+		if (!s->di.use_global) {
+			GSM_SetDebugLevel(s->CurrentConfig->DebugLevel, &s->di);
+			error = GSM_SetDebugFile(s->CurrentConfig->DebugFile, &s->di);
+			if (error != ERR_NONE) {
+				GSM_LogError(s, "Init:GSM_SetDebugFile" , error);
+				return error;
+			}
 		}
 
 		smprintf_level(s, D_ERROR, "[Gammu            - %s built %s %s using %s]\n",
@@ -664,7 +681,7 @@ int GSM_ReadDevice (GSM_StateMachine *s, bool waitforreply)
 		res = s->Device.Functions->ReadDevice(s, buff, 255);
 		if (!waitforreply) break;
 		if (res > 0) break;
-		my_sleep(5);
+		usleep(5000);
 		GSM_GetCurrentDateTime(&Date);
 	}
 
@@ -723,7 +740,11 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 		}
 
 		/* Some data received. Reset timer */
-		if (GSM_ReadDevice(s,true)!=0) i=0;
+		if (GSM_ReadDevice(s, true) != 0) {
+			i = 0;
+		} else {
+			usleep(10000);
+		}
 
 		if (length != 0) {
 			free (sentmsg.Buffer);
