@@ -21,7 +21,8 @@
 #include <locale.h>
 #include <limits.h>
 #ifdef WIN32
-#  include "windows.h"
+#  define WIN32_LEAN_AND_MEAN
+#  include <windows.h>
 #endif
 
 #include "../misc.h"
@@ -249,7 +250,7 @@ int DecodeWithUnicodeAlphabet(wchar_t src, unsigned char *dest)
         }
 }
 
-void DecodeUnicode (const unsigned char *src, unsigned char *dest)
+void DecodeUnicode (const unsigned char *src, char *dest)
 {
  	int 		i=0,o=0;
  	wchar_t 	wc;
@@ -263,7 +264,7 @@ void DecodeUnicode (const unsigned char *src, unsigned char *dest)
 }
 
 /* Decode Unicode string and return as function result */
-unsigned char *DecodeUnicodeString (const unsigned char *src)
+char *DecodeUnicodeString (const unsigned char *src)
 {
  	static char dest[500];
 
@@ -274,7 +275,7 @@ unsigned char *DecodeUnicodeString (const unsigned char *src)
 /* Decode Unicode string to UTF8 or other console charset
  * and return as function result
  */
-unsigned char *DecodeUnicodeConsole(const unsigned char *src)
+char *DecodeUnicodeConsole(const unsigned char *src)
 {
  	static char dest[500];
 
@@ -302,9 +303,9 @@ unsigned char *DecodeUnicodeConsole(const unsigned char *src)
 /* Encode string to Unicode. Len is number of input chars */
 void DecodeISO88591 (unsigned char *dest, const char *src, int len)
 {
-	int 		i = 0;
+	int i;
 
-	while (src[i] != 0) {
+	for (i = 0; i < len; i++) {
 		/* Hack for Euro sign */
 		if ((unsigned char)src[i] == 0x80) {
 			dest[2 * i] = 0x20;
@@ -313,7 +314,6 @@ void DecodeISO88591 (unsigned char *dest, const char *src, int len)
 			dest[2 * i] = 0;
 			dest[(2 * i) + 1] = src[i];
 		}
-		i++;
 	}
 	dest[2 * i] = 0;
 	dest[(2 * i) + 1] = 0;
@@ -381,9 +381,15 @@ void EncodeBCD (unsigned char *dest, const unsigned char *src, int len, bool fil
 
 int DecodeWithHexBinAlphabet (unsigned char mychar)
 {
-	if (mychar>='A' && mychar<='F') return mychar-'A'+10;
-	if (mychar>='a' && mychar<='f') return mychar-'a'+10;
-	if (mychar>='0' && mychar<='9') return mychar-'0';
+	if (mychar >= 'A' && mychar <= 'F')
+		return mychar - 'A' + 10;
+
+	if (mychar >= 'a' && mychar <= 'f')
+		return mychar - 'a' + 10;
+
+	if (mychar >= '0' && mychar <= '9')
+		return mychar - '0';
+
 	return -1;
 }
 
@@ -394,57 +400,47 @@ unsigned char EncodeWithHexBinAlphabet (int digit)
 	return 0;
 }
 
-void DecodeHexUnicode (unsigned char *dest, const unsigned char *src, int len)
+void DecodeHexUnicode (unsigned char *dest, const char *src, size_t len)
 {
-	int 	i,current=0;
-	bool 	first = false;
+	size_t i, current = 0;
 
-	if (len != 0 && src[0] == '0' && src[1] == '0') {
-		first = true;
-	} else if (len != 0 && src[2] == '0' && src[3] == '0') {
-		first = false;
-	} else {
-		first = (10 * (src[0] - '0') + (src[1] - '0')) < (10 * (src[2] - '0')+ (src[3] - '0'));
-	}
-	for (i = 0; i < len/4 ; i++) {
-		if (first) {
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+0])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+1]);
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+2])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+3]);
-		} else {
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+2])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+3]);
-			dest[current++] = DecodeWithHexBinAlphabet(src[i*4+0])*16+
-					  DecodeWithHexBinAlphabet(src[i*4+1]);
-		}
+	for (i = 0; i < len ; i += 4) {
+		dest[current++] =
+			(DecodeWithHexBinAlphabet(src[i + 0]) << 4) +
+			DecodeWithHexBinAlphabet(src[i + 1]);
+		dest[current++] =
+			(DecodeWithHexBinAlphabet(src[i + 2]) << 4) +
+			DecodeWithHexBinAlphabet(src[i + 3]);
 	}
 	dest[current++] = 0;
 	dest[current++] = 0;
 }
 
-void EncodeHexUnicode (unsigned char *dest, const unsigned char *src, int len)
+void EncodeHexUnicode (char *dest, const unsigned char *src, size_t len)
 {
-	int i,current=0;
+	size_t i, current = 0;
 
 	for (i = 0; i < len; i++) {
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] >> 0x04);
+		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] >> 4);
 		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] & 0x0f);
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] >> 0x04);
+		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] >> 4);
 		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] & 0x0f);
 	}
 	dest[current++] = 0;
 }
 
-void DecodeHexBin (unsigned char *dest, const unsigned char *src, int len)
+bool DecodeHexBin (unsigned char *dest, const unsigned char *src, int len)
 {
-	int i,current=0;
+	int i,current=0, low, high;
 
 	for (i = 0; i < len/2 ; i++) {
-		dest[current++] = DecodeWithHexBinAlphabet(src[i*2])*16+
-				  DecodeWithHexBinAlphabet(src[i*2+1]);
+		low = DecodeWithHexBinAlphabet(src[i*2+1]);
+		high = DecodeWithHexBinAlphabet(src[i*2]);
+		if (low < 0 || high < 0) return false;
+		dest[current++] = (high << 4) | low;
 	}
 	dest[current++] = 0;
+	return true;
 }
 
 void EncodeHexBin (unsigned char *dest, const unsigned char *src, int len)
@@ -518,9 +514,9 @@ static unsigned char GSM_DefaultAlphabetCharsExtension[][4] =
 	{0x00,0x00,0x00,0x00}
 };
 
-void DecodeDefault (unsigned char *dest, const unsigned char *src, int len, bool UseExtensions, unsigned char *ExtraAlphabet)
+void DecodeDefault (unsigned char *dest, const unsigned char *src, size_t len, bool UseExtensions, unsigned char *ExtraAlphabet)
 {
-	int 	i,current=0,j;
+	size_t 	i,current=0,j;
 	bool	FoundSpecial = false;
 
 #ifdef DEBUG
@@ -582,9 +578,9 @@ static unsigned char ConvertTable[] =
 "\x00\x74\x01\x66\x00\x54\x01\x67\x00\x74\x00\xd9\x00\x55\x00\xda\x00\x55\x00\xfa\x00\x75\x00\xdb\x00\x55\x00\xfb\x00\x75\x01\x68\x00\x55\x01\x69\x00\x75\x01\x6a\x00\x55\x01\x6b\x00\x75\x01\x6c\x00\x55\x01\x6d\x00\x75\x01\x6e\x00\x55\x01\x6f\x00\x75\x01\x70\x00\x55\x01\x71\x00\x75\x01\x72\x00\x55\x01\x73\x00\x75\x01\xaf\x00\x55\x01\xb0\x00\x75\x01\xd3\x00\x55\x01\xd4\x00\x75\x01\xd5\x00\x55\x01\xd6\x00\x75\x01\xd7\x00\x55\x01\xd8\x00\x75\x01\xd9\x00\x55\x01\xda\x00\x75\x01\xdb\x00\x55\x01\xdc\x00\x75\x1e\xe4\x00\x55\x1e\xe5\x00\x75\x1e\xe6\x00\x55\x1e\xe7\x00\x75\x1e\xe8\x00\x55\x1e\xe9\x00\x75\x1e\xea\x00\x55\x1e\xeb\x00\x75\x1e\xec\x00\x55\x1e\xed\x00\x75\x1e\xee\x00\x55\x1e\xef\x00\x75\x1e\xf0\x00\x55\x1e\xf1\x00\x75\x01\x74\x00\x57\x01\x75\x00\x77\x1e\x80\x00\x57\x1e\x81\x00\x77\x1e\x82"\
 "\x00\x57\x1e\x83\x00\x77\x1e\x84\x00\x57\x1e\x85\x00\x77\x00\xdd\x00\x59\x00\xfd\x00\x79\x00\xff\x00\x79\x01\x76\x00\x59\x01\x77\x00\x79\x01\x78\x00\x59\x1e\xf2\x00\x59\x1e\xf3\x00\x75\x1e\xf4\x00\x59\x1e\xf5\x00\x79\x1e\xf6\x00\x59\x1e\xf7\x00\x79\x1e\xf8\x00\x59\x1e\xf9\x00\x79\x01\x79\x00\x5a\x01\x7a\x00\x7a\x01\x7b\x00\x5a\x01\x7c\x00\x7a\x01\x7d\x00\x5a\x01\x7e\x00\x7a\x01\xfc\x00\xc6\x01\xfd\x00\xe6\x01\xfe\x00\xd8\x01\xff\x00\xf8\x00\x00";
 
-void EncodeDefault(unsigned char *dest, const unsigned char *src, int *len, bool UseExtensions, unsigned char *ExtraAlphabet)
+void EncodeDefault(unsigned char *dest, const unsigned char *src, size_t *len, bool UseExtensions, unsigned char *ExtraAlphabet)
 {
-	int 	i,current=0,j,z;
+	size_t 	i,current=0,j,z;
 	char 	ret;
 	bool	FoundSpecial,FoundNormal;
 
@@ -664,9 +660,9 @@ void EncodeDefault(unsigned char *dest, const unsigned char *src, int *len, bool
 }
 
 /* You don't have to use ConvertTable here - 1 char is replaced there by 1 char */
-void FindDefaultAlphabetLen(const unsigned char *src, int *srclen, int *smslen, int maxlen)
+void FindDefaultAlphabetLen(const unsigned char *src, size_t *srclen, size_t *smslen, size_t maxlen)
 {
-	int 	current=0,j,i;
+	size_t 	current=0,j,i;
 	bool	FoundSpecial;
 
 	i = 0;
@@ -701,14 +697,11 @@ void FindDefaultAlphabetLen(const unsigned char *src, int *srclen, int *smslen,
 	*smslen = current;
 }
 
-#ifndef ENABLE_LGPL
-#  define ByteMask ((1 << Bits) - 1)
-#endif
+#define ByteMask ((1 << Bits) - 1)
 
 int GSM_UnpackEightBitsToSeven(int offset, int in_length, int out_length,
                            unsigned char *input, unsigned char *output)
 {
-#ifndef ENABLE_LGPL
 	/* (c) by Pavel Janik and Pawel Kot */
 
         unsigned char *OUTPUT 	= output; /* Current pointer to the output buffer */
@@ -744,14 +737,10 @@ int GSM_UnpackEightBitsToSeven(int offset, int in_length, int out_length,
         }
 
         return OUTPUT - output;
-#else
-	return 0;
-#endif
 }
 
 int GSM_PackSevenBitsToEight(int offset, unsigned char *input, unsigned char *output, int length)
 {
-#ifndef ENABLE_LGPL
 	/* (c) by Pavel Janik and Pawel Kot */
 
         unsigned char 	*OUTPUT = output; /* Current pointer to the output buffer */
@@ -783,44 +772,60 @@ int GSM_PackSevenBitsToEight(int offset, unsigned char *input, unsigned char *ou
                 INPUT++;
         }
         return (OUTPUT - output);
-#else
-	return 0;
-#endif
 }
 
 void GSM_UnpackSemiOctetNumber(unsigned char *retval, unsigned char *Number, bool semioctet)
 {
-	unsigned char	Buffer[50]	= "";
+	unsigned char	Buffer[GSM_MAX_NUMBER_LENGTH + 1];
 	int		length		= Number[0];
 
+	/* Default ouput on error */
+	strcpy(Buffer, "<NOT DECODED>");
+
 	if (semioctet) {
 		/* Convert number of semioctets to number of chars */
 		if (length % 2) length++;
 		length=length / 2 + 1;
 	}
 
+	/* Check length */
+	if (length > GSM_MAX_NUMBER_LENGTH) {
+		dbgprintf("Number too big, not decoding! (Length=%d, MAX=%d)\n", length, GSM_MAX_NUMBER_LENGTH);
+		goto out;
+	}
+
 	/*without leading byte with format of number*/
 	length--;
 
-	switch ((Number[1] & 112)) {
-	case (NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN & 112):
+	switch ((Number[1] & 0x70)) {
+	case (NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN & 0x70):
 		if (length > 6) length++;
 		dbgprintf("Alphanumeric number, length %i\n",length);
 		GSM_UnpackEightBitsToSeven(0, length, length, Number+2, Buffer);
 		Buffer[length]=0;
 		break;
-	case (NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN & 112):
+	case (NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN & 0x70):
 		dbgprintf("International number\n");
 		Buffer[0]='+';
 		DecodeBCD(Buffer+1,Number+2, length);
 		break;
 	default:
-		dbgprintf("Default number %02x\n",Number[1]);
+		dbgprintf("Default number %02x (%d %d %d %d|%d %d %d %d)\n",Number[1],
+				Number[1] & 0x80 ? 1 : 0,
+				Number[1] & 0x40 ? 1 : 0,
+				Number[1] & 0x20 ? 1 : 0,
+				Number[1] & 0x10 ? 1 : 0,
+				Number[1] & 0x08 ? 1 : 0,
+				Number[1] & 0x04 ? 1 : 0,
+				Number[1] & 0x02 ? 1 : 0,
+				Number[1] & 0x01 ? 1 : 0
+				);
 		DecodeBCD (Buffer, Number+2, length);
 		break;
 	}
 
 	dbgprintf("Len %i\n",length);
+out:
 	EncodeUnicode(retval,Buffer,strlen(Buffer));
 }
 
@@ -841,21 +846,23 @@ void GSM_UnpackSemiOctetNumber(unsigned char *retval, unsigned char *Number, boo
  */
 int GSM_PackSemiOctetNumber(unsigned char *Number, unsigned char *Output, bool semioctet)
 {
-	unsigned char	format, buffer[50];
+	unsigned char	format, buffer[GSM_MAX_NUMBER_LENGTH + 1];
 	int		length, i;
 
-	length=UnicodeLength(Number);
-	memcpy(buffer,DecodeUnicodeString(Number),length+1);
+	length = UnicodeLength(Number);
+	memcpy(buffer, DecodeUnicodeString(Number), length + 1);
 
 	/* Checking for format number */
-	format = NUMBER_UNKNOWN_NUMBERING_PLAN_ISDN;
-	for (i=0;i<length;i++) {
-		/* first byte is '+'. Number can be international */
-		if (i==0 && buffer[i]=='+') {
-			format=NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
-		} else {
-			/*char is not number. It must be alphanumeric*/
-			if (!isdigit(buffer[i])) format=NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN;
+	if (buffer[0] == '+' || (buffer[0] == '0' && buffer[1] == '0')) {
+		format = NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
+	} else {
+		format = NUMBER_UNKNOWN_NUMBERING_PLAN_ISDN;
+	}
+	for (i = 0; i < length; i++) {
+		/* If there is something which can not be in normal
+		 * number, mark it as alphanumberic */
+		if (strchr("+0123456789*#pP", buffer[i]) == NULL) {
+			format = NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN;
 		}
 	}
 
@@ -887,7 +894,7 @@ int GSM_PackSemiOctetNumber(unsigned char *Number, unsigned char *Output, bool s
 	return length / 2 + 1;
 }
 
-void CopyUnicodeString(unsigned char *Dest, unsigned char *Source)
+void CopyUnicodeString(unsigned char *Dest, const unsigned char *Source)
 {
 	int j = 0;
 
@@ -922,7 +929,7 @@ void ReverseUnicodeString(unsigned char *String)
 
 /* All input is in Unicode. First char can show Unicode minor/major order.
    Output is Unicode string in Gammu minor/major order */
-void ReadUnicodeFile(unsigned char *Dest, unsigned char *Source)
+void ReadUnicodeFile(unsigned char *Dest, const unsigned char *Source)
 {
 	int j = 0, current = 0;
 
@@ -943,22 +950,22 @@ void ReadUnicodeFile(unsigned char *Dest, unsigned char *Source)
 	Dest[current++]	= 0;
 }
 
-int GetBit(unsigned char *Buffer, int BitNum)
+INLINE int GetBit(unsigned char *Buffer, size_t BitNum)
 {
 	return Buffer[BitNum/8] & 1<<(7-(BitNum%8));
 }
 
-int SetBit(unsigned char *Buffer, int BitNum)
+INLINE int SetBit(unsigned char *Buffer, size_t BitNum)
 {
 	return Buffer[BitNum/8] |= 1<<(7-(BitNum%8));
 }
 
-int ClearBit(unsigned char *Buffer, int BitNum)
+int ClearBit(unsigned char *Buffer, size_t BitNum)
 {
 	return Buffer[BitNum/8] &= 255 - (1 << (7-(BitNum%8)));
 }
 
-void BufferAlign(unsigned char *Destination, int *CurrentBit)
+void BufferAlign(unsigned char *Destination, size_t *CurrentBit)
 {
 	int i=0;
 
@@ -970,7 +977,7 @@ void BufferAlign(unsigned char *Destination, int *CurrentBit)
 	(*CurrentBit) = (*CurrentBit) + i;
 }
 
-void BufferAlignNumber(int *CurrentBit)
+void BufferAlignNumber(size_t *CurrentBit)
 {
 	int i=0;
 
@@ -982,11 +989,11 @@ void BufferAlignNumber(int *CurrentBit)
 }
 
 void AddBuffer(unsigned char 	*Destination,
-	       int 		*CurrentBit,
+	       size_t		*CurrentBit,
 	       unsigned char 	*Source,
-	       int 		BitsToProcess)
+	       size_t 		BitsToProcess)
 {
-	int i=0;
+	size_t i=0;
 
 	while (i!=BitsToProcess) {
 		if (GetBit(Source, i)) {
@@ -1000,9 +1007,9 @@ void AddBuffer(unsigned char 	*Destination,
 }
 
 void AddBufferByte(unsigned char *Destination,
-		   int 		 *CurrentBit,
+		   size_t		 *CurrentBit,
 		   unsigned char Source,
-		   int 		 BitsToProcess)
+		   size_t		 BitsToProcess)
 {
 	unsigned char Byte;
 
@@ -1012,11 +1019,11 @@ void AddBufferByte(unsigned char *Destination,
 }
 
 void GetBuffer(unsigned char *Source,
-	       int 	     *CurrentBit,
+	       size_t	     *CurrentBit,
 	       unsigned char *Destination,
-	       int 	     BitsToProcess)
+	       size_t	     BitsToProcess)
 {
-	int i=0;
+	size_t i=0;
 
 	while (i!=BitsToProcess) {
 		if (GetBit(Source, (*CurrentBit)+i)) {
@@ -1030,11 +1037,11 @@ void GetBuffer(unsigned char *Source,
 }
 
 void GetBufferInt(unsigned char *Source,
-		  int 		*CurrentBit,
+		  size_t	*CurrentBit,
 		  int 		*integer,
-		  int 		BitsToProcess)
+		  size_t	BitsToProcess)
 {
-	int l=0,z=128,i=0;
+	size_t l=0,z=128,i=0;
 
 	while (i!=BitsToProcess) {
 		if (GetBit(Source, (*CurrentBit)+i)) l=l+z;
@@ -1046,11 +1053,11 @@ void GetBufferInt(unsigned char *Source,
 }
 
 void GetBufferI(unsigned char 	*Source,
-		int 		*CurrentBit,
+		size_t 		*CurrentBit,
 		int 		*result,
-		int 		BitsToProcess)
+		size_t 		BitsToProcess)
 {
-	int l=0,z,i=0;
+	size_t l=0,z,i=0;
 
 	z = 1<<(BitsToProcess-1);
 
@@ -1268,7 +1275,7 @@ bool myiswspace(unsigned const char *src)
 
 #ifndef HAVE_TOWLOWER
 /* FreeBSD boxes 4.7-STABLE does't have it, although it's ANSI standard */
-int towlower(wchar_t c)
+wchar_t towlower(wchar_t c)
 {
 	unsigned char dest[10];
 
@@ -1296,7 +1303,9 @@ unsigned char *mywstrstr (const unsigned char *haystack, const unsigned char *ne
 {
 /* One crazy define to convert unicode used in Gammu to standard wchar_t */
 #define tolowerwchar(x) (towlower((wchar_t)( (((&(x))[0] & 0xff) << 8) | (((&(x))[1] & 0xff)) )))
-	 register wchar_t b, c;
+	register wint_t a, b, c;
+	register const unsigned char *rhaystack, *rneedle;
+
 
 	if ((b = tolowerwchar(*needle)) != L'\0') {
 		haystack -= 2;				/* possible ANSI violation */
@@ -1313,9 +1322,6 @@ unsigned char *mywstrstr (const unsigned char *haystack, const unsigned char *ne
 		goto jin;
 
 		for (;;) {
-			register wchar_t a;
-			register const unsigned char *rhaystack, *rneedle;
-
 			do {
 				haystack += 2;
 				if ((a = tolowerwchar(*haystack)) == L'\0')
@@ -1365,36 +1371,41 @@ ret0:
 #undef tolowerwchar
 }
 
-/**
- * Gets line from buffer.
- *
- * @param MergeLines: determine whether merge lines ending with = (quoted printable)
- * @param Buffer: Data source to parse
- * @param Pos: Current position in data
- * @param OutBuffer: Buffer where line will be written
- * @param MaxLen: Maximal length of data to write
- */
-void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int MaxLen, bool MergeLines)
+GSM_Error MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int MaxLen, size_t MaxOutLen, bool MergeLines)
 {
 	bool skip = false;
+	bool quoted_printable = false;
+	bool was_cr = false, was_lf = false;
 	size_t pos;
 	int tmp;
 
 	OutBuffer[0] = 0;
 	pos = 0;
-	if (Buffer == NULL) return;
+	if (Buffer == NULL) return ERR_NONE;
 	while ((*Pos) < MaxLen) {
 		switch (Buffer[*Pos]) {
 		case 0x00:
-			return;
+			return ERR_NONE;
 		case 0x0A:
 		case 0x0D:
+			if (skip) {
+				if (Buffer[*Pos] == 0x0d) {
+					if (was_cr && skip) return ERR_NONE;
+					was_cr = true;
+				} else {
+					if (was_lf && skip) return ERR_NONE;
+					was_lf = true;
+				}
+			}
 			if (pos != 0 && !skip) {
-				if (MergeLines) { 
+				if (MergeLines) {
 					/* (Quote printable new line) Does string end with = ? */
-					if (OutBuffer[pos - 1] == '=') {
-						OutBuffer[pos - 1] = 0;
+					if (OutBuffer[pos - 1] == '=' && quoted_printable) {
+						pos--;
+						OutBuffer[pos] = 0;
 						skip = true;
+						was_cr = (Buffer[*Pos] == 0x0d);
+						was_lf = (Buffer[*Pos] == 0x0a);
 						break;
 					}
 					/* (vCard continuation) Next line start with space? */
@@ -1407,17 +1418,24 @@ void MyGetLine(unsigned char *Buffer, int *Pos, unsigned char *OutBuffer, int Ma
 						break;
 					}
 				}
-				return;
+				return ERR_NONE;
 			}
 			break;
-		default  :
+		default:
+			/* Detect quoted printable for possible escaping */
+			if (Buffer[*Pos] == ':' &&
+					strstr(OutBuffer, ";ENCODING=QUOTED-PRINTABLE") != NULL) {
+				quoted_printable = true;
+			}
 			skip = false;
 			OutBuffer[pos]     = Buffer[*Pos];
 			pos++;
 			OutBuffer[pos] = 0;
+			if (pos + 1 >= MaxOutLen) return ERR_MOREMEMORY;
 		}
 		(*Pos)++;
 	}
+	return ERR_NONE;
 }
 
 void StringToDouble(char *text, double *d)
@@ -1441,17 +1459,17 @@ void StringToDouble(char *text, double *d)
 }
 
 /* When char can be converted, convert it from Unicode to UTF8 */
-int EncodeWithUTF8Alphabet2(unsigned char mychar1, unsigned char mychar2, unsigned char *ret)
+int EncodeWithUTF8Alphabet(unsigned char mychar1, unsigned char mychar2, unsigned char *ret)
 {
 	int src = mychar1*256+mychar2;
 
-	//Unicode 0080-07FF -> UTF8 110xxxxx 10xxxxxx
+	/* Unicode 0080-07FF -> UTF8 110xxxxx 10xxxxxx */
 	if (src >=128 && src <=2047) {
 		ret[0] = 192 + (src / 64);
 		ret[1] = 128 + (src % 64);
 		return 2;
 	}
-	//Unicode 0800-FFFF -> UTF8 1110xxxx 10xxxxxx 10xxxxxx
+	/* Unicode 0800-FFFF -> UTF8 1110xxxx 10xxxxxx 10xxxxxx */
 	if (src >2047) {
 		ret[0] = 224 + (src / 4096);
 		ret[1] = 128 + ((src / 64) % 64);
@@ -1459,7 +1477,7 @@ int EncodeWithUTF8Alphabet2(unsigned char mychar1, unsigned char mychar2, unsign
 		return 3;
 	}
 
-	//Unicode 0000-007F -> UTF8 0xxxxxxx
+	/* Unicode 0000-007F -> UTF8 0xxxxxxx */
 	ret[0] = mychar2;
 	return 1;
 }
@@ -1474,7 +1492,7 @@ bool EncodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src)
 	len = UnicodeLength(src);
 
 	for (i = 0; i < len; i++) {
-		z = EncodeWithUTF8Alphabet2(src[i*2],src[i*2+1],mychar);
+		z = EncodeWithUTF8Alphabet(src[i*2],src[i*2+1],mychar);
 		if (z>1) {
 			for (w=0;w<z;w++) {
 				sprintf(dest+j, "=%02X",mychar[w]);
@@ -1502,7 +1520,7 @@ bool EncodeUTF8(unsigned char *dest, const unsigned char *src)
 	bool		retval = false;
 
 	for (i = 0; i < (int)(UnicodeLength(src)); i++) {
-		z = EncodeWithUTF8Alphabet2(src[i*2],src[i*2+1],mychar);
+		z = EncodeWithUTF8Alphabet(src[i*2],src[i*2+1],mychar);
 		if (z>1) {
 			memcpy(dest+j,mychar,z);
 			j+=z;
@@ -1516,7 +1534,7 @@ bool EncodeUTF8(unsigned char *dest, const unsigned char *src)
 }
 
 /* Decode UTF8 char to Unicode char */
-int DecodeWithUTF8Alphabet2(unsigned char *src, wchar_t *dest, int len)
+int DecodeWithUTF8Alphabet(const unsigned char *src, wchar_t *dest, int len)
 {
 	if (len < 1) return 0;
 	if (src[0] < 128) {
@@ -1576,16 +1594,16 @@ void DecodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src, in
 				break;
 			}
 			mychar[z] = 16*DecodeWithHexBinAlphabet(src[z*3+i+1])+DecodeWithHexBinAlphabet(src[z*3+i+2]);
-			/* Is it plain ASCII? */
-			if (z==0 && mychar[0]<194) break;
 			z++;
+			/* Is it plain ASCII? */
+			if (z == 1 && mychar[0] < 194) break;
 			/* Do we already have valid UTF-8 char? */
-			if (DecodeWithUTF8Alphabet2(mychar,&ret,z) == z) break;
+			if (DecodeWithUTF8Alphabet(mychar, &ret, z) == z) break;
 		}
 		if (z>0) {
-			i+=z*3;
-			// we ignore wrong sequence
-			if (DecodeWithUTF8Alphabet2(mychar,&ret,z)==0) continue;
+			i += z * 3;
+			/*  we ignore wrong sequence */
+			if (DecodeWithUTF8Alphabet(mychar,&ret,z)==0) continue;
 		} else {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
 		}
@@ -1602,7 +1620,7 @@ void DecodeUTF8(unsigned char *dest, const unsigned char *src, int len)
 	wchar_t		ret;
 
 	while (i < len) {
-		z = DecodeWithUTF8Alphabet2((unsigned char *)src+i,&ret,len-i);
+		z = DecodeWithUTF8Alphabet(src+i,&ret,len-i);
 		if (z<2) {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
 		} else {
@@ -1668,7 +1686,7 @@ void DecodeXMLUTF8(unsigned char *dest, const unsigned char *src, int len)
 			}
 			dbgprintf("Unicode char 0x%04lx\n", c);
 			tmplen = strlen(tmp);
-			tmplen += EncodeWithUTF8Alphabet2((c >> 8) & 0xff, c & 0xff, tmp + tmplen);
+			tmplen += EncodeWithUTF8Alphabet((c >> 8) & 0xff, c & 0xff, tmp + tmplen);
 			tmp[tmplen] = 0;
 		} else if (strcmp(entity, "amp") == 0) {
 			strcat(tmp, "&");
@@ -1748,7 +1766,7 @@ OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
-static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], int len)
+static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], size_t len)
 {
 	/* BASE64 translation Table as described in RFC1113 */
 	unsigned char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -1759,10 +1777,10 @@ static void EncodeBASE64Block(unsigned char in[3], unsigned char out[4], int len
 	out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
 }
 
-void EncodeBASE64(const unsigned char *Input, unsigned char *Output, int Length)
+void EncodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Length)
 {
 	unsigned char 	in[3], out[4];
-	int 		i, pos = 0, len, outpos = 0;
+	size_t 		i, pos = 0, len, outpos = 0;
 
 	while (pos < Length) {
 		len = 0;
@@ -1790,11 +1808,11 @@ static void DecodeBASE64Block(unsigned char in[4], unsigned char out[3])
 	out[2] = (unsigned char) (((in[2] << 6) & 0xc0) | in[3]);
 }
 
-int DecodeBASE64(const unsigned char *Input, unsigned char *Output, int Length)
+int DecodeBASE64(const unsigned char *Input, unsigned char *Output, size_t Length)
 {
 	unsigned char 	cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";
 	unsigned char 	in[4], out[3], v;
-	int 		i, len, pos = 0, outpos = 0;
+	size_t 		i, len, pos = 0, outpos = 0;
 
 	while (pos < Length) {
 		len = 0;
@@ -1822,6 +1840,60 @@ int DecodeBASE64(const unsigned char *Input, unsigned char *Output, int Length)
 	return outpos;
 }
 
+#ifdef ICONV_FOUND
+
+#include <iconv.h>
+
+#ifdef ICONV_SECOND_ARGUMENT_IS_CONST
+#  define SECOND_ICONV_ARG const char *
+#else
+#  define SECOND_ICONV_ARG char *
+#endif
+
+bool IconvDecode(const char *charset, const char *input, const size_t inlen, unsigned char *output, size_t outlen)
+{
+	iconv_t ic;
+	/* Add one to convert also trailing zero, this is broken for
+	 * multibyte input, but we don't use iconv for this so far */
+	size_t rest = inlen + 1;
+	SECOND_ICONV_ARG in;
+	char *out;
+
+	ic = iconv_open("UCS-2BE", charset);
+	if (ic == (iconv_t)(-1)) return false;
+
+	/* I know I loose const here, but it's iconv choice... */
+	in = (SECOND_ICONV_ARG)input;
+	out = output;
+	iconv(ic, &in, &rest, &out, &outlen);
+
+	iconv_close(ic);
+
+	return (rest == 0);
+}
+
+bool IconvEncode(const char *charset, const unsigned char *input, const size_t inlen, char *output, size_t outlen)
+{
+	iconv_t ic;
+	size_t rest = inlen;
+	SECOND_ICONV_ARG in;
+	char *out;
+
+	ic = iconv_open(charset, "UCS-2BE");
+	if (ic == (iconv_t)(-1)) return false;
+
+	/* I know I loose const here, but it's iconv choice... */
+	in = (SECOND_ICONV_ARG)input;
+	out = output;
+	iconv(ic, &in, &rest, &out, &outlen);
+
+	iconv_close(ic);
+
+	return (rest == 0);
+}
+#endif
+
+
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
