@@ -1,8 +1,10 @@
 /* (c) 2002-2005 by Marcin Wiacek and Michal Cihar */
 /* Checking used compiler (c) 2002 by Michal Cihar */
 
-#include "../gsmstate.h"
+#include <gammu-config.h>
+#include <gammu-misc.h>
 #include "misc.h"
+#include "coding/coding.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -10,9 +12,11 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <locale.h>
+#include <stdlib.h>
 #include <sys/timeb.h>
 #ifdef WIN32
-#  include "windows.h"
+#  define WIN32_LEAN_AND_MEAN
+#  include <windows.h>
 #endif
 #ifdef HAVE_SYS_UTSNAME_H
 #  include <sys/utsname.h>
@@ -29,6 +33,17 @@ int RecalcDateTime(struct tm *st, const int year, const int month, const int day
 {
 	const int days[] = {31,28,31,30,31,30,31,31,30,31,30,31};
 	int i, p, q, r;
+	GSM_DateTime Date;
+
+	Date.Year	= year;
+	Date.Month	= month;
+	Date.Day	= day;
+	Date.Hour	= hour;
+	Date.Minute	= minute;
+	Date.Second	= second;
+	Date.Timezone	= 0;
+
+	if (!CheckDate(&Date) || !CheckTime(&Date)) return 0;
 
 	memset(st, 0, sizeof(*st));
 
@@ -51,6 +66,8 @@ int RecalcDateTime(struct tm *st, const int year, const int month, const int day
 	st->tm_mon = month - 1;
 	st->tm_mday = day;
 
+	st->tm_isdst = -1; /* FIXME */
+
 	return 1;
 }
 
@@ -67,7 +84,7 @@ int RecalcDate(struct tm *st, const int year, const int month, const int day)
 /**
  * Return day of year index.
  */
-int GetDayOfYear(int year, int month, int day)
+int GetDayOfYear(unsigned int year, unsigned int month, unsigned int day)
 {
 	struct tm st;
 
@@ -79,7 +96,7 @@ int GetDayOfYear(int year, int month, int day)
 /**
  * Return day of week index.
  */
-int GetWeekOfMonth(int year, int month, int day)
+int GetWeekOfMonth(unsigned int year, unsigned int month, unsigned int day)
 {
 	struct tm st;
 
@@ -91,7 +108,7 @@ int GetWeekOfMonth(int year, int month, int day)
 /**
  * Return day of week index.
  */
-int GetDayOfWeek(int year, int month, int day)
+int GetDayOfWeek(unsigned int year, unsigned int month, unsigned int day)
 {
 	struct tm st;
 
@@ -103,7 +120,7 @@ int GetDayOfWeek(int year, int month, int day)
 /**
  * Return textual representation of day of week;
  */
-char *DayOfWeek (int year, int month, int day)
+char *DayOfWeek (unsigned int year, unsigned int month, unsigned int day)
 {
 	static char 	DayOfWeekChar[10];
 
@@ -120,6 +137,15 @@ char *DayOfWeek (int year, int month, int day)
 	return DayOfWeekChar;
 }
 
+int GSM_GetLocalTimezoneOffset() {
+	struct tm *tg, *tl;
+	time_t now = time(NULL);
+	tg = gmtime(&now);
+	tl = localtime(&now);
+	// Returns offset including daylight saving (found as boolean in tl.tm_isdst).
+	return (tl->tm_hour - tg->tm_hour) * 3600 + (tl->tm_min - tg->tm_min) * 60 + (tl->tm_sec - tg->tm_sec);
+}
+
 void Fill_GSM_DateTime(GSM_DateTime *Date, time_t timet)
 {
 	struct tm *now;
@@ -131,7 +157,7 @@ void Fill_GSM_DateTime(GSM_DateTime *Date, time_t timet)
 	Date->Hour	= now->tm_hour;
 	Date->Minute	= now->tm_min;
 	Date->Second	= now->tm_sec;
-	Date->Timezone	= 0; /* FIXME: can we get it from the OS? */
+	Date->Timezone	= GSM_GetLocalTimezoneOffset();
 }
 
 void GSM_GetCurrentDateTime (GSM_DateTime *Date)
@@ -210,19 +236,21 @@ char *OSDateTime (GSM_DateTime dt, bool TimeZone)
 	struct tm 	timeptr;
 	static char 	retval[200],retval2[200];
 
+	if (!RecalcDateTime(&timeptr, dt.Year, dt.Month, dt.Day,
+				dt.Hour, dt.Minute, dt.Second)) {
+		retval2[0] = 0;
+		return retval2;
+	}
+
 #ifdef WIN32
 	setlocale(LC_ALL, ".OCP");
 #endif
 
-	RecalcDateTime(&timeptr, dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
-
+	/* This is not Y2K safe */
 	strftime(retval2, 200, "%c", &timeptr);
 	if (TimeZone) {
-		if (dt.Timezone >= 0) {
-			sprintf(retval," +%02i00",dt.Timezone);
-		} else {
-			sprintf(retval," -%02i00",dt.Timezone);
-		}
+		snprintf(retval, sizeof(retval) - 1, " %+03i%02i",
+			dt.Timezone / 3600, abs((dt.Timezone % 3600) / 60));
 		strcat(retval2,retval);
 	}
 	/* If don't have weekday name, include it */
@@ -266,6 +294,7 @@ char *OSDate (GSM_DateTime dt)
 	timeptr.tm_zone		= NULL;
 #endif
 
+	/* This is not Y2K safe */
 	strftime(retval2, 200, "%x", &timeptr);
 
 	/* If don't have weekday name, include it */
@@ -303,9 +332,9 @@ bool CheckDate(GSM_DateTime *date)
 
 bool CheckTime(GSM_DateTime *date)
 {
-	return date->Hour <= 23 && date->Hour >= 0 &&
-		date->Minute <= 59 && date->Minute >= 0 &&
-		date->Second <= 59 && date->Second >= 0;
+	return date->Hour <= 23 &&
+		date->Minute <= 59 &&
+		date->Second <= 59;
 }
 
 int GetLine(FILE *File, char *Line, int count)
@@ -314,9 +343,8 @@ int GetLine(FILE *File, char *Line, int count)
 
 	if (fgets(Line, count, File) != NULL) {
 		num = strlen(Line) - 1;
-		while(1) {
+		while (num > 0) {
 			if (Line[num] != '\n' && Line[num] != '\r') break;
-			if (num == 0) break;
 			Line[num--] = 0;
 		}
 		return strlen(Line);
@@ -324,16 +352,39 @@ int GetLine(FILE *File, char *Line, int count)
 	return -1;
 }
 
-void SplitLines(unsigned char *message, int messagesize, GSM_Lines *lines, unsigned char *whitespaces, int spaceslen, bool eot)
+void InitLines(GSM_CutLines *lines)
+{
+	lines->numbers = NULL;
+	lines->allocated = 0;
+}
+
+void FreeLines(GSM_CutLines *lines)
+{
+	free(lines->numbers);
+	lines->numbers = NULL;
+	lines->allocated = 0;
+}
+
+void SplitLines(const char *message, int messagesize, GSM_CutLines *lines, unsigned char *whitespaces, int spaceslen, bool eot)
 {
 	int 	i,number=0,j;
 	bool 	whitespace=true, nowwhite;
 
-	for (i=0;i<MAX_LINES*2;i++) lines->numbers[i]=0;
+	for (i = 0; i < lines->allocated; i++)
+		lines->numbers[i]=0;
+
+	for (i = 0; i < messagesize; i++) {
+		/* Reallocate buffer if needed */
+		if (number + 1 >= lines->allocated - 1) {
+			lines->allocated += 20;
+			lines->numbers = realloc(lines->numbers, lines->allocated * sizeof(int));
+			if (lines->numbers == NULL) return;
+			for (j = lines->allocated - 20; j < lines->allocated; j++)
+				lines->numbers[j] = 0;
+		}
 
-	for (i=0;i<messagesize;i++) {
 		nowwhite = false;
-		for (j=0;j<spaceslen;j++) {
+		for (j = 0; j < spaceslen; j++) {
 			if (whitespaces[j] == message[i]) {
 				nowwhite = true;
 				break;
@@ -354,74 +405,46 @@ void SplitLines(unsigned char *message, int messagesize, GSM_Lines *lines, unsig
 
 		}
 	}
-    	if (eot && !whitespace) lines->numbers[number]=messagesize;
+    	if (eot && !whitespace)
+		lines->numbers[number] = messagesize;
 }
 
-char *GetLineString(unsigned char *message, GSM_Lines lines, int start)
+char *GetLineString(const char *message, const GSM_CutLines *lines, int start)
 {
-	static char retval[800];
+	static char *retval = NULL;
+	int len;
 
-	memcpy(retval,message + lines.numbers[start*2-2],lines.numbers[start*2-2+1]-lines.numbers[start*2-2]);
-	retval[lines.numbers[start*2-2+1]-lines.numbers[start*2-2]]=0;
+	len = GetLineLength(message, lines, start);
+	retval = realloc(retval, len + 1);
+	if (retval == NULL) {
+		dbgprintf("Allocation failed!\n");
+		return NULL;
+	}
+
+	memcpy(retval, message + lines->numbers[start * 2 - 2], len);
+
+	retval[len] = 0;
 
 	return retval;
 }
 
-int GetLineLength(unsigned char *message, GSM_Lines lines, int start)
+int GetLineLength(const char *message UNUSED, const GSM_CutLines *lines, int start)
 {
-	return lines.numbers[start*2-2+1]-lines.numbers[start*2-2];
+	return lines->numbers[start*2-2+1] - lines->numbers[start*2-2];
 }
 
-void CopyLineString(unsigned char *dest, unsigned char *src, GSM_Lines lines, int start)
+void CopyLineString(char *dest, const char *src, const GSM_CutLines *lines, int start)
 {
-	memcpy(dest,GetLineString(src, lines, start),strlen(GetLineString(src, lines, start)));
-	dest[strlen(GetLineString(src, lines, start))] = 0;
+	int len = GetLineLength(src, lines, start);
+	memcpy(dest, GetLineString(src, lines, start), len);
+	dest[len] = 0;
 }
 
-Debug_Info di = {0,NULL,false,"",false};
+GSM_Debug_Info di = {0,NULL,false,"",false};
 
-#ifdef DEBUG
-#if defined(__GNUC__) && !defined(printf)
-__attribute__((format(printf, 1, 2)))
-#endif
-int dbgprintf(const char *format, ...)
+PRINTF_STYLE(2, 0)
+static int dbg_vprintf(GSM_Debug_Info *d, const char *format, va_list argp)
 {
-	va_list			argp;
-	int 			result;
-	static unsigned char 	nextline[2000]="";
-	unsigned char		buffer[2000];
-	GSM_DateTime 		date_time;
-
-	if (di.df != NULL && (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE)) {
-		va_start(argp, format);
-		result = vsprintf(buffer, format, argp);
-		strcat(nextline, buffer);
-		if (strstr(buffer, "\n")) {
-			if (di.dl == DL_TEXTALLDATE) {
-				GSM_GetCurrentDateTime(&date_time);
-				fprintf(di.df,"%s %4d/%02d/%02d %02d:%02d:%02d: %s",
-		                	DayOfWeek(date_time.Year, date_time.Month, date_time.Day),
-		                	date_time.Year, date_time.Month, date_time.Day,
-		                	date_time.Hour, date_time.Minute, date_time.Second,nextline);
-			} else {
-				fprintf(di.df,"%s",nextline);
-			}
-			strcpy(nextline, "");
-		}
-		fflush(di.df);
-		va_end(argp);
-		return result;
-	}
-	return 0;
-}
-#endif
-
-#if defined(__GNUC__) && !defined(printf)
-__attribute__((format(printf, 2, 3)))
-#endif
-int smfprintf(Debug_Info *d, const char *format, ...)
-{
-        va_list 		argp;
 	int 			result=0;
 	char			buffer[3000];
 	char			*pos, *end;
@@ -430,18 +453,12 @@ int smfprintf(Debug_Info *d, const char *format, ...)
 	FILE			*f;
 	Debug_Level		l;
 
-	if (d->use_global) {
-		f = di.df;
-		l = di.dl;
-	} else {
-		f = d->df;
-		l = d->dl;
-	}
+	f = d->df;
+	l = d->dl;
 
 	if (l == DL_NONE || f == NULL) return 0;
 
-	va_start(argp, format);
-	result = vsprintf(buffer, format, argp);
+	result = vsnprintf(buffer, sizeof(buffer) - 1, format, argp);
 	pos = buffer;
 
 	while (*pos != 0) {
@@ -488,108 +505,144 @@ int smfprintf(Debug_Info *d, const char *format, ...)
 	}
 
 	/* Flush buffers, this might be configurable, but it could cause drop of last log messages */
-	fflush(d->df);
+	fflush(f);
 
-	va_end(argp);
 	return result;
 }
 
-bool GSM_SetDebugLevel(char *info, Debug_Info *di)
+#ifdef DEBUG
+PRINTF_STYLE(1, 2)
+int dbgprintf(const char *format, ...)
 {
-	if (info == NULL)			{di->dl = DL_NONE;	 	return true;}
-	if (!strcmp(info,"nothing")) 		{di->dl = DL_NONE;	 	return true;}
-	if (!strcmp(info,"text")) 		{di->dl = DL_TEXT;	 	return true;}
-	if (!strcmp(info,"textall")) 		{di->dl = DL_TEXTALL;    	return true;}
-	if (!strcmp(info,"binary"))  		{di->dl = DL_BINARY;     	return true;}
-	if (!strcmp(info,"errors"))  		{di->dl = DL_TEXTERROR;  	return true;}
-	if (!strcmp(info,"textdate")) 		{di->dl = DL_TEXTDATE;	 	return true;}
-	if (!strcmp(info,"textalldate")) 	{di->dl = DL_TEXTALLDATE;    	return true;}
-	if (!strcmp(info,"errorsdate"))  	{di->dl = DL_TEXTERRORDATE;  	return true;}
+        va_list 		argp;
+	int 			result;
+
+	va_start(argp, format);
+	result = dbg_vprintf(&di, format, argp);
+	va_end(argp);
+
+	return result;
+}
+#endif
+
+PRINTF_STYLE(2, 3)
+int smfprintf(GSM_Debug_Info *d, const char *format, ...)
+{
+        va_list 		argp;
+	int 			result;
+	GSM_Debug_Info		*tmpdi;
+
+	if (d->use_global) {
+		tmpdi = &di;
+	} else {
+		tmpdi = d;
+	}
+
+	va_start(argp, format);
+	result = dbg_vprintf(tmpdi, format, argp);
+	va_end(argp);
+
+	return result;
+}
+
+bool GSM_SetDebugLevel(const char *info, GSM_Debug_Info *privdi)
+{
+	if (info == NULL)			{privdi->dl = DL_NONE;	 	return true;}
+	if (!strcmp(info,"nothing")) 		{privdi->dl = DL_NONE;	 	return true;}
+	if (!strcmp(info,"text")) 		{privdi->dl = DL_TEXT;	 	return true;}
+	if (!strcmp(info,"textall")) 		{privdi->dl = DL_TEXTALL;    	return true;}
+	if (!strcmp(info,"binary"))  		{privdi->dl = DL_BINARY;     	return true;}
+	if (!strcmp(info,"errors"))  		{privdi->dl = DL_TEXTERROR;  	return true;}
+	if (!strcmp(info,"textdate")) 		{privdi->dl = DL_TEXTDATE;	 	return true;}
+	if (!strcmp(info,"textalldate")) 	{privdi->dl = DL_TEXTALLDATE;    	return true;}
+	if (!strcmp(info,"errorsdate"))  	{privdi->dl = DL_TEXTERRORDATE;  	return true;}
 	return false;
 }
 
-/* Dumps a message */
-void DumpMessage(Debug_Info *d, const unsigned char *message, int messagesize)
+bool GSM_SetDebugCoding(const char *info, GSM_Debug_Info *privdi)
 {
-	int 		i,j=0,len=16;
-	unsigned char	buffer[200];
+	privdi->coding = info;
+	return true;
+}
+
+bool GSM_SetDebugGlobal(bool info, GSM_Debug_Info *privdi)
+{
+	if (privdi->use_global && info) {
+		GSM_SetDebugFileDescriptor(NULL, privdi);
+	}
+	privdi->use_global = info;
+	return true;
+}
+
+#define CHARS_PER_LINE (16)
+
+/* Dumps a message */
+void DumpMessage(GSM_Debug_Info *d, const unsigned char *message, const int messagesize)
+{
+	int i, j = 0;
+	char buffer[(CHARS_PER_LINE * 5) + 1];
 
 	if (d->df == NULL || messagesize == 0) return;
 
 	smfprintf(d, "\n");
 
-	memset(buffer,0x20,sizeof(buffer));
-	buffer[len*5-1]=0;
+	memset(buffer, ' ', CHARS_PER_LINE * 5);
+	buffer[CHARS_PER_LINE * 5] = 0;
 
 	for (i = 0; i < messagesize; i++) {
-		sprintf(buffer+j*4,"%02X",message[i]);
-		buffer[j*4+2] = 0x20;
-		// 9 = tab
-		if (isprint(message[i]) && message[i]!=0x09
-			// 0x01 = beep in windows xp
-			&& message[i]!=0x01
-			// these are empty in windows xp
-			&& message[i]!=0x85 && message[i]!=0x95
-  			&& message[i]!=0xA6 && message[i]!=0xB7) {
-			if (j != len-1) buffer[j*4+2] 	= message[i];
-			buffer[(len-1)*4+j+3]		= message[i];
+		/* Write hex number */
+		snprintf(buffer + (j * 4), 3, "%02X", message[i]);
+		buffer[(j * 4) + 2] = ' '; /* wipe snprintf's \0 */
+
+		/* Write char if possible */
+		if (isprint(message[i])
+				/* 0x09 = tab */
+				&& message[i] != 0x09
+				/* 0x01 = beep in windows xp */
+				&& message[i] != 0x01
+				/* these are empty in windows xp */
+				&& message[i] != 0x85
+				&& message[i] != 0x95
+				&& message[i] != 0xA6
+				&& message[i] != 0xB7) {
+			buffer[(j * 4) + 2] = message[i];
+			buffer[(CHARS_PER_LINE - 1) * 4 + j + 4] = message[i];
 		} else {
-			buffer[(len-1)*4+j+3]		= '.';
+			buffer[(CHARS_PER_LINE - 1) * 4 + j + 4] = '.';
 		}
-		if (j != len-1 && i != messagesize-1) buffer[j*4+3] = '|';
-		if (j == len-1) {
+
+		/* Write char separator */
+		if (j != CHARS_PER_LINE - 1 && i != messagesize - 1) {
+			buffer[j * 4 + 3] = '|';
+		}
+
+		/* Print out buffer */
+		if (j == CHARS_PER_LINE - 1) {
 			smfprintf(d, "%s\n", buffer);
-			memset(buffer,0x20,sizeof(buffer));
-			buffer[len*5-1]=0;
+			memset(buffer, ' ', CHARS_PER_LINE * 5);
 			j = 0;
 		} else {
 			j++;
 		}
 	}
-	if (j != 0) smfprintf(d, "%s\n", buffer);
+
+	/* Anything remains to be printed? */
+	if (j != 0) {
+		smfprintf(d, "%s\n", buffer);
+	}
 }
 
-#ifdef WIN32
-#include <malloc.h>
+#undef CHARS_PER_LINE
 
-bool HeapCheck(char* loc) {
-   /* Check heap status */
-   int heapstatus = _heapchk();
-   switch( heapstatus )
-   {
-   case _HEAPOK:
-		dbgprintf("%s: OK - heap is fine\n", loc );
-		return false;
-		break;
-   case _HEAPEMPTY:
-	   dbgprintf("%s: OK - heap is empty\n", loc );
-		return false;
-		break;
-   case _HEAPBADBEGIN:
-	   dbgprintf( "%s: ERROR - bad start of heap\n", loc );
-		return true;
-		break;
-   case _HEAPBADNODE:
-	   dbgprintf( "%s: ERROR - bad node in heap\n", loc );
-		return true;
-		break;
-   }
-   return false;
-}
-#else
-bool HeapCheck(char* loc) {
-	return false;
-}
-#endif
-
-char *GetOS(void)
+const char *GetOS(void)
 {
 #ifdef WIN32
 	OSVERSIONINFOEX Ver;
 	bool		Extended = true;
-#endif
-#ifdef HAVE_SYS_UTSNAME_H
+#else
+#  ifdef HAVE_SYS_UTSNAME_H
 	struct utsname	Ver;
+#  endif
 #endif
 	static char 	Buffer[100] = {0x00};
 
@@ -601,14 +654,8 @@ char *GetOS(void)
 		Extended 		= false;
 	      	Ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
 	        if (!GetVersionEx((OSVERSIONINFO *)&Ver)) {
-//#ifdef _MSC_VER
-//			Ver.dwMajorVersion = _winmajor;
-//			Ver.dwMinorVersion = _winminor;
-//			Ver.dwBuildNumber  = _osver;
-//#else
-			sprintf(Buffer, "Windows");
+			snprintf(Buffer, sizeof(Buffer) - 1, "Windows");
 			return Buffer;
-//#endif
 		}
 	}
 
@@ -616,129 +663,149 @@ char *GetOS(void)
 
 	/* no info about Win95 SP1, Win95 OSR2.1, Win95 OSR2.5.... */
 	if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 950) {
-		sprintf(Buffer,"Windows 95");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 95");
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 1111) {
-		sprintf(Buffer,"Windows 95 OSR2.x");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 95 OSR2.x");
 
 	/* no info about Win98 SP1.... */
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 10 && Ver.dwBuildNumber == 1998) {
-		sprintf(Buffer,"Windows 98");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 98");
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 10 && Ver.dwBuildNumber == 2222) {
-		sprintf(Buffer,"Windows 98 SE");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 98 SE");
 
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 90 && Ver.dwBuildNumber == 3000) {
-		sprintf(Buffer,"Windows ME");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows ME");
 
 	/* ---------------- NT family ------------------- */
 
 	} else if (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 1381) {
-		sprintf(Buffer,"Windows NT 4.0");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows NT 4.0");
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 0 && Ver.dwBuildNumber == 2195) {
-		sprintf(Buffer,"Windows 2000");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 2000");
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 1 && Ver.dwBuildNumber == 2600) {
-		sprintf(Buffer,"Windows XP");
-#if _MSC_VER > 1200 //6.0 has it undeclared
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows XP");
+#if _MSC_VER > 1200 /* 6.0 has it undeclared */
 		if (Extended) {
 			if (Ver.wSuiteMask & VER_SUITE_PERSONAL) {
-				sprintf(Buffer+strlen(Buffer)," Home");
+				snprintf(Buffer+strlen(Buffer), sizeof(Buffer) - 1 - strlen(Buffer)," Home");
 			} else {
-				sprintf(Buffer+strlen(Buffer)," Pro");
+				snprintf(Buffer+strlen(Buffer), sizeof(Buffer) - 1 - strlen(Buffer)," Pro");
 			}
 		}
 #endif
 
 	} else if (Ver.dwMajorVersion == 5 && Ver.dwMinorVersion == 2) {
-		sprintf(Buffer,"Windows 2003");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows 2003");
 
 	} else if (Ver.dwMajorVersion == 6 && Ver.dwMinorVersion == 0) {
-		sprintf(Buffer,"Windows Vista");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows Vista");
 
 	} else if (Ver.dwMajorVersion == 6 && Ver.dwMinorVersion > 0) {
-		sprintf(Buffer,"Windows Server 2007");
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows Server 2007");
 
 	} else {
-		sprintf(Buffer, "Windows %i.%i.%i",(int)Ver.dwMajorVersion,(int)Ver.dwMinorVersion,(int)Ver.dwBuildNumber);
+		snprintf(Buffer, sizeof(Buffer) - 1, "Windows %i.%i.%i",(int)Ver.dwMajorVersion,(int)Ver.dwMinorVersion,(int)Ver.dwBuildNumber);
 	}
 
 	if (Extended && Ver.wServicePackMajor != 0) {
-		sprintf(Buffer+strlen(Buffer)," SP%i",Ver.wServicePackMajor);
+		snprintf(Buffer+strlen(Buffer), sizeof(Buffer) - 1 - strlen(Buffer)," SP%i",Ver.wServicePackMajor);
 	}
 #elif defined(HAVE_SYS_UTSNAME_H)
 	uname(&Ver);
-	sprintf(Buffer, "%s, kernel %s (%s)", Ver.sysname, Ver.release, Ver.version);
+	snprintf(Buffer, sizeof(Buffer) - 1, "%s, kernel %s (%s)", Ver.sysname, Ver.release, Ver.version);
 #elif defined(__FreeBSD__)
-	sprintf(Buffer, "FreeBSD");
+	snprintf(Buffer, sizeof(Buffer) - 1, "FreeBSD");
 #elif defined(__NetBSD__)
-	sprintf(Buffer, "NetBSD");
+	snprintf(Buffer, sizeof(Buffer) - 1, "NetBSD");
 #elif defined(__OpenBSD__)
-	sprintf(Buffer, "OpenBSD");
+	snprintf(Buffer, sizeof(Buffer) - 1, "OpenBSD");
 #elif defined(__GNU__)
-	sprintf(Buffer, "GNU/Hurd");
+	snprintf(Buffer, sizeof(Buffer) - 1, "GNU/Hurd");
 #elif defined(sun) || defined(__sun) || defined(__sun__)
 #  ifdef __SVR4
-	sprintf(Buffer, "Sun Solaris");
+	snprintf(Buffer, sizeof(Buffer) - 1, "Sun Solaris");
 #  else
-	sprintf(Buffer, "SunOS");
+	snprintf(Buffer, sizeof(Buffer) - 1, "SunOS");
 #  endif
 #elif defined(hpux) || defined(__hpux) || defined(__hpux__)
-	sprintf(Buffer, "HP-UX");
+	snprintf(Buffer, sizeof(Buffer) - 1, "HP-UX");
 #elif defined(ultrix) || defined(__ultrix) || defined(__ultrix__)
-	sprintf(Buffer, "DEC Ultrix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "DEC Ultrix");
 #elif defined(sgi) || defined(__sgi)
-	sprintf(Buffer, "SGI Irix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "SGI Irix");
 #elif defined(__osf__)
-	sprintf(Buffer, "OSF Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "OSF Unix");
 #elif defined(bsdi) || defined(__bsdi__)
-	sprintf(Buffer, "BSDI Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "BSDI Unix");
 #elif defined(_AIX)
-	sprintf(Buffer, "AIX Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "AIX Unix");
 #elif defined(_UNIXWARE)
-	sprintf(Buffer, "SCO Unixware");
+	snprintf(Buffer, sizeof(Buffer) - 1, "SCO Unixware");
 #elif defined(DGUX)
-	sprintf(Buffer, "DG Unix");
+	snprintf(Buffer, sizeof(Buffer) - 1, "DG Unix");
 #elif defined(__QNX__)
-	sprintf(Buffer, "QNX");
+	snprintf(Buffer, sizeof(Buffer) - 1, "QNX");
 #endif
 	return Buffer;
 }
 
-char *GetCompiler(void)
+const char *GetCompiler(void)
 {
 	static char Buffer[100] = {0x00};
 
 #ifdef WIN32
 #  ifdef _MSC_VER
-	if (_MSC_VER == 1200) { //?
-		sprintf(Buffer, "MS VC 6.0");
+	if (_MSC_VER == 1200) { /* ? */
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC 6.0");
 	} else if (_MSC_VER == 1300) {
-		sprintf(Buffer, "MS VC .NET 2002");
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC .NET 2002");
 	} else if (_MSC_VER == 1310) {
-		sprintf(Buffer, "MS VC .NET 2003");
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC .NET 2003");
 	} else if (_MSC_VER == 1400) {
-		sprintf(Buffer, "MS VC .NET 2005");
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC .NET 2005");
 	} else {
-		sprintf(Buffer, "MS VC %i",_MSC_VER);
+		snprintf(Buffer, sizeof(Buffer) - 1, "MS VC %i",_MSC_VER);
 	}
 #  elif defined(__BORLANDC__)
-	sprintf(Buffer, "Borland C++ %i",__BORLANDC__);
+	snprintf(Buffer, sizeof(Buffer) - 1, "Borland C++ %i",__BORLANDC__);
+#  elif defined(__MINGW32__)
+	snprintf(Buffer, sizeof(Buffer) - 1, "GCC %i.%i, MinGW %i.%i", __GNUC__, __GNUC_MINOR__, __MINGW32_MAJOR_VERSION, __MINGW32_MINOR_VERSION);
+#  elif defined(__GNUC__)
+	snprintf(Buffer, sizeof(Buffer) - 1, "GCC %i.%i", __GNUC__, __GNUC_MINOR__);
+#  else
+	snprintf(Buffer, sizeof(Buffer) - 1, "unknown compiler");
 #  endif
 #elif defined(DJGPP)
-	sprintf(Buffer, "djgpp");
+	snprintf(Buffer, sizeof(Buffer) - 1, "djgpp %d.%d", __DJGPP, __DJGPP_MINOR);
 #elif defined(__GNUC__)
-	sprintf(Buffer, "gcc %i.%i", __GNUC__, __GNUC_MINOR__);
+	snprintf(Buffer, sizeof(Buffer) - 1, "GCC %i.%i", __GNUC__, __GNUC_MINOR__);
 #elif defined(__SUNPRO_CC)
-	sprintf(Buffer, "Sun C++ %x", __SUNPRO_CC);
+	snprintf(Buffer, sizeof(Buffer) - 1, "Sun C++ %x", __SUNPRO_CC);
 #elif defined(__INTEL_COMPILER)
-	sprintf(Buffer, "Intel Compiler %ld", __INTEL_COMPILER);
+	snprintf(Buffer, sizeof(Buffer) - 1, "Intel Compiler %ld", __INTEL_COMPILER);
 #else
-	sprintf(Buffer, "unknown compiler");
+	snprintf(Buffer, sizeof(Buffer) - 1, "unknown compiler");
 #endif
 
 	return Buffer;
 }
 
+bool GSM_IsNewerVersion(const char *latest_version, const char *current_version)
+{
+	size_t i;
+	size_t len = strlen(latest_version);
+
+	for (i = 0; i < len ; i++) {
+		if (latest_version[i] > current_version[i]) {
+			return true;
+		}
+	}
+
+	return false;
+}
+
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
