@@ -13,13 +13,15 @@
  * to publish their code under.
  */
 
-#include "../../../gsmstate.h"
+#include <gammu-config.h>
 
 #ifdef GSM_ENABLE_NOKIA7110
 
 #include <string.h>
 #include <time.h>
 
+#include <gammu-nokia.h>
+
 #include "../../../misc/coding/coding.h"
 #include "../../../gsmcomon.h"
 #include "../../../service/gsmlogo.h"
@@ -29,14 +31,14 @@
 #include "n7110.h"
 #include "dct3func.h"
 
-static GSM_Error N7110_GetAlarm(GSM_StateMachine *s, GSM_Alarm *alarm)
+static GSM_Error N7110_GetAlarm(GSM_StateMachine *s, GSM_Alarm *Alarm)
 {
-	return DCT3_GetAlarm(s, alarm, 0x19);
+	return DCT3_GetAlarm(s, Alarm, 0x19);
 }
 
-static GSM_Error N7110_SetAlarm(GSM_StateMachine *s, GSM_Alarm *alarm)
+static GSM_Error N7110_SetAlarm(GSM_StateMachine *s, GSM_Alarm *Alarm)
 {
-	return DCT3_SetAlarm(s, alarm, 0x19);
+	return DCT3_SetAlarm(s, Alarm, 0x19);
 }
 
 static GSM_Error N7110_ReplyGetMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
@@ -44,13 +46,11 @@ static GSM_Error N7110_ReplyGetMemory(GSM_Protocol_Message msg, GSM_StateMachine
 	GSM_Phone_Data *Data = &s->Phone.Data;
 
 	smprintf(s, "Phonebook entry received\n");
-	switch (msg.Buffer[6]) {
-	case 0x0f:
+
+	if (msg.Buffer[6] == 0x0f)
 		return N71_65_ReplyGetMemoryError(msg.Buffer[10], s);
-	default:
-		return N71_65_DecodePhonebook(s, Data->Memory,Data->Bitmap,Data->SpeedDial,msg.Buffer+18,msg.Length-18,false);
-	}
-	return ERR_UNKNOWN;
+
+	return N71_65_DecodePhonebook(s, Data->Memory,Data->Bitmap,Data->SpeedDial,msg.Buffer+18,msg.Length-18,false);
 }
 
 static GSM_Error N7110_GetMemory (GSM_StateMachine *s, GSM_MemoryEntry *entry)
@@ -111,9 +111,9 @@ static void N7110_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsig
 
 	/* simulate flat SMS memory */
 	if (sms->Folder==0x00) {
-		ifolderid = sms->Location / PHONE_MAXSMSINFOLDER;
+		ifolderid = sms->Location / GSM_PHONE_MAXSMSINFOLDER;
 		*folderid = (ifolderid + 1) * 0x08;
-		*location = sms->Location - ifolderid * PHONE_MAXSMSINFOLDER;
+		*location = sms->Location - ifolderid * GSM_PHONE_MAXSMSINFOLDER;
 	} else {
 		*folderid = sms->Folder * 0x08;
 		*location = sms->Location;
@@ -125,7 +125,7 @@ static void N7110_GetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsig
 static void N7110_SetSMSLocation(GSM_StateMachine *s, GSM_SMSMessage *sms, unsigned char folderid, int location)
 {
 	sms->Folder	= 0;
-	sms->Location	= (folderid / 0x08 - 1) * PHONE_MAXSMSINFOLDER + location;
+	sms->Location	= (folderid / 0x08 - 1) * GSM_PHONE_MAXSMSINFOLDER + location;
 	smprintf(s, "7110 folder %i & location %i -> SMS folder %i & location %i\n",
 		folderid,location,sms->Folder,sms->Location);
 }
@@ -192,8 +192,8 @@ static GSM_Error N7110_ReplyGetSMSFolderStatus(GSM_Protocol_Message msg, GSM_Sta
 	smprintf(s, "Locations: ");
 	for (i=0;i<Priv->LastSMSFolder.Number;i++) {
 		Priv->LastSMSFolder.Location[i]=msg.Buffer[6+(i*2)]*256+msg.Buffer[(i*2)+7];
-		if (Priv->LastSMSFolder.Location[i] > PHONE_MAXSMSINFOLDER) {
-			smprintf(s, "Increase PHONE_MAXSMSINFOLDER\n");
+		if (Priv->LastSMSFolder.Location[i] > GSM_PHONE_MAXSMSINFOLDER) {
+			smprintf(s, "Increase GSM_PHONE_MAXSMSINFOLDER\n");
 			return ERR_UNKNOWNRESPONSE;
 		}
 		smprintf(s, "%i ",Priv->LastSMSFolder.Location[i]);
@@ -247,7 +247,7 @@ static GSM_SMSMessageLayout N7110_SMSTemplate = {
 static GSM_Error N7110_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
 	int			i;
-	int			Width, Height;
+	size_t			Width, Height;
 	unsigned char		output[500], output2[500];
 	GSM_Phone_Data		*Data = &s->Phone.Data;
 
@@ -415,7 +415,7 @@ static GSM_Error N7110_GetSMSMessage(GSM_StateMachine *s, GSM_MultiSMSMessage *s
 {
 	GSM_Error		error;
 	unsigned char		folderid;
-	int			location;
+	unsigned int		location;
 	GSM_Phone_N7110Data	*Priv = &s->Phone.Data.Priv.N7110;
 	int			i;
 	bool			found = false;
@@ -437,7 +437,7 @@ static GSM_Error N7110_GetNextSMSMessage(GSM_StateMachine *s, GSM_MultiSMSMessag
 {
 	GSM_Phone_N7110Data	*Priv = &s->Phone.Data.Priv.N7110;
 	unsigned char		folderid;
-	int			location;
+	unsigned int		location;
 	GSM_Error		error;
 	int			i;
 	bool			findnextfolder = false;
@@ -487,7 +487,8 @@ static int N7110_ReturnBinaryRingtoneLocation(char *model)
 
 static GSM_Error N7110_ReplyGetRingtone(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	int			tmp,i;
+	int			tmp;
+	size_t i;
 	GSM_Phone_Data		*Data = &s->Phone.Data;
 
 	smprintf(s, "Ringtone received\n");
@@ -574,6 +575,10 @@ static GSM_Error N7110_GetPictureImageLocation(GSM_StateMachine *s, GSM_Bitmap *
 					 0x00,			/* Folder ID */
 					 0x0f, 0x07};
 
+	/* Just to have some defined values */
+	*folder = 0;
+	*location = 0;
+
 	error=N7110_GetSMSFolders (s, &folders);
 	if (error!=ERR_NONE) return error;
 
@@ -637,7 +642,7 @@ static GSM_Error N7110_GetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 		pbk.Location   = Bitmap->Location;
 		smprintf(s, "Getting caller group logo\n");
 		error=N7110_GetMemory(s,&pbk);
-		if (error==ERR_NONE) NOKIA_GetDefaultCallerGroupName(s,Bitmap);
+		if (error==ERR_NONE) NOKIA_GetDefaultCallerGroupName(Bitmap);
 		return error;
 	case GSM_OperatorLogo:
 		smprintf(s, "Getting operator logo\n");
@@ -658,7 +663,7 @@ static GSM_Error N7110_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone,
 	GSM_Ringtone	dest;
 	GSM_Error	error;
 	GSM_NetworkInfo	NetInfo;
-	int		size=200;
+	size_t		size=200;
 	unsigned char	req[1000] = {0x7C, 0x01, 0x00, 0x0D, 0x00,
 				     0x00, 0x00, 0x00, 0x00, 0x00,
 				     0x00};	  	/*Length*/
@@ -679,7 +684,7 @@ static GSM_Error N7110_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone,
 			req[10] = size;
 			error = s->Protocol.Functions->WriteMessage(s, req, size+11, 0x00);
 			if (error!=ERR_NONE) return error;
-			my_sleep(1000);
+			sleep(1);
 			/* We have to make something (not important, what) now */
 			/* no answer from phone*/
 			return DCT3_GetNetworkInfo(s,&NetInfo);
@@ -697,7 +702,7 @@ static GSM_Error N7110_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone,
 	memcpy(req2+37,dest.NokiaBinary.Frame,dest.NokiaBinary.Length);
 	error = s->Protocol.Functions->WriteMessage(s, req2, 37+dest.NokiaBinary.Length, 0x1F);
 	if (error!=ERR_NONE) return error;
-	my_sleep(1000);
+	sleep(1);
 	/* We have to make something (not important, what) now */
 	/* no answer from phone*/
 	return DCT3_GetNetworkInfo(s,&NetInfo);
@@ -836,13 +841,13 @@ static GSM_Error N7110_AddSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 	return N7110_PrivSetSMSMessage(s, sms);
 }
 
-static GSM_Error N7110_ReplyClearOperatorLogo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplyClearOperatorLogo(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Clearing operator logo.....\n");
 	return ERR_NONE;
 }
 
-static GSM_Error N7110_ReplySetOperatorLogo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplySetOperatorLogo(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Setting operator logo.....\n");
 	return ERR_NONE;
@@ -850,7 +855,8 @@ static GSM_Error N7110_ReplySetOperatorLogo(GSM_Protocol_Message msg, GSM_StateM
 
 static GSM_Error N7110_SetCallerLogo(GSM_StateMachine *s, GSM_Bitmap *bitmap)
 {
-	int		block=0, i, Width, Height;
+	int		block=0, i;
+	size_t Width, Height;
 	unsigned int 	count = 18;
 	char		string[500];
 	unsigned char   req[500] = {N6110_FRAME_HEADER, 0x0b, 0x00,
@@ -863,7 +869,7 @@ static GSM_Error N7110_SetCallerLogo(GSM_StateMachine *s, GSM_Bitmap *bitmap)
 
 	/* Enabling/disabling logo */
 	if (bitmap->DefaultBitmap) {
-		string[0] = 0; //disabling
+		string[0] = 0; /* disabling */
 	} else {
 		string[0] = bitmap->BitmapEnabled?1:0;
 	}
@@ -921,7 +927,8 @@ static GSM_Error N7110_SetPictureImage(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 {
 	unsigned char		folder;
 	GSM_Error		error;
-	int			location, i, count, Width, Height;
+	int			location, i, count;
+	size_t Width, Height;
 	GSM_Phone_Bitmap_Types	Type = GSM_NokiaPictureImage;
 	unsigned char 		req[500] = {N6110_FRAME_HEADER, 0x50, 0x07,
 					    0x00,	/* location	*/
@@ -994,7 +1001,7 @@ static GSM_Error N7110_SetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 {
 	GSM_Error		error;
 	GSM_Phone_Bitmap_Types	Type;
-	int			Width, Height, i;
+	size_t			Width, Height, i;
 	unsigned char reqStartup[1000] = {N7110_FRAME_HEADER, 0xec,
 		0x15,			/* Startup Logo setting */
 		0x00, 0x00, 0x00, 0x04, 0xc0, 0x02, 0x00,
@@ -1072,7 +1079,7 @@ static GSM_Error N7110_SetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 	return ERR_NOTSUPPORTED;
 }
 
-static GSM_Error N7110_ReplyDeleteMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplyDeleteMemory(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Phonebook entry deleted\n");
 	return ERR_NONE;
@@ -1098,7 +1105,8 @@ static GSM_Error N7110_DeleteMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 
 static GSM_Error N7110_SetMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 {
-	int 		count = 18, blocks;
+	int 		count = 18;
+	size_t blocks;
 	unsigned char 	req[5000] = {N7110_FRAME_HEADER, 0x0b, 0x00,
 				    0x01, 0x01, 0x00, 0x00, 0x0c,
 				    0x00, 0x00,		/* memory type	*/
@@ -1113,7 +1121,7 @@ static GSM_Error N7110_SetMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 	req[12] = entry->Location >> 8;
 	req[13] = entry->Location & 0xff;
 
-	count   = count + N71_65_EncodePhonebookFrame(s, req+18, entry, &blocks, false, IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_VOICETAGS));
+	count   = count + N71_65_EncodePhonebookFrame(s, req+18, entry, &blocks, false, GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_VOICETAGS));
 	req[17] = blocks;
 
 	smprintf(s, "Writing phonebook entry\n");
@@ -1245,12 +1253,12 @@ static GSM_Error N7110_GetProfile(GSM_StateMachine *s, GSM_Profile *Profile)
 		error = GSM_WaitFor (s, req, 9, 0x39, 4, ID_GetProfile);
 		if (error!=ERR_NONE) return error;
 	}
-	NOKIA_GetDefaultProfileName(s, Profile);
+	NOKIA_GetDefaultProfileName(Profile);
 	Profile->Active = false;
 	return error;
 }
 
-static GSM_Error N7110_ReplySetProfileFeature(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplySetProfileFeature(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Profile feature set\n");
 	return ERR_NONE;
@@ -1372,7 +1380,7 @@ static GSM_Error N7110_ReplyGetCalendarNotePos(GSM_Protocol_Message msg, GSM_Sta
 static GSM_Error N7110_GetNextCalendar(GSM_StateMachine *s,  GSM_CalendarEntry *Note, bool start)
 {
 	return N71_65_GetNextCalendar1(s,Note,start,&s->Phone.Data.Priv.N7110.LastCalendar,&s->Phone.Data.Priv.N7110.LastCalendarYear,&s->Phone.Data.Priv.N7110.LastCalendarPos);
-//	return N71_65_GetNextCalendar2(s,Note,start,&s->Phone.Data.Priv.N7110.LastCalendarYear,&s->Phone.Data.Priv.N7110.LastCalendarPos);
+/* 	return N71_65_GetNextCalendar2(s,Note,start,&s->Phone.Data.Priv.N7110.LastCalendarYear,&s->Phone.Data.Priv.N7110.LastCalendarPos); */
 }
 
 static GSM_Error N7110_GetCalendarStatus(GSM_StateMachine *s, GSM_CalendarStatus *Status)
@@ -1390,16 +1398,16 @@ static GSM_Error N7110_GetCalendarStatus(GSM_StateMachine *s, GSM_CalendarStatus
 	return ERR_NONE;
 
     	/* Method 2 */
-//	return GE_NOTSUPPORTED;
+/* 	return GE_NOTSUPPORTED; */
 }
 
 static GSM_Error N7110_AddCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 {
-//	return N71_65_AddCalendar1(s, Note, NULL);
+/* 	return N71_65_AddCalendar1(s, Note, NULL); */
 	return N71_65_AddCalendar2(s,Note);
 }
 
-static GSM_Error N7110_ReplyGetNetworkInfoError(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error N7110_ReplyGetNetworkInfoError(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Probably means no PIN\n");
 	return ERR_SECURITYERROR;
@@ -1409,24 +1417,27 @@ static GSM_Error N7110_SetIncomingCall(GSM_StateMachine *s, bool enable)
 {
 #ifndef GSM_ENABLE_N71_92INCOMINGINFO
 	return ERR_SOURCENOTAVAILABLE;
-#endif
+#else
 	return NOKIA_SetIncomingCall(s,enable);
+#endif
 }
 
 static GSM_Error N7110_SetIncomingUSSD(GSM_StateMachine *s, bool enable)
 {
 #ifndef GSM_ENABLE_N71_92INCOMINGINFO
 	return ERR_SOURCENOTAVAILABLE;
-#endif
+#else
 	return NOKIA_SetIncomingUSSD(s,enable);
+#endif
 }
 
 static GSM_Error N7110_SetIncomingSMS(GSM_StateMachine *s, bool enable)
 {
 #ifndef GSM_ENABLE_N71_92INCOMINGINFO
 	return ERR_SOURCENOTAVAILABLE;
-#endif
+#else
 	return NOKIA_SetIncomingSMS(s,enable);
+#endif
 }
 
 GSM_Error N7110_AnswerCall(GSM_StateMachine *s, int ID, bool all)
@@ -1675,6 +1686,7 @@ GSM_Phone_Functions N7110Phone = {
  	NOTIMPLEMENTED,			/* 	AddSMSFolder		*/
  	NOTIMPLEMENTED,			/* 	DeleteSMSFolder		*/
 	DCT3_DialVoice,
+        NOTIMPLEMENTED,			/*	DialService		*/
 	N7110_AnswerCall,
 	DCT3_CancelCall,
  	NOTIMPLEMENTED,			/* 	HoldCall 		*/
@@ -1743,6 +1755,7 @@ GSM_Phone_Functions N7110Phone = {
 	NOTSUPPORTED,			/*	SetFileAttributes	*/
 	NOTSUPPORTED,			/*	GetFilePart		*/
 	NOTSUPPORTED,			/* 	AddFile			*/
+	NOTSUPPORTED,			/* 	SendFilePart		*/
 	NOTSUPPORTED, 			/* 	GetFileSystemStatus	*/
 	NOTSUPPORTED,			/*	DeleteFile		*/
 	NOTSUPPORTED,			/*	AddFolder		*/
