@@ -6,10 +6,12 @@
 #include <ctype.h>
 #include <time.h>
 
+#include <gammu-nokia.h>
+
 #include "../../gsmstate.h"
+#include "../../gsmphones.h"
 #include "../../misc/coding/coding.h"
 #include "../../misc/locales.h"
-#include "../../service/sms/gsmsms.h"
 #include "../pfunc.h"
 #include "nfunc.h"
 
@@ -28,9 +30,9 @@ unsigned char N71_65_MEMORY_TYPES[] = {
 	  0x00,		 0x00
 };
 
-int N71_65_PackPBKBlock(GSM_StateMachine *s, int id, int size, int no, unsigned char *buf, unsigned char *block)
+size_t N71_65_PackPBKBlock(GSM_StateMachine *s, int id, size_t size, int no, unsigned char *buf, unsigned char *block)
 {
-	smprintf(s, "Packing phonebook block with ID = %i, block number = %i, block length = %i\n",id,no+1,size+6);
+	smprintf(s, "Packing phonebook block with ID = %i, block number = %i, block length = " SIZE_T_FORMAT "\n",id,no+1,size+6);
 
 	block[0] 	= id;
 	block[1] 	= 0;
@@ -43,7 +45,7 @@ int N71_65_PackPBKBlock(GSM_StateMachine *s, int id, int size, int no, unsigned
 	return (size + 6);
 }
 
-int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_MemoryEntry *entry, int *block2, bool DCT4, bool VoiceTag)
+size_t N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_MemoryEntry *entry, size_t *block2, bool DCT4, bool VoiceTag)
 {
 	int		count=0, len, i, block=0, j;
 	unsigned char	string[500];
@@ -55,7 +57,7 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 	}
 	memset(string,0,sizeof(string));
 	found = false;
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
  		for (i = 0; i < entry->EntriesNum; i++) {
 			if (entry->Entries[i].EntryType == PBK_Text_LastName ||
 			    entry->Entries[i].EntryType == PBK_Text_FirstName) {
@@ -180,7 +182,12 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 		}
 		dbgprintf("entry num %i %i\n",i,entry->EntriesNum);
 		if (entry->Entries[i].EntryType == PBK_Text_Note)   type = N7110_PBK_NOTE;
-		if (entry->Entries[i].EntryType == PBK_Text_Postal) type = N7110_PBK_POSTAL;
+		if (entry->Entries[i].EntryType == PBK_Text_Postal) {
+			if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKNOPOSTAL)) {
+				continue;
+			}
+			type = N7110_PBK_POSTAL;
+		}
 		if (entry->Entries[i].EntryType == PBK_Text_Email)  type = N7110_PBK_EMAIL;
 		if (entry->Entries[i].EntryType == PBK_Text_Email2) type = N7110_PBK_EMAIL;
 		if (entry->Entries[i].EntryType == PBK_Text_URL) {
@@ -198,9 +205,9 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			continue;
 		}
 		if (entry->Entries[i].EntryType == PBK_Caller_Group) {
-			if (!IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
+			if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
 				entry->Entries[i].AddError = ERR_NONE;
-				if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
+				if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
 					string[0] = 0;
 					string[1] = 0;
 					count += N71_65_PackPBKBlock(s, N6510_PBK_GROUP2_ID, 2, block++, string, req + count);
@@ -214,7 +221,7 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			continue;
 		}
 		if (entry->Entries[i].EntryType == PBK_RingtoneID) {
-			if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBK35)) {
 				entry->Entries[i].AddError = ERR_NONE;
 				string[0] = 0x00;
 				string[1] = 0x00;
@@ -226,7 +233,7 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			continue;
 		}
 		if (entry->Entries[i].EntryType == PBK_PictureID) {
-			if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKIMG)) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKIMG)) {
 				entry->Entries[i].AddError = ERR_NONE;
 				string[0] = 0x00;
 				string[1] = 0x00;
@@ -243,8 +250,32 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			}
 			continue;
 		}
+		/* Maybe we should use separate feature for these new entries... */
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SERIES40_30)) {
+			type = 0;
+			if (entry->Entries[i].EntryType == PBK_Text_FormalName) type = S4030_PBK_FORMALNAME;
+			if (entry->Entries[i].EntryType == PBK_Text_JobTitle) type = S4030_PBK_JOBTITLE;
+			if (entry->Entries[i].EntryType == PBK_Text_Company) type = S4030_PBK_COMPANY;
+			if (entry->Entries[i].EntryType == PBK_Text_NickName) type = S4030_PBK_NICKNAME;
+			if (type != 0) {
+				if (entry->Entries[i].AddError==ERR_NOTSUPPORTED) entry->Entries[i].AddError = ERR_NONE;
+				len = UnicodeLength(entry->Entries[i].Text);
+				string[0] = len*2+2;
+				CopyUnicodeString(string+1,entry->Entries[i].Text);
+				string[len*2+1] = 0;
+				count += N71_65_PackPBKBlock(s, type, len * 2 + 2, block++, string, req + count);
+				continue;
+			}
+			if (entry->Entries[i].EntryType == PBK_Date) {
+				entry->Entries[i].AddError = ERR_NONE;
+
+				NOKIA_EncodeDateTime(s, string + 1, &(entry->Entries[i].Date));
+				count += N71_65_PackPBKBlock(s, S4030_PBK_BIRTHDAY, 6, block++, string, req + count);
+				continue;
+			}
+		}
 		if (entry->Entries[i].EntryType == PBK_Text_UserID) {
-			if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKUSER)) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKUSER)) {
 				entry->Entries[i].AddError = ERR_NONE;
 				string[0] = UnicodeLength(entry->Entries[i].Text)*2;
 				CopyUnicodeString(string+1,entry->Entries[i].Text);
@@ -253,6 +284,37 @@ int N71_65_EncodePhonebookFrame(GSM_StateMachine *s, unsigned char *req, GSM_Mem
 			}
 			continue;
 		}
+		if (entry->Entries[i].EntryType == PBK_PushToTalkID) {
+			entry->Entries[i].AddError = ERR_NONE;
+			string[0] = UnicodeLength(entry->Entries[i].Text)*2;
+			CopyUnicodeString(string+1,entry->Entries[i].Text);
+			count += N71_65_PackPBKBlock(s, N6510_PBK_PUSHTOTALK_ID, string[0]+2, block++, string, req+count);
+			req[count-1]--;
+			continue;
+		}
+		if (entry->Entries[i].EntryType == PBK_Number_Messaging) {
+			if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_PBKFAVORITEMESSAGE)) {
+				entry->Entries[i].AddError = ERR_INVALIDDATA;
+				/* The favorite messaging number is stored as a phone number,
+				 * the phone wants an id to a previously supplied entry, so we search for that.
+				 * In case there was an error in the previous entries, we stop.
+				 * Otherwise we would point past the supplied entries. */
+				for (j = 0; j < i && entry->Entries[j].AddError == ERR_NONE; j++) {
+					if (mywstrncmp(entry->Entries[i].Text, entry->Entries[j].Text, -1)) {
+						string[0] = j + 1;
+						string[1] = 0x00;
+						string[2] = 0x00;
+						count += N71_65_PackPBKBlock(s, N2630_PBK_FAVMESSAGING, 3, block++, string, req + count);
+						count --;
+						req[count-5] = 8;
+						entry->Entries[i].AddError = ERR_NONE;
+						break;
+					}
+
+				}
+			}
+			continue;
+		}
 	}
 
 	*block2=block;
@@ -283,7 +345,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		bitmap->DefaultRingtone 	= true;
 		bitmap->FileSystemPicture 	= false;
 	}
-	if (entry->MemoryType==MEM6510_CG2 && IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
+	if (entry->MemoryType==MEM6510_CG2 && GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_6230iCALLER)) {
 		bitmap->DefaultName 		= false;
 		bitmap->DefaultBitmap 		= true;
 		bitmap->DefaultRingtone 	= true;
@@ -294,8 +356,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 	while (true) {
 		if (bs != 0) {
 			length = length + bs;
-//			if (length == MessageLength) break;
-			//bb5
+			/* bb5 */
 			if (length >= MessageLength-1) break;
 			Block = &Block[bs];
 		}
@@ -322,7 +383,13 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 				smprintf(s, "Too long text\n");
 				return ERR_UNKNOWNRESPONSE;
 			}
-			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			/* No text? */
+			if (Block[5] < 2) {
+				entry->Entries[entry->EntriesNum].Text[0] = 0;
+				entry->Entries[entry->EntriesNum].Text[1] = 0;
+			} else {
+				memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			}
 			entry->Entries[entry->EntriesNum].EntryType=Type;
 			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
 			entry->EntriesNum ++;
@@ -335,8 +402,6 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 	while (true) {
 		if (bs != 0) {
 			length = length + bs;
-//			if (length == MessageLength) break;
-			//bb5
 			if (length >= MessageLength-1) break;
 			Block = &Block[bs];
 		}
@@ -377,15 +442,24 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 				smprintf(s, "Too long text\n");
 				return ERR_UNKNOWNRESPONSE;
 			}
-			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			/* No text? */
+			if (Block[5] < 2) {
+				entry->Entries[entry->EntriesNum].Text[0] = 0;
+				entry->Entries[entry->EntriesNum].Text[1] = 0;
+			} else {
+				memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			}
 			entry->Entries[entry->EntriesNum].EntryType=Type;
 			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
 			if (Block[0] == N7110_PBK_NAME) {
-				if (entry->MemoryType==MEM7110_CG) {
-					memcpy(bitmap->Text,Block+6,Block[5]);
-				}
-				if (entry->MemoryType==MEM6510_CG2) {
-					memcpy(bitmap->Text,Block+6,Block[5]);
+				if (entry->MemoryType == MEM7110_CG || entry->MemoryType == MEM6510_CG2) {
+					/* No text? */
+					if (Block[5] < 2) {
+						bitmap->Text[0] = 0;
+						bitmap->Text[1] = 0;
+					} else {
+						memcpy(bitmap->Text,Block+6,Block[5]);
+					}
 				}
 			}
 			entry->EntriesNum ++;
@@ -394,7 +468,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 		if (Block[0] == N7110_PBK_DATETIME) {
 			entry->Entries[entry->EntriesNum].EntryType=PBK_Date;
 			NOKIA_DecodeDateTime(s, Block+6, &entry->Entries[entry->EntriesNum].Date);
-			//some phones reverse it
+			/* some phones reverse it */
 			if (entry->Entries[entry->EntriesNum].Date.Year > 3000) {
 				entry->Entries[entry->EntriesNum].Date.Year = Block[7]*256+Block[6];
 			}
@@ -436,6 +510,10 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			if (Block[5] == 0x55) {
 				Type = PBK_Number_General;    smprintf(s,"General number ");
 			}
+			/* Yet another unknown General number */
+			if (Block[5] == 0x08) {
+				Type = PBK_Number_General;    smprintf(s,"General number ");
+			}
 			if (Block[5] == N7110_PBK_NUMBER_GENERAL) {
 				Type = PBK_Number_General;    smprintf(s,"General number ");
 			}
@@ -471,7 +549,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->EntriesNum ++;
 			continue;
 		}
-		//to checking
+		/* to checking */
 		if (Block[0] == S4030_PBK_CALLLENGTH) {
 			entry->Entries[entry->EntriesNum].CallLength = Block[9]*256*256+Block[10]*256+Block[11];
 			entry->Entries[entry->EntriesNum].EntryType=PBK_CallLength;
@@ -590,6 +668,71 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->EntriesNum ++;
 			continue;
 		}
+		if (Block[0] == S4030_PBK_FORMALNAME) {
+			Type = PBK_Text_FormalName;    	smprintf(s,"FormalName ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_JOBTITLE) {
+			Type = PBK_Text_JobTitle;    	smprintf(s,"JobTitle ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_COMPANY) {
+			Type = PBK_Text_Company;    	smprintf(s,"Company ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_NICKNAME) {
+			Type = PBK_Text_NickName;    	smprintf(s,"NickName ");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=Type;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == S4030_PBK_BIRTHDAY) {
+			entry->Entries[entry->EntriesNum].EntryType=PBK_Date;
+			NOKIA_DecodeDateTime(s, Block+6, &entry->Entries[entry->EntriesNum].Date);
+			/* We don't have seconds available here */
+			entry->Entries[entry->EntriesNum].Date.Second = 0;
+			/* some phones reverse it */
+			if (entry->Entries[entry->EntriesNum].Date.Year > 3000) {
+				entry->Entries[entry->EntriesNum].Date.Year = Block[7]*256+Block[6];
+			}
+			if (DayMonthReverse) {
+				i = entry->Entries[entry->EntriesNum].Date.Month;
+				entry->Entries[entry->EntriesNum].Date.Month = entry->Entries[entry->EntriesNum].Date.Day;
+				entry->Entries[entry->EntriesNum].Date.Day   = i;
+			}
+			entry->EntriesNum ++;
+			continue;
+		}
 
 
 		if (Block[0] == N7110_PBK_RINGTONE_ID) {
@@ -696,12 +839,11 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 				bitmap->RingtoneID		= Block[10]*256+Block[11];
 				bitmap->DefaultRingtone 	= false;
 			} else {
-				//series 40 3.0
+				/* series 40 3.0 */
 				smprintf(s, "Filesystem ringtone ID: %02x\n",Block[10]*256+Block[11]);
 				entry->Entries[entry->EntriesNum].EntryType=PBK_RingtoneID;
 				entry->Entries[entry->EntriesNum].Number=Block[10]*256+Block[11];
 				entry->EntriesNum ++;
-//				return ERR_UNKNOWNRESPONSE;
 			}
 			continue;
 		}
@@ -730,7 +872,19 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			continue;
 		}
 		if (Block[0] == N6510_PBK_PUSHTOTALK_ID) {
-			smprintf(s,"SIP Address (Push to Talk address) - ignored\n");
+			smprintf(s,"SIP Address (Push to Talk address)\n");
+			if (Block[5]/2>GSM_PHONEBOOK_TEXT_LENGTH) {
+				smprintf(s, "Too long text\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			memcpy(entry->Entries[entry->EntriesNum].Text,Block+6,Block[5]);
+			entry->Entries[entry->EntriesNum].EntryType=PBK_PushToTalkID;
+			smprintf(s, " \"%s\"\n",DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
+		if (Block[0] == N6510_PBK_UNKNOWN5) {
+			smprintf(s,"Unknown entry\n");
 			continue;
 		}
 		if (Block[0] == N6510_PBK_GROUP2_ID) {
@@ -742,6 +896,21 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 			entry->EntriesNum ++;
 			continue;
 		}
+		if (Block[0] == N2630_PBK_FAVMESSAGING) {
+			/* The phone sends an entry id. We store it as phone number because
+			 * entry->Entries doesn't retain order. */
+			int num = (int)Block[5];
+			entry->Entries[entry->EntriesNum].EntryType=PBK_Number_Messaging;
+			if (num >= entry->EntriesNum) {
+				smprintf(s, "ERROR: Number_Messaging references future entry\n");
+				return ERR_UNKNOWNRESPONSE;
+			}
+			CopyUnicodeString(entry->Entries[entry->EntriesNum].Text,entry->Entries[num-1].Text);
+			smprintf(s,"Marked entry #%i (%s) as favorite messaging number\n",
+				num,DecodeUnicodeString(entry->Entries[entry->EntriesNum].Text));
+			entry->EntriesNum ++;
+			continue;
+		}
 		smprintf(s, "ERROR: unknown pbk entry 0x%02x\n",Block[0]);
 		return ERR_UNKNOWNRESPONSE;
 	}
@@ -751,7 +920,7 @@ GSM_Error N71_65_DecodePhonebook(GSM_StateMachine	*s,
 	return ERR_NONE;
 }
 
-void NOKIA_GetDefaultCallerGroupName(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
+void NOKIA_GetDefaultCallerGroupName(GSM_Bitmap *Bitmap)
 {
 	Bitmap->DefaultName = false;
 	if (Bitmap->Text[0]==0x00 && Bitmap->Text[1]==0x00) {
@@ -834,7 +1003,7 @@ GSM_Error NOKIA_GetManufacturer(GSM_StateMachine *s)
  * 3. string (unicode, no termination)
  * This function read string to output and increases counter
  */
-void NOKIA_GetUnicodeString(GSM_StateMachine *s, int *current, unsigned char *input, unsigned char *output, bool FullLength)
+void NOKIA_GetUnicodeString(GSM_StateMachine *s UNUSED, int *current, unsigned char *input, unsigned char *output, bool FullLength)
 {
 	int length;
 
@@ -852,7 +1021,7 @@ void NOKIA_GetUnicodeString(GSM_StateMachine *s, int *current, unsigned char *in
 	output[length+1] = 0;
 }
 
-int NOKIA_SetUnicodeString(GSM_StateMachine *s, unsigned char *dest, unsigned char *string, bool FullLength)
+int NOKIA_SetUnicodeString(GSM_StateMachine *s UNUSED, unsigned char *dest, unsigned char *string, bool FullLength)
 {
 	int length;
 
@@ -870,7 +1039,7 @@ int NOKIA_SetUnicodeString(GSM_StateMachine *s, unsigned char *dest, unsigned ch
 }
 
 /* Returns correct ID for concrete memory type */
-GSM_MemoryType NOKIA_GetMemoryType(GSM_StateMachine *s, GSM_MemoryType memory_type, unsigned char *ID)
+GSM_MemoryType NOKIA_GetMemoryType(GSM_StateMachine *s UNUSED, GSM_MemoryType memory_type, unsigned char *ID)
 {
 	int i=0;
 
@@ -881,7 +1050,7 @@ GSM_MemoryType NOKIA_GetMemoryType(GSM_StateMachine *s, GSM_MemoryType memory_ty
 	return 0xff;
 }
 
-void NOKIA_EncodeDateTime(GSM_StateMachine *s, unsigned char* buffer, GSM_DateTime *datetime)
+void NOKIA_EncodeDateTime(GSM_StateMachine *s UNUSED, unsigned char* buffer, GSM_DateTime *datetime)
 {
 	buffer[0] = datetime->Year / 256;
 	buffer[1] = datetime->Year % 256;
@@ -919,7 +1088,7 @@ void NOKIA_SortSMSFolderStatus(GSM_StateMachine *s, GSM_NOKIASMSFolder *Folder)
 	}
 }
 
-void NOKIA_GetDefaultProfileName(GSM_StateMachine *s, GSM_Profile *Profile)
+void NOKIA_GetDefaultProfileName(GSM_Profile *Profile)
 {
 	if (Profile->DefaultName) {
 		switch(Profile->Location) {
@@ -1091,7 +1260,7 @@ GSM_Error DCT3DCT4_GetActiveConnectSet(GSM_StateMachine *s)
 	return GSM_WaitFor (s, GetSetreq, 4, 0x3f, 4, ID_GetConnectSet);
 }
 
-GSM_Error DCT3DCT4_ReplySetActiveConnectSet(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3DCT4_ReplySetActiveConnectSet(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Connection settings activated\n");
 	return ERR_NONE;
@@ -1116,7 +1285,7 @@ GSM_Error DCT3DCT4_SendDTMF(GSM_StateMachine *s, char *DTMFSequence)
 	unsigned char req[100] = {N6110_FRAME_HEADER, 0x50,
 				  0x00}; 	/* Length */
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NODTMF)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_NODTMF)) return ERR_NOTSUPPORTED;
 	if (strlen(DTMFSequence) > 100 - 5) return ERR_NOTSUPPORTED;
 
   	req[4] = strlen(DTMFSequence);
@@ -1191,7 +1360,7 @@ GSM_Error DCT3DCT4_ReplySetWAPBookmark(GSM_Protocol_Message msg, GSM_StateMachin
 	return ERR_UNKNOWNRESPONSE;
 }
 
-GSM_Error DCT3DCT4_ReplyEnableConnectFunc(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3DCT4_ReplyEnableConnectFunc(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Connection functions enabled\n");
 	return ERR_NONE;
@@ -1201,13 +1370,13 @@ GSM_Error DCT3DCT4_EnableWAPFunctions(GSM_StateMachine *s)
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x00};
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
 
 	smprintf(s, "Enabling WAP\n");
 	return GSM_WaitFor (s, req, 4, 0x3f, 4, ID_EnableConnectFunc);
 }
 
-GSM_Error DCT3DCT4_ReplyDisableConnectFunc(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error DCT3DCT4_ReplyDisableConnectFunc(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Connection functions disabled\n");
 	return ERR_NONE;
@@ -1217,7 +1386,7 @@ GSM_Error DCT3DCT4_DisableConnectionFunctions(GSM_StateMachine *s)
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x03};
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOWAP)) return ERR_NOTSUPPORTED;
 
 	smprintf(s, "Disabling connection settings\n");
 	return GSM_WaitFor (s, req, 4, 0x3f, 4, ID_DisableConnectFunc);
@@ -1311,22 +1480,26 @@ GSM_Error DCT3DCT4_AnswerCall(GSM_StateMachine *s, int ID)
 
 GSM_Error DCT3DCT4_ReplyGetModelFirmware(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	GSM_Lines	lines;
+	GSM_CutLines	lines;
 	GSM_Phone_Data	*Data = &s->Phone.Data;
 
+	InitLines(&lines);
+
 	SplitLines(msg.Buffer, msg.Length, &lines, "\x20\x0A", 2, false);
 
-	strcpy(Data->Model,GetLineString(msg.Buffer, lines, 4));
+	strcpy(Data->Model,GetLineString(msg.Buffer, &lines, 4));
 	smprintf(s, "Received model %s\n",Data->Model);
-	Data->ModelInfo = GetModelData(NULL,Data->Model,NULL);
+	Data->ModelInfo = GetModelData(s, NULL, Data->Model, NULL);
 
-	strcpy(Data->VerDate,GetLineString(msg.Buffer, lines, 3));
+	strcpy(Data->VerDate,GetLineString(msg.Buffer, &lines, 3));
 	smprintf(s, "Received firmware date %s\n",Data->VerDate);
 
-	strcpy(Data->Version,GetLineString(msg.Buffer, lines, 2));
+	strcpy(Data->Version,GetLineString(msg.Buffer, &lines, 2));
 	smprintf(s, "Received firmware version %s\n",Data->Version);
 	GSM_CreateFirmwareNumber(s);
 
+	FreeLines(&lines);
+
 	return ERR_NONE;
 }
 
@@ -1339,13 +1512,13 @@ GSM_Error DCT3DCT4_GetModel (GSM_StateMachine *s)
 
 	smprintf(s, "Getting model\n");
 	error=GSM_WaitFor (s, req, 5, 0xd1, 3, ID_GetModel);
-	if (error==ERR_NONE) {
-		if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
-		    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
-			smprintf(s, "[Connected model  - \"%s\"]\n",s->Phone.Data.Model);
-			smprintf(s, "[Firmware version - \"%s\"]\n",s->Phone.Data.Version);
-			smprintf(s, "[Firmware date    - \"%s\"]\n",s->Phone.Data.VerDate);
-		}
+	if (error == ERR_NONE) {
+		smprintf_level(s, D_TEXT, "[Connected model  - \"%s\"]\n",
+				s->Phone.Data.Model);
+		smprintf_level(s, D_TEXT, "[Firmware version - \"%s\"]\n",
+				s->Phone.Data.Version);
+		smprintf_level(s, D_TEXT, "[Firmware date    - \"%s\"]\n",
+				s->Phone.Data.VerDate);
 	}
 	return error;
 }
@@ -1359,13 +1532,13 @@ GSM_Error DCT3DCT4_GetFirmware (GSM_StateMachine *s)
 
 	smprintf(s, "Getting firmware version\n");
 	error=GSM_WaitFor (s, req, 5, 0xd1, 3, ID_GetFirmware);
-	if (error==ERR_NONE) {
-		if (s->di.dl==DL_TEXT || s->di.dl==DL_TEXTALL ||
-		    s->di.dl==DL_TEXTDATE || s->di.dl==DL_TEXTALLDATE) {
-			smprintf(s, "[Connected model  - \"%s\"]\n",s->Phone.Data.Model);
-			smprintf(s, "[Firmware version - \"%s\"]\n",s->Phone.Data.Version);
-			smprintf(s, "[Firmware date    - \"%s\"]\n",s->Phone.Data.VerDate);
-		}
+	if (error == ERR_NONE) {
+		smprintf_level(s, D_TEXT, "[Connected model  - \"%s\"]\n",
+				s->Phone.Data.Model);
+		smprintf_level(s, D_TEXT, "[Firmware version - \"%s\"]\n",
+				s->Phone.Data.Version);
+		smprintf_level(s, D_TEXT, "[Firmware date    - \"%s\"]\n",
+				s->Phone.Data.VerDate);
 	}
 	return error;
 }
@@ -1379,9 +1552,12 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 		smprintf(s, "No PIN\n");
 		return ERR_SECURITYERROR;
 	case 0x30:
+		if (s->Phone.Data.Memory->MemoryType == MEM_ME ||
+				s->Phone.Data.Memory->MemoryType == MEM_SM) {
+			smprintf(s, "Empty entry\n");
+			return ERR_EMPTY;
+		}
 		smprintf(s, "Invalid memory type\n");
-		if (s->Phone.Data.Memory->MemoryType == MEM_ME) return ERR_EMPTY;
-		if (s->Phone.Data.Memory->MemoryType == MEM_SM) return ERR_EMPTY;
 		return ERR_NOTSUPPORTED;
 	case 0x31:
 		smprintf(s, "Invalid memory type?\n");
@@ -1394,10 +1570,14 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 	case 0x34:
 		smprintf(s, "Too high location ?\n");
 		return ERR_INVALIDLOCATION;
-	case 0x3B: //Tim Dreessen, 6230
+	case 0x3B: /* Tim Dreessen, 6230 */
 		smprintf(s, "Empty location\n");
 		s->Phone.Data.Memory->EntriesNum = 0;
-		return ERR_EMPTY;
+		/*
+		 * This is really empty, but this entry is calculated to
+		 * entries count, so we must return something.
+		 */
+		return ERR_NONE;
 	default:
 		smprintf(s, "ERROR: unknown %i\n",error);
 		return ERR_UNKNOWNRESPONSE;
@@ -1406,17 +1586,25 @@ GSM_Error N71_65_ReplyGetMemoryError(unsigned char error, GSM_StateMachine *s)
 
 GSM_Error N71_65_ReplyWritePhonebook(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	smprintf(s, "Phonebook entry written ");
 	switch (msg.Buffer[6]) {
 	case 0x0f:
-		smprintf(s, " - error\n");
+		smprintf(s, "Phonebook entry writing failed\n");
 		switch (msg.Buffer[10]) {
+		case 0xf:
+			smprintf(s, "Invalid block sent\n");
+			return ERR_BUG;
+		case 0x23:
+			smprintf(s, "Block size does not match a definition\n");
+			return ERR_BUG;
 		case 0x25:
 			smprintf(s, "when you try to save into entry with caller group assigment in phone with caller groups standard 2 (like in 6230i)\n");
 			return ERR_PERMISSION;
 		case 0x29:
 			smprintf(s, "no caller group with given number (6230i)\n");
 			return ERR_MEMORY;
+		case 0x32:
+			smprintf(s, "Ignoring ERROR: unknown 50 (probably group contains 50 entries)\n");
+			return ERR_NONE;
 		case 0x36:
 			smprintf(s, "Too long name\n");
 			return ERR_NOTSUPPORTED;
@@ -1429,15 +1617,15 @@ GSM_Error N71_65_ReplyWritePhonebook(GSM_Protocol_Message msg, GSM_StateMachine
 		case 0x3e:
 			smprintf(s, "Too much entries\n");
 			return ERR_NOTSUPPORTED;
-		case 50:
-			smprintf(s, "Ignoring ERROR: unknown 50 (probably group contains 50 entries)\n");
-			return ERR_NONE;
+		case 0x43:
+			smprintf(s, "Incorrect characters\n");
+			return ERR_NOTSUPPORTED;
 		default:
 			smprintf(s, "ERROR: unknown %i\n",msg.Buffer[10]);
 			return ERR_UNKNOWNRESPONSE;
 		}
 	default:
-		smprintf(s, " - OK\n");
+		smprintf(s, "Phonebook entry written\n");
 		return ERR_NONE;
 	}
 }
@@ -1568,7 +1756,8 @@ GSM_Error NOKIA_SetIncomingSMS(GSM_StateMachine *s, bool enable)
 
 GSM_Error N71_65_ReplyUSSDInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	unsigned char buffer[2000],buffer2[4000];
+	unsigned char buffer[2000];
+	GSM_USSDMessage ussd;
 
 	if (s->Phone.Data.RequestID == ID_Divert) return ERR_NONE;
 
@@ -1578,8 +1767,12 @@ GSM_Error N71_65_ReplyUSSDInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	smprintf(s, "USSD reply: \"%s\"\n",buffer);
 
 	if (s->Phone.Data.EnableIncomingUSSD && s->User.IncomingUSSD!=NULL) {
-		EncodeUnicode(buffer2,buffer,strlen(buffer));
-		s->User.IncomingUSSD(s, buffer2);
+		EncodeUnicode(ussd.Text,buffer,strlen(buffer));
+		/**
+		 * @todo: Should determine status.
+		 */
+		ussd.Status = USSD_Unknown;
+		s->User.IncomingUSSD(s, ussd);
 	}
 
 	return ERR_NONE;
@@ -1600,7 +1793,7 @@ GSM_Error NOKIA_SetIncomingUSSD(GSM_StateMachine *s, bool enable)
 
 GSM_Error NOKIA_SetIncomingCall(GSM_StateMachine *s, bool enable)
 {
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOCALLINFO)) return ERR_NOTSUPPORTED;
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo,F_NOCALLINFO)) return ERR_NOTSUPPORTED;
 
 	s->Phone.Data.EnableIncomingCall = enable;
 #ifdef DEBUG
@@ -1629,7 +1822,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x03:
 		smprintf(s, "Call started\n");
-		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		tmp = 6;
 		NOKIA_GetUnicodeString(s, &tmp, msg.Buffer,buffer,false);
 		smprintf(s, "Number     : \"%s\"\n",DecodeUnicodeString(buffer));
@@ -1639,7 +1832,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x04:
 		smprintf(s, "Remote end hang up\n");
-		smprintf(s, "Cause Type : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Cause Type : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		smprintf(s, "CC         : %i\n",msg.Buffer[6]);
 		smprintf(s, "MM(?)      : %i\n",msg.Buffer[7]);
 		smprintf(s, "RR(?)      : %i\n",msg.Buffer[8]);
@@ -1648,7 +1841,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x05:
 		smprintf(s, "Incoming call\n");
-		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		tmp = 6;
 		NOKIA_GetUnicodeString(s, &tmp, msg.Buffer,buffer,false);
 		smprintf(s, "Number     : \"%s\"\n",DecodeUnicodeString(buffer));
@@ -1677,7 +1870,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 				      else smprintf(s, "Audio disabled\n");
 		call.CallIDAvailable = false;
 		break;
-	case 0x0f: //6111
+	case 0x0f: /* 6111 */
 		if (msg.Buffer[8]==0x01) {
 			smprintf(s, "Calling from phone keypad ?\n");
 			if (msg.Buffer[14]==0x03) {
@@ -1720,7 +1913,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		break;
 	case 0x53:
 		smprintf(s, "Outgoing call\n");
-		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);//such interpretation is in gnokii
+		smprintf(s, "Call mode  : %i\n",msg.Buffer[5]);/* such interpretation is in gnokii */
 		tmp = 6;
 		NOKIA_GetUnicodeString(s, &tmp, msg.Buffer,buffer,false);
 		smprintf(s, "Number     : \"%s\"\n",DecodeUnicodeString(buffer));
@@ -1764,7 +1957,7 @@ GSM_Error N71_65_ReplyCallInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 }
 
 /* method 2 */
-GSM_Error N71_65_ReplyAddCalendar2(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error N71_65_ReplyAddCalendar2(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Calendar note added\n");
 	return ERR_NONE;
@@ -1794,8 +1987,8 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 
  	NoteType = N71_65_FindCalendarType(Note->Type, s->Phone.Data.ModelInfo);
 
-	if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62) ||
-	    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62) ||
+	    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65)) {
 		switch(NoteType) {
 		case GSM_CAL_MEETING 	: req[18] = 0x01; length = 25; break;
 		case GSM_CAL_CALL    	: req[18] = 0x02; length = 27; break;
@@ -1815,7 +2008,10 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 
 	GSM_CalendarFindDefaultTextTimeAlarmPhone(Note, &Text, &Time, &Alarm, &Phone, &EndTime, &Location);
 
-	if (Time == -1) return ERR_UNKNOWN;
+	if (Time == -1) {
+		smprintf(s, "Can not save entry without time!\n");
+		return ERR_UNKNOWN;
+	}
 	if (NoteType != GSM_CAL_BIRTHDAY) {
 		Date.Year 	= 2030;	Date.Month 	= 01; Date.Day    = 01;
 		Date.Hour 	= 00;	Date.Minute 	= 00; Date.Second = 00;
@@ -1831,9 +2027,9 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 		/* 6230 and probably other new models handle it differently
                    we don't make difference from 1980 year
                  */
-		if (IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65) ||
-		    IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62)) {
+		if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL65) ||
+		    GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL62)) {
 			Date.Year = 1980;
 		}
 		Date.Hour = 22; Date.Minute = 58; Date.Second = 58;
@@ -1885,7 +2081,7 @@ GSM_Error N71_65_AddCalendar2(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 
 		/* Sometimes we have difference in minutes */
 		if (NoteType == GSM_CAL_MEETING) diff = diff / 60;
-		if (!IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35)) {
+		if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CAL35)) {
 			if (NoteType == GSM_CAL_MEMO || NoteType == GSM_CAL_CALL) {
 				diff = diff / 60;
 			}
@@ -1990,7 +2186,10 @@ GSM_Error N71_65_AddCalendar1(GSM_StateMachine *s, GSM_CalendarEntry *Note, int
 
 	GSM_CalendarFindDefaultTextTimeAlarmPhone(Note, &Text, &Time, &Alarm, &Phone, &EndTime, &Location);
 
-	if (Time == -1) return ERR_UNKNOWN;
+	if (Time == -1) {
+		smprintf(s, "Can not save entry without time!\n");
+		return ERR_UNKNOWN;
+	}
 	memcpy(&DT,&Note->Entries[Time].Date,sizeof(GSM_DateTime));
 	req[8]	= DT.Year / 256;
 	req[9]	= DT.Year % 256;
@@ -2009,13 +2208,11 @@ GSM_Error N71_65_AddCalendar1(GSM_StateMachine *s, GSM_CalendarEntry *Note, int
 		req[count++] = 0xff;
 		req[count++] = 0xff;
 		if (Alarm != -1) {
-#ifndef ENABLE_LGPL
 			/* Comment from original source by Gabriele Zappi:
 			 * I try with Time.Year = Alarm.Year. If negative, I increase 1 year,
 			 * but only once ! This thing, because I may have Alarm period across
 			 * a year. (eg. Birthday on 2001-01-10 and Alarm on 2000-12-27)
 			 */
-#endif
 			DT.Year = Note->Entries[Alarm].Date.Year;
 			seconds = Fill_Time_T(DT)-Fill_Time_T(Note->Entries[Alarm].Date);
 			if (seconds<0L) {
@@ -2130,7 +2327,7 @@ GSM_Error N71_65_DelCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 /* method 1 */
 GSM_Error N71_65_ReplyGetCalendarInfo1(GSM_Protocol_Message msg, GSM_StateMachine *s, GSM_NOKIACalToDoLocations *LastCalendar)
 {
-	int i,j=0;
+	size_t i,j=0;
 
 	smprintf(s, "Info with calendar notes locations received method 1\n");
 	while (LastCalendar->Location[j] != 0x00) j++;
@@ -2149,7 +2346,7 @@ GSM_Error N71_65_ReplyGetCalendarInfo1(GSM_Protocol_Message msg, GSM_StateMachin
 		smprintf(s, "%i ",LastCalendar->Location[j-1]);
 		i++;
 	}
-	smprintf(s, "\nNumber of Entries in frame: %i\n",i);
+	smprintf(s, "\nNumber of Entries in frame: " SIZE_T_FORMAT "\n",i);
 	smprintf(s, "\n");
 	LastCalendar->Location[j] = 0;
 	if (i == 1 && msg.Buffer[8+(0*2)]*256+msg.Buffer[9+(0*2)] == 0) return ERR_EMPTY;
@@ -2194,7 +2391,7 @@ GSM_Error N71_65_GetCalendarInfo1(GSM_StateMachine *s, GSM_NOKIACalToDoLocations
 /* method 1 */
 GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachine *s)
 {
-	int 			alarm,i;
+	int 			timedelta,i;
 	GSM_CalendarEntry	*entry = s->Phone.Data.Cal;
 
 	smprintf(s, "Calendar note received method 1\n");
@@ -2216,11 +2413,11 @@ GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachin
 		smprintf(s, "Meeting\n");
 		entry->Type = GSM_CAL_MEETING;
 
-		alarm=msg.Buffer[14]*256+msg.Buffer[15];
-		if (alarm != 0xffff) {
-			smprintf(s, "  Difference : %i seconds\n", alarm);
+		timedelta=msg.Buffer[14]*256+msg.Buffer[15];
+		if (timedelta != 0xffff) {
+			smprintf(s, "  Difference : %i seconds\n", timedelta);
 			memcpy(&entry->Entries[1].Date,&entry->Entries[0].Date,sizeof(GSM_DateTime));
-			GetTimeDifference(alarm, &entry->Entries[1].Date, false, 60);
+			GetTimeDifference(timedelta, &entry->Entries[1].Date, false, 60);
 			entry->Entries[1].EntryType = CAL_TONE_ALARM_DATETIME;
 			entry->EntriesNum++;
 		}
@@ -2237,11 +2434,11 @@ GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachin
 		smprintf(s, "Call\n");
 		entry->Type = GSM_CAL_CALL;
 
-		alarm=msg.Buffer[14]*256+msg.Buffer[15];
-		if (alarm != 0xffff) {
-			smprintf(s, "  Difference : %i seconds\n", alarm);
+		timedelta=msg.Buffer[14]*256+msg.Buffer[15];
+		if (timedelta != 0xffff) {
+			smprintf(s, "  Difference : %i seconds\n", timedelta);
 			memcpy(&entry->Entries[1].Date,&entry->Entries[0].Date,sizeof(GSM_DateTime));
-			GetTimeDifference(alarm, &entry->Entries[1].Date, false, 60);
+			GetTimeDifference(timedelta, &entry->Entries[1].Date, false, 60);
 			entry->Entries[1].EntryType = CAL_TONE_ALARM_DATETIME;
 			entry->EntriesNum++;
 		}
@@ -2272,14 +2469,14 @@ GSM_Error N71_65_ReplyGetNextCalendar1(GSM_Protocol_Message msg, GSM_StateMachin
 		entry->Entries[0].Date.Minute	= 59;
 		entry->Entries[0].Date.Second	= 58;
 
-		alarm  = ((unsigned int)msg.Buffer[14]) << 24;
-		alarm += ((unsigned int)msg.Buffer[15]) << 16;
-		alarm += ((unsigned int)msg.Buffer[16]) << 8;
-		alarm += msg.Buffer[17];
-		if (alarm != 0xffff) {
-			smprintf(s, "  Difference : %i seconds\n", alarm);
+		timedelta  = ((unsigned int)msg.Buffer[14]) << 24;
+		timedelta += ((unsigned int)msg.Buffer[15]) << 16;
+		timedelta += ((unsigned int)msg.Buffer[16]) << 8;
+		timedelta += msg.Buffer[17];
+		if (timedelta != 0xffff) {
+			smprintf(s, "  Difference : %i seconds\n", timedelta);
 			memcpy(&entry->Entries[1].Date,&entry->Entries[0].Date,sizeof(GSM_DateTime));
-			GetTimeDifference(alarm, &entry->Entries[1].Date, false, 1);
+			GetTimeDifference(timedelta, &entry->Entries[1].Date, false, 1);
 			entry->Entries[1].EntryType = CAL_TONE_ALARM_DATETIME;
 			if (msg.Buffer[20]!=0x00) {
 				entry->Entries[1].EntryType = CAL_SILENT_ALARM_DATETIME;
@@ -2413,16 +2610,16 @@ GSM_CalendarNoteType N71_65_FindCalendarType(GSM_CalendarNoteType Type, OnePhone
 	case GSM_CAL_BIRTHDAY:
 		return GSM_CAL_BIRTHDAY;
 	case GSM_CAL_MEETING:
-		if (IsPhoneFeatureAvailable(model, F_CAL35)) {
+		if (GSM_IsPhoneFeatureAvailable(model, F_CAL35)) {
 			return GSM_CAL_REMINDER;
 		} else return GSM_CAL_MEETING;
 	case GSM_CAL_MEMO:
-		if (IsPhoneFeatureAvailable(model, F_CAL35)) {
+		if (GSM_IsPhoneFeatureAvailable(model, F_CAL35)) {
 			return GSM_CAL_REMINDER;
 		} else return GSM_CAL_MEMO;
 	case GSM_CAL_REMINDER:
-		if (IsPhoneFeatureAvailable(model, F_CAL62) ||
-		    IsPhoneFeatureAvailable(model, F_CAL65)) {
+		if (GSM_IsPhoneFeatureAvailable(model, F_CAL62) ||
+		    GSM_IsPhoneFeatureAvailable(model, F_CAL65)) {
 			return GSM_CAL_CALL;
 		} else return GSM_CAL_REMINDER;
 	default:
