@@ -29,7 +29,6 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 {
 	unsigned int i;
 	int start=-1,frequency=-1,dow=-1,day=-1,month=-1,end=-1,Recurrance = 0, Repeat=0, j;
-	char 		Buf[20];
 	GSM_DateTime	DT;
 	time_t		t_time1,t_time2;
 
@@ -53,16 +52,9 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 		}
 	}
 
-	sprintf(Buf,"%s",DayOfWeek(entry->Entries[start].Date.Year,
+	i = GetDayOfWeek(entry->Entries[start].Date.Year,
 			entry->Entries[start].Date.Month,
-			entry->Entries[start].Date.Day));
-	if (!strcmp(Buf,"Mon")) i = 1;
-	if (!strcmp(Buf,"Tue")) i = 2;
-	if (!strcmp(Buf,"Wed")) i = 3;
-	if (!strcmp(Buf,"Thu")) i = 4;
-	if (!strcmp(Buf,"Fri")) i = 5;
-	if (!strcmp(Buf,"Sat")) i = 6;
-	if (!strcmp(Buf,"Sun")) i = 7;
+			entry->Entries[start].Date.Day);
 
 	if (frequency != -1 && dow != -1 && day == -1 && month == -1) {
 		if (entry->Entries[frequency].Number == 1 &&
@@ -142,7 +134,6 @@ void GSM_SetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 void GSM_GetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday, GSM_CalendarEntry *entry)
 {
 	int 	Recurrance,num=-1,i;
-	char 	Buf[20];
 
 	Recurrance = rec[0]*256 + rec[1];
 	if (Recurrance == 0) return;
@@ -159,9 +150,6 @@ void GSM_GetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 		}
 	}
 	if (num == -1) return;
-	sprintf(Buf,"%s",DayOfWeek(entry->Entries[num].Date.Year,
-			entry->Entries[num].Date.Month,
-			entry->Entries[num].Date.Day));
 
 	if (Recurrance == 24    || Recurrance == 24*7 ||
 	    Recurrance == 24*30 || Recurrance == 24*365) {
@@ -176,21 +164,9 @@ void GSM_GetCalendarRecurranceRepeat(unsigned char *rec, unsigned char *endday,
 	}
 	if (Recurrance == 24*7 || Recurrance == 24*14) {
 		entry->Entries[entry->EntriesNum].EntryType	 = CAL_REPEAT_DAYOFWEEK;
-		if (!strcmp(Buf,"Mon")) {
-			entry->Entries[entry->EntriesNum].Number = 1;
-		} else if (!strcmp(Buf,"Tue")) {
-			entry->Entries[entry->EntriesNum].Number = 2;
-		} else if (!strcmp(Buf,"Wed")) {
-			entry->Entries[entry->EntriesNum].Number = 3;
-		} else if (!strcmp(Buf,"Thu")) {
-			entry->Entries[entry->EntriesNum].Number = 4;
-		} else if (!strcmp(Buf,"Fri")) {
-			entry->Entries[entry->EntriesNum].Number = 5;
-		} else if (!strcmp(Buf,"Sat")) {
-			entry->Entries[entry->EntriesNum].Number = 6;
-		} else if (!strcmp(Buf,"Sun")) {
-			entry->Entries[entry->EntriesNum].Number = 7;
-		}
+		entry->Entries[entry->EntriesNum].Number = GetDayOfWeek(entry->Entries[num].Date.Year,
+			entry->Entries[num].Date.Month,
+			entry->Entries[num].Date.Day);
 		entry->EntriesNum++;
 	}
 	if (Recurrance == 24*30) {
@@ -1591,7 +1567,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 	Calendar->EntriesNum 	= 0;
 	ToDo->EntriesNum 	= 0;
 	lBuffer = strlen(Buffer);
-	trigger.Timezone = -999;
+	trigger.Timezone = -999 * 3600;
 
 	if (CalVer == Mozilla_iCalendar && *Pos ==0) {
 		error = GSM_Make_VCAL_Lines (Buffer, &lBuffer);
@@ -1599,7 +1575,8 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 	}
 
 	while (1) {
-		MyGetLine(Buffer, Pos, Line, lBuffer, true);
+		error = MyGetLine(Buffer, Pos, Line, lBuffer, sizeof(Line), true);
+		if (error != ERR_NONE) return error;
 		if (strlen(Line) == 0) break;
 
 		switch (Level) {
@@ -1639,7 +1616,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 				}
 				if (Calendar->EntriesNum == 0) return ERR_EMPTY;
 
-				if (trigger.Timezone != -999) {
+				if (trigger.Timezone != -999 * 3600) {
 					Alarm = Calendar->EntriesNum;
 					Calendar->Entries[Alarm].Date = GSM_AddTime (Calendar->Entries[Time].Date, trigger);
 					Calendar->Entries[Alarm].EntryType = CAL_TONE_ALARM_DATETIME;
@@ -1691,7 +1668,8 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(unsigned char *Buffer, int *Pos, GSM_Calenda
 			}
 
 			if (strstr(Line,"BEGIN:VALARM")) {
-				MyGetLine(Buffer, Pos, Line, lBuffer, true);
+				error = MyGetLine(Buffer, Pos, Line, lBuffer, sizeof(Line), true);
+				if (error != ERR_NONE) return error;
 				if (strlen(Line) == 0) break;
 				if (ReadVCALText(Line, "TRIGGER;VALUE=DURATION", Buff, CalVer == Mozilla_iCalendar)) {
 					trigger = ReadVCALTriggerTime(DecodeUnicodeString(Buff));
@@ -1932,12 +1910,14 @@ GSM_Error GSM_DecodeVNOTE(unsigned char *Buffer, int *Pos, GSM_NoteEntry *Note)
 {
 	unsigned char   Line[2000],Buff[2000];
 	int	     Level = 0;
+	GSM_Error	error;
 
 	Note->Text[0] = 0;
 	Note->Text[1] = 0;
 
 	while (1) {
-		MyGetLine(Buffer, Pos, Line, strlen(Buffer), true);
+		error = MyGetLine(Buffer, Pos, Line, strlen(Buffer), sizeof(Line), true);
+		if (error != ERR_NONE) return error;
 		if (strlen(Line) == 0) break;
 		switch (Level) {
 		case 0:
