@@ -1,6 +1,9 @@
 /* (c) 2001-2005 by Marcin Wiacek, Michal Cihar... */
 
 #include <string.h>
+#include <stdio.h>
+
+#include <gammu-misc.h>
 
 #include "../misc/coding/coding.h"
 #include "gsmpbk.h"
@@ -99,11 +102,12 @@ void GSM_PhonebookFindDefaultNameNumberGroup(GSM_MemoryEntry *entry, int *Name,
 	}
 }
 
-void GSM_EncodeVCARD(char *Buffer, int *Length, GSM_MemoryEntry *pbk, bool header, GSM_VCardVersion Version)
+void GSM_EncodeVCARD(char *Buffer, size_t *Length, GSM_MemoryEntry *pbk, bool header, GSM_VCardVersion Version)
 {
 	int Name, Number, Group, i;
 	int firstname = -1, lastname = -1;
 	int address = -1, city = -1, state = -1, zip = -1, country = -1;
+	int workaddress = -1, workcity = -1, workstate = -1, workzip = -1, workcountry = -1;
 	unsigned char buffer[1024];
 	int pos;
 	bool ignore;
@@ -123,10 +127,17 @@ void GSM_EncodeVCARD(char *Buffer, int *Length, GSM_MemoryEntry *pbk, bool heade
 		if (header) *Length+=sprintf(Buffer+(*Length),"BEGIN:VCARD%c%cVERSION:2.1%c%c",13,10,13,10);
 		for (i=0; i < pbk->EntriesNum; i++) {
 			ignore = false;
+			pbk->Entries[i].AddError = ERR_NONE;
 			switch(pbk->Entries[i].EntryType) {
 				case PBK_Text_Name:
 					*Length+=sprintf(Buffer+(*Length),"N");
 					break;
+				case PBK_Text_NickName:
+					*Length+=sprintf(Buffer+(*Length),"NICKNAME");
+					break;
+				case PBK_Text_FormalName:
+					*Length+=sprintf(Buffer+(*Length),"FN");
+					break;
 				case PBK_Text_FirstName:
 					firstname = i;
 					ignore = true;
@@ -155,6 +166,26 @@ void GSM_EncodeVCARD(char *Buffer, int *Length, GSM_MemoryEntry *pbk, bool heade
 					country = i;
 					ignore = true;
 					break;
+				case PBK_Text_WorkStreetAddress:
+					workaddress = i;
+					ignore = true;
+					break;
+				case PBK_Text_WorkCity:
+					workcity = i;
+					ignore = true;
+					break;
+				case PBK_Text_WorkState:
+					workstate = i;
+					ignore = true;
+					break;
+				case PBK_Text_WorkZip:
+					workzip = i;
+					ignore = true;
+					break;
+				case PBK_Text_WorkCountry:
+					workcountry = i;
+					ignore = true;
+					break;
 				case PBK_Date:
 					SaveVCALDate(Buffer, Length, &(pbk->Entries[i].Date), "BDAY");
 					ignore = true;
@@ -164,62 +195,135 @@ void GSM_EncodeVCARD(char *Buffer, int *Length, GSM_MemoryEntry *pbk, bool heade
 					ignore = true;
 					break;
 				case PBK_Number_General:
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TEL");
 					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
 					break;
 				case PBK_Number_Other:
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TEL");
 					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
+					*Length+=sprintf(Buffer+(*Length),";OTHER");
 					break;
 				case PBK_Number_Pager:
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TEL");
 					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
 					*Length+=sprintf(Buffer+(*Length),";PAGER");
 					break;
 				case PBK_Number_Mobile  :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TEL");
 					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
 					*Length+=sprintf(Buffer+(*Length),";CELL");
 					break;
 				case PBK_Number_Work    :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TEL");
 					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
 					*Length+=sprintf(Buffer+(*Length),";WORK");
 					break;
 				case PBK_Number_Fax     :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TEL");
 					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
 					*Length+=sprintf(Buffer+(*Length),";FAX");
 					break;
 				case PBK_Number_Home    :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TEL");
 					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
 					*Length+=sprintf(Buffer+(*Length),";HOME");
 					break;
+				case PBK_Number_Messaging    :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
+					*Length+=sprintf(Buffer+(*Length),"TEL");
+					if (Version != SonyEricsson_VCard21 && Number == i) (*Length)+=sprintf(Buffer+(*Length),";PREF");
+					*Length+=sprintf(Buffer+(*Length),";MSG");
+					break;
 				case PBK_Text_Note      :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"NOTE");
 					break;
 				case PBK_Text_Postal    :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					/* Don't ask why. Nokia phones save postal address
 					 * double - once like LABEL, second like ADR
 					 */
 					SaveVCALText(Buffer, Length, pbk->Entries[i].Text, "LABEL", false);
-					*Length+=sprintf(Buffer+(*Length),"ADR");
+					*Length+=sprintf(Buffer+(*Length),"ADR;HOME");
+					break;
+				case PBK_Text_WorkPostal    :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
+					*Length+=sprintf(Buffer+(*Length),"ADR;WORK");
 					break;
 				case PBK_Text_Email     :
 				case PBK_Text_Email2    :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"EMAIL");
 					break;
 				case PBK_Text_URL       :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"URL");
 					break;
 				case PBK_Text_LUID      :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"X-IRMC-LUID");
 					break;
 				case PBK_Text_JobTitle:
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"TITLE");
 					break;
 				case PBK_Text_Company:
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = true;
+						break;
+					}
 					*Length+=sprintf(Buffer+(*Length),"ORG");
 					break;
 				/* Not supported fields */
@@ -234,12 +338,15 @@ void GSM_EncodeVCARD(char *Buffer, int *Length, GSM_MemoryEntry *pbk, bool heade
 				case PBK_Text_Custom2:
 				case PBK_Text_Custom3:
 				case PBK_Text_Custom4:
+				case PBK_Text_PictureName:
+				case PBK_PushToTalkID:
+					pbk->Entries[i].AddError = ERR_NOTSUPPORTED;
 					ignore = true;
 					break;
-				}
-				if (!ignore) {
-					SaveVCALText(Buffer, Length, pbk->Entries[i].Text, "", false);
-				}
+			}
+			if (!ignore) {
+				SaveVCALText(Buffer, Length, pbk->Entries[i].Text, "", false);
+			}
 		}
 		/* Save name if it is composed from parts */
 		if (firstname != -1 || lastname != -1) {
@@ -259,6 +366,51 @@ void GSM_EncodeVCARD(char *Buffer, int *Length, GSM_MemoryEntry *pbk, bool heade
 			buffer[2*pos + 1] = 0;
 			SaveVCALText(Buffer, Length, buffer, "N", false);
 		}
+		/* Save workaddress if it is composed from parts */
+		if (workaddress != -1 || workcity != -1 || workstate != -1 || workzip != -1 || workcountry != -1) {
+			pos = 0;
+			buffer[2*pos] = 0;
+			buffer[2*pos + 1] = ';';
+			pos++;
+			buffer[2*pos] = 0;
+			buffer[2*pos + 1] = ';';
+			pos++;
+			if (workaddress != -1) {
+				CopyUnicodeString(buffer + 2*pos, pbk->Entries[workaddress].Text);
+				pos += UnicodeLength(pbk->Entries[workaddress].Text);
+			}
+			buffer[2*pos] = 0;
+			buffer[2*pos + 1] = ';';
+			pos++;
+			if (workcity != -1) {
+				CopyUnicodeString(buffer + 2*pos, pbk->Entries[workcity].Text);
+				pos += UnicodeLength(pbk->Entries[workcity].Text);
+			}
+			buffer[2*pos] = 0;
+			buffer[2*pos + 1] = ';';
+			pos++;
+			if (workstate != -1) {
+				CopyUnicodeString(buffer + 2*pos, pbk->Entries[workstate].Text);
+				pos += UnicodeLength(pbk->Entries[workstate].Text);
+			}
+			buffer[2*pos] = 0;
+			buffer[2*pos + 1] = ';';
+			pos++;
+			if (workzip != -1) {
+				CopyUnicodeString(buffer + 2*pos, pbk->Entries[workzip].Text);
+				pos += UnicodeLength(pbk->Entries[workzip].Text);
+			}
+			buffer[2*pos] = 0;
+			buffer[2*pos + 1] = ';';
+			pos++;
+			if (workcountry != -1) {
+				CopyUnicodeString(buffer + 2*pos, pbk->Entries[workcountry].Text);
+				pos += UnicodeLength(pbk->Entries[workcountry].Text);
+			}
+			buffer[2*pos] = 0;
+			buffer[2*pos + 1] = 0;
+			SaveVCALText(Buffer, Length, buffer, "ADR;WORK", false);
+		}
 		/* Save address if it is composed from parts */
 		if (address != -1 || city != -1 || state != -1 || zip != -1 || country != -1) {
 			pos = 0;
@@ -302,7 +454,7 @@ void GSM_EncodeVCARD(char *Buffer, int *Length, GSM_MemoryEntry *pbk, bool heade
 			}
 			buffer[2*pos] = 0;
 			buffer[2*pos + 1] = 0;
-			SaveVCALText(Buffer, Length, buffer, "ADR", false);
+			SaveVCALText(Buffer, Length, buffer, "ADR;HOME", false);
 		}
 		if (header) *Length+=sprintf(Buffer+(*Length),"END:VCARD%c%c",13,10);
 	}
@@ -357,19 +509,23 @@ void GSM_TweakInternationalNumber(unsigned char *Number, const GSM_NumberType nu
 }
 
 
-GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk, GSM_VCardVersion Version)
+/**
+ * \bug We should avoid using static buffers here.
+ */
+GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk, GSM_VCardVersion Version UNUSED)
 {
-	unsigned char   Line[2000],Buff[2000];
+	unsigned char   Line[20000], Buff[20000];
 	int	     Level = 0;
 	unsigned char   *s;
 	int		pos;
-	bool		address = false;
+	GSM_Error	error;
 
 	Buff[0]	 = 0;
 	Pbk->EntriesNum = 0;
 
 	while (1) {
-		MyGetLine(Buffer, Pos, Line, strlen(Buffer), true);
+		error = MyGetLine(Buffer, Pos, Line, strlen(Buffer), sizeof(Line), true);
+		if (error != ERR_NONE) return error;
 		if (strlen(Line) == 0) break;
 		switch (Level) {
 		case 0:
@@ -381,6 +537,7 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 				if (Pbk->EntriesNum == 0) return ERR_EMPTY;
 				return ERR_NONE;
 			}
+			Pbk->Entries[Pbk->EntriesNum].AddError = ERR_NONE;
 			if (ReadVCALText(Line, "N", Buff, false)) {
 				pos = 0;
 				s = VCALGetTextPart(Buff, &pos);
@@ -406,8 +563,7 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 			}
 			if (ReadVCALText(Line, "TEL",		   Buff, false) ||
 			    ReadVCALText(Line, "TEL;VOICE",	     Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF",	      Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;VOICE",	Buff, false)) {
+			    ReadVCALText(Line, "TEL;TYPE=VOICE",	     Buff, false)) {
 				if (Buff[1] == '+') {
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
@@ -419,8 +575,10 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 			}
 			if (ReadVCALText(Line, "TEL;CELL",	      Buff, false) ||
 			    ReadVCALText(Line, "TEL;CELL;VOICE",	Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;CELL",	 Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;CELL;VOICE",   Buff, false)) {
+			    ReadVCALText(Line, "TEL;TYPE=CELL",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=CELL,VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=CELL;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=CELL;VOICE",   Buff, false)) {
 				if (Buff[1] == '+') {
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
@@ -431,9 +589,11 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 				Pbk->EntriesNum++;
 			}
 			if (ReadVCALText(Line, "TEL;WORK",	      Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;WORK",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=WORK",	 Buff, false) ||
 			    ReadVCALText(Line, "TEL;WORK;VOICE",	Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;WORK;VOICE",   Buff, false)) {
+			    ReadVCALText(Line, "TEL;TYPE=WORK;VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=WORK;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=WORK,VOICE",   Buff, false)) {
 				if (Buff[1] == '+') {
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
@@ -443,10 +603,58 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
 			}
+			if (ReadVCALText(Line, "TEL;OTHER",	      Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=OTHER",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;OTHER;VOICE",	Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=OTHER;VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=OTHER;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=OTHER,VOICE",   Buff, false)) {
+				if (Buff[1] == '+') {
+					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
+				}
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
+				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
+				Pbk->EntriesNum++;
+			}
+			if (ReadVCALText(Line, "TEL;PAGER",	      Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;PAGER;VOICE",	Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER;VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=PAGER,VOICE",   Buff, false)) {
+				if (Buff[1] == '+') {
+					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
+				}
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
+				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
+				Pbk->EntriesNum++;
+			}
+			if (ReadVCALText(Line, "TEL;MSG",	      Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;MSG;VOICE",	Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG;VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=MSG,VOICE",   Buff, false)) {
+				if (Buff[1] == '+') {
+					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
+				}
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
+				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
+				Pbk->EntriesNum++;
+			}
+			/* FAX + VOICE looks like nonsense */
 			if (ReadVCALText(Line, "TEL;FAX",	       Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;FAX",	  Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=FAX",	  Buff, false) ||
 			    ReadVCALText(Line, "TEL;FAX;VOICE",	 Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;FAX;VOICE",    Buff, false)) {
+			    ReadVCALText(Line, "TEL;TYPE=FAX;VOICE",    Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=FAX;TYPE=VOICE",    Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=FAX,VOICE",    Buff, false)) {
 				if (Buff[1] == '+') {
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
@@ -457,9 +665,11 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 				Pbk->EntriesNum++;
 			}
 			if (ReadVCALText(Line, "TEL;HOME",	      Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;HOME",	 Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=HOME",	 Buff, false) ||
 			    ReadVCALText(Line, "TEL;HOME;VOICE",	Buff, false) ||
-			    ReadVCALText(Line, "TEL;PREF;HOME;VOICE",   Buff, false)) {
+			    ReadVCALText(Line, "TEL;TYPE=HOME,VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=HOME;TYPE=VOICE",   Buff, false) ||
+			    ReadVCALText(Line, "TEL;TYPE=HOME;VOICE",   Buff, false)) {
 				if (Buff[1] == '+') {
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
@@ -482,8 +692,6 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 			if (ReadVCALText(Line, "LABEL", Buff, false) ||
 			    ReadVCALText(Line, "ADR", Buff, false) ||
 			    ReadVCALText(Line, "ADR;HOME", Buff, false)) {
-				if (address) continue;
-				address = true;
 				pos = 0;
 				s = VCALGetTextPart(Buff, &pos); /* PO box, ignore for now */
 				if (s == NULL) {
@@ -528,10 +736,54 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 					Pbk->EntriesNum++;
 				}
 			}
+			if (ReadVCALText(Line, "ADR;WORK", Buff, false)) {
+				pos = 0;
+				s = VCALGetTextPart(Buff, &pos); /* PO box, ignore for now */
+				if (s == NULL) {
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkPostal;
+					Pbk->EntriesNum++;
+				} else {
+					s = VCALGetTextPart(Buff, &pos); /* Don't know ... */
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkStreetAddress;
+					Pbk->EntriesNum++;
+					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkCity;
+					Pbk->EntriesNum++;
+					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkState;
+					Pbk->EntriesNum++;
+					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkZip;
+					Pbk->EntriesNum++;
+					if (Pbk->EntriesNum == GSM_PHONEBOOK_ENTRIES) return ERR_MOREMEMORY;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkCountry;
+					Pbk->EntriesNum++;
+				}
+			}
 			if (ReadVCALText(Line, "EMAIL", Buff, false) ||
-			    ReadVCALText(Line, "EMAIL;PREF", Buff, false) ||
-			    ReadVCALText(Line, "EMAIL;INTERNET", Buff, false) ||
-			    ReadVCALText(Line, "EMAIL;INTERNET;PREF", Buff, false)) {
+			    ReadVCALText(Line, "EMAIL;TYPE=OTHER", Buff, false) ||
+			    ReadVCALText(Line, "EMAIL;INTERNET", Buff, false)) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Email;
 				Pbk->EntriesNum++;
@@ -551,6 +803,16 @@ GSM_Error GSM_DecodeVCARD(unsigned char *Buffer, int *Pos, GSM_MemoryEntry *Pbk,
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Company;
 				Pbk->EntriesNum++;
 			}
+			if (ReadVCALText(Line, "NICKNAME", Buff, false)) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_NickName;
+				Pbk->EntriesNum++;
+			}
+			if (ReadVCALText(Line, "FN", Buff, false)) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_FormalName;
+				Pbk->EntriesNum++;
+			}
 			if (ReadVCALText(Line, "CATEGORIES", Buff, false)) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].Number = -1;
