@@ -4,16 +4,14 @@
 #include <string.h>
 #include <time.h>
 
-#include "../../gsmcomon.h"
+#include <gammu-unicode.h>
+#include <gammu-debug.h>
+
 #include "../../misc/coding/coding.h"
-#include "../gsmcal.h"
-#include "../gsmpbk.h"
+#include "../../misc/misc.h"
 #include "../gsmlogo.h"
-#include "../gsmring.h"
-#include "../gsmdata.h"
-#include "../gsmnet.h"
-#include "gsmsms.h"
 #include "gsmmulti.h"
+#include "../gsmring.h"
 
 /* EMS Developers' Guidelines from www.sonyericsson.com
  * docs from Alcatel
@@ -22,16 +20,19 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				    GSM_MultiSMSMessage 	*SMS,
 				    GSM_UDH			UDHType)
 {
-	unsigned char		Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
-	int 			i,UsedText,j,Length,Width,Height,z,x,y;
+	unsigned char		Buffer[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
+	int			i,j,z;
+	long long			Length;
+	size_t EncodeLength;
 	unsigned int		Len;
-	int 			Used,FreeText,FreeBytes,Width2,CopiedText,CopiedSMSText;
+	size_t FreeText,FreeBytes,UsedText,Width,Height,x,y,Width2;
+	size_t 			Used,CopiedText,CopiedSMSText;
 	unsigned char		UDHID;
 	GSM_Bitmap		Bitmap,Bitmap2;
 	GSM_Ringtone		Ring;
 	GSM_Coding_Type 	Coding 	= SMS_Coding_Default_No_Compression;
 	GSM_Phone_Bitmap_Types	BitmapType;
-	MultiPartSMSEntry 	*Entry;
+	GSM_MultiPartSMSEntry 	*Entry;
 	bool			start;
 	GSM_DateTime		Date;
 
@@ -42,7 +43,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	if (Info->UnicodeCoding) Coding = SMS_Coding_Unicode_No_Compression;
 
 	/* Cleaning on the start */
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		GSM_SetDefaultSMSData(&SMS->SMS[i]);
 		SMS->SMS[i].UDH.Type = UDHType;
 		GSM_EncodeUDHHeader(&SMS->SMS[i].UDH);
@@ -93,7 +94,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				}
 				Len += CopiedText;
 				if (Len == UnicodeLength(Entry->Buffer)) break;
-				dbgprintf("%u %zu\n",Len,UnicodeLength(Entry->Buffer));
+				dbgprintf("%u " SIZE_T_FORMAT "\n",Len,UnicodeLength(Entry->Buffer));
 			}
 			break;
 		case SMS_EMSPredefinedSound:
@@ -120,26 +121,26 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,4,true,&UsedText,&CopiedText,&CopiedSMSText);
 			}
 
-			Length = 128; /* 128 bytes is maximal length from specs */
+			EncodeLength = 128; /* 128 bytes is maximal length from specs */
 			switch (Entry->ID) {
 			case SMS_EMSSound10:
-				Entry->RingtoneNotes = GSM_EncodeEMSSound(*Entry->Ringtone, Buffer+3, &Length, 1.0, true);
+				Entry->RingtoneNotes = GSM_EncodeEMSSound(*Entry->Ringtone, Buffer+3, &EncodeLength, GSM_Ring_IMelody10, true);
 				break;
 			case SMS_EMSSound12:
-				Entry->RingtoneNotes = GSM_EncodeEMSSound(*Entry->Ringtone, Buffer+3, &Length, 1.2, true);
+				Entry->RingtoneNotes = GSM_EncodeEMSSound(*Entry->Ringtone, Buffer+3, &EncodeLength, GSM_Ring_IMelody12, true);
 				break;
 			case SMS_EMSSonyEricssonSound:
-				Entry->RingtoneNotes = GSM_EncodeEMSSound(*Entry->Ringtone, Buffer+3, &Length, 0, true);
+				Entry->RingtoneNotes = GSM_EncodeEMSSound(*Entry->Ringtone, Buffer+3, &EncodeLength, GSM_Ring_NoHeader, true);
 				break;
 			default:
 				break;
 			}
 
 			Buffer[0] = 0x0C;	/* ID for EMS sound 	*/
-			Buffer[1] = Length+1;	/* Length of rest 	*/
+			Buffer[1] = EncodeLength+1;	/* Length of rest 	*/
 			Buffer[2] = 0x00; 	/* Position in EMS msg 	*/
-			GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,Length+3,true,&UsedText,&CopiedText,&CopiedSMSText);
-			SMS->SMS[SMS->Number].UDH.Text[SMS->SMS[SMS->Number].UDH.Length-Length-1] = UsedText;
+			GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,EncodeLength+3,true,&UsedText,&CopiedText,&CopiedSMSText);
+			SMS->SMS[SMS->Number].UDH.Text[SMS->SMS[SMS->Number].UDH.Length-EncodeLength-1] = UsedText;
 			break;
 		case SMS_EMSSonyEricssonSoundLong:
 		case SMS_EMSSound10Long:
@@ -147,16 +148,16 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			Ring = *Entry->Ringtone;
 
 			/* First check if we can use classic format */
-			Length = 128; /* 128 bytes is maximal length from specs */
+			EncodeLength = 128; /* 128 bytes is maximal length from specs */
 			switch (Entry->ID) {
 			case SMS_EMSSound10Long:
-				Entry->RingtoneNotes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 1.0, true);
+				Entry->RingtoneNotes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_IMelody10, true);
 				break;
 			case SMS_EMSSound12Long:
-				Entry->RingtoneNotes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 1.2, true);
+				Entry->RingtoneNotes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_IMelody12, true);
 				break;
 			case SMS_EMSSonyEricssonSoundLong:
-				Entry->RingtoneNotes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 0, true);
+				Entry->RingtoneNotes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_NoHeader, true);
 				break;
 			default:
 				break;
@@ -171,10 +172,10 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				}
 
 				Buffer[0] = 0x0C;	/* ID for EMS sound 	*/
-				Buffer[1] = Length+1;	/* Length of rest 	*/
+				Buffer[1] = EncodeLength+1;	/* Length of rest 	*/
 				Buffer[2] = 0x00; 	/* Position in EMS msg 	*/
-				GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,Length+3,true,&UsedText,&CopiedText,&CopiedSMSText);
-				SMS->SMS[SMS->Number].UDH.Text[SMS->SMS[SMS->Number].UDH.Length-Length-1] = UsedText;
+				GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,EncodeLength+3,true,&UsedText,&CopiedText,&CopiedSMSText);
+				SMS->SMS[SMS->Number].UDH.Text[SMS->SMS[SMS->Number].UDH.Length-EncodeLength-1] = UsedText;
 				break;
 			}
 
@@ -191,6 +192,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			Used 		= 0;
 			FreeBytes 	= 0;
 			start		= true;
+			EncodeLength = Length;
 			while (1) {
 				if (FreeBytes != 0) {
 					z = 0;
@@ -200,17 +202,17 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 					}
 					Ring.NoteTone.NrCommands -= FreeBytes;
 					if (Ring.NoteTone.NrCommands == 0) break;
-					Length = 128; /* 128 bytes is maximal length from specs */
+					EncodeLength = 128; /* 128 bytes is maximal length from specs */
 				}
 				switch (Entry->ID) {
 				case SMS_EMSSound10Long:
-					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 1.0, start);
+					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_IMelody10, true);
 					break;
 				case SMS_EMSSound12Long:
-					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 1.2, start);
+					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_IMelody12, true);
 					break;
 				case SMS_EMSSonyEricssonSoundLong:
-					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 0, start);
+					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_NoHeader, true);
 					break;
 				default:
 					break;
@@ -218,7 +220,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				start = false;
 				Used++;
 			}
-			dbgprintf("Used SMS: %i\n",Used);
+			dbgprintf("Used SMS: " SIZE_T_FORMAT "\n",Used);
 
 			if (Entry->Protected) {
 				Buffer[0] = 0x17;   /* ID for ODI 		   */
@@ -245,6 +247,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			/* Saving */
 			FreeBytes = 0;
 			start	  = true;
+			EncodeLength = Length;
 			while (1) {
 				if (FreeBytes != 0) {
 					z = 0;
@@ -254,26 +257,26 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 					}
 					Ring.NoteTone.NrCommands -= FreeBytes;
 					if (Ring.NoteTone.NrCommands == 0) break;
-					Length = 128; /* 128 bytes is maximal length from specs */
+					EncodeLength = 128; /* 128 bytes is maximal length from specs */
 				}
 				switch (Entry->ID) {
 				case SMS_EMSSound10Long:
-					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 1.0, start);
+					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_IMelody10, true);
 					break;
 				case SMS_EMSSound12Long:
-					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 1.2, start);
+					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_IMelody12, true);
 					break;
 				case SMS_EMSSonyEricssonSoundLong:
-					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &Length, 0, start);
+					FreeBytes = GSM_EncodeEMSSound(Ring, Buffer+3, &EncodeLength, GSM_Ring_NoHeader, true);
 					break;
 				default:
 					break;
 				}
 				Buffer[0] = 0x0C;	/* ID for EMS sound 	*/
-				Buffer[1] = Length+1;	/* Length of rest 	*/
+				Buffer[1] = EncodeLength+1;	/* Length of rest 	*/
 				Buffer[2] = 0x00; 	/* Position in EMS msg 	*/
-				GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,Length+3,true,&UsedText,&CopiedText,&CopiedSMSText);
-				SMS->SMS[SMS->Number].UDH.Text[SMS->SMS[SMS->Number].UDH.Length-Length-1] = UsedText;
+				GSM_AddSMS_Text_UDH(SMS,Coding,Buffer,EncodeLength+3,true,&UsedText,&CopiedText,&CopiedSMSText);
+				SMS->SMS[SMS->Number].UDH.Text[SMS->SMS[SMS->Number].UDH.Length-EncodeLength-1] = UsedText;
 				start = false;
 			}
 
@@ -386,7 +389,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			while (FreeBytes != Width) {
 				Width2 = 8;
 				while (FreeBytes + Width2 != Width) {
-					if (PHONE_GetBitmapSize(BitmapType,Width2+8,Height) > Length) break;
+					if (PHONE_GetBitmapSize(BitmapType,Width2+8,Height) > (size_t)Length) break;
 
 					Width2 = Width2 + 8;
 				}
@@ -394,7 +397,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				Length 		= 128;
 				Used ++;
 			}
-			dbgprintf("Used SMS: %i\n",Used);
+			dbgprintf("Used SMS: " SIZE_T_FORMAT "\n",Used);
 
 			if (Entry->Protected) {
 				Buffer[0] = 0x17;   /* ID for ODI 		   */
@@ -425,7 +428,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			while (FreeBytes != Width) {
 				Width2 = 8;
 				while (FreeBytes + Width2 != Width) {
-					if (PHONE_GetBitmapSize(BitmapType,Width2+8,Height) > Length) break;
+					if (PHONE_GetBitmapSize(BitmapType,Width2+8,Height) > (size_t)Length) break;
 
 					Width2 = Width2 + 8;
 				}
@@ -530,7 +533,7 @@ GSM_Error GSM_EncodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 	for (i=0;i<SMS->Number;i++) {
 		dbgprintf("UDH length %i\n",SMS->SMS[i].UDH.Length);
 		DumpMessage(&di, SMS->SMS[i].UDH.Text, SMS->SMS[i].UDH.Length);
-		dbgprintf("SMS length %zu\n",UnicodeLength(SMS->SMS[i].Text)*2);
+		dbgprintf("SMS length " SIZE_T_FORMAT "\n",UnicodeLength(SMS->SMS[i].Text)*2);
 		DumpMessage(&di, SMS->SMS[i].Text, UnicodeLength(SMS->SMS[i].Text)*2);
 	}
 #endif
@@ -580,12 +583,13 @@ static bool AddEMSText(GSM_SMSMessage *SMS, GSM_MultiPartSMSInfo *Info, int *Pos
 bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 			       GSM_MultiSMSMessage 	*SMS)
 {
-	int  			i, w, Pos, z, UPI = 1, width, height;
+	int  			i, w, Pos, UPI = 1, width, height;
+	size_t			z;
  	bool 			RetVal = false, NewPicture = true;
 	GSM_Phone_Bitmap_Types 	BitmapType;
 	GSM_Bitmap		Bitmap,Bitmap2;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		Info->Entries[i].ID	   = 0;
 	}
 
@@ -593,7 +597,7 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 		Pos 	= 0;
 		w	= 1;
 		while (w < SMS->SMS[i].UDH.Length) {
-			if (Info->EntriesNum + 1 == MAX_MULTI_SMS) {
+			if (Info->EntriesNum + 1 == GSM_MAX_MULTI_SMS) {
 				dbgprintf("Couldn't parse SMS, contains too many EMS parts!\n");
 				return false;
 			}
@@ -644,9 +648,11 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 				Info->Entries[Info->EntriesNum].ID 	= SMS_EMSPredefinedSound;
 				RetVal = true;
 				break;
-//			case 0x0C:
-//				dbgprintf("UDH part - EMS sound\n");
-//				break;
+#if 0
+			case 0x0C:
+				dbgprintf("UDH part - EMS sound\n");
+				break;
+#endif
 			case 0x0D:
 				dbgprintf("UDH part - default EMS animation\n");
 				if (SMS->SMS[i].UDH.Text[w+2] > Pos) {
@@ -718,17 +724,12 @@ bool GSM_DecodeEMSMultiPartSMS(GSM_MultiPartSMSInfo 	*Info,
 					z = Pos;
 					if (!AddEMSText(&SMS->SMS[i], Info, &Pos, SMS->SMS[i].UDH.Text[w+2]-z)) return false;
 				}
-				if (NewPicture) {
-					(Info->EntriesNum)++;
-					Info->Entries[Info->EntriesNum].Bitmap->Number = 0;
-					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapWidth  = 0;
-					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapHeight = 0;
-				}
 				Bitmap.BitmapWidth  = SMS->SMS[i].UDH.Text[w+3]*8;
 				Bitmap.BitmapHeight = SMS->SMS[i].UDH.Text[w+4];
 				Info->Entries[Info->EntriesNum].Bitmap = (GSM_MultiBitmap *)malloc(sizeof(GSM_MultiBitmap));
 				if (Info->Entries[Info->EntriesNum].Bitmap == NULL) return false;
 				if (NewPicture) {
+					Info->Entries[Info->EntriesNum].Bitmap->Number = 0;
 					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapWidth  = Bitmap.BitmapWidth;
 					Info->Entries[Info->EntriesNum].Bitmap->Bitmap[0].BitmapHeight = Bitmap.BitmapHeight;
 					PHONE_DecodeBitmap(GSM_EMSVariablePicture,
