@@ -4,17 +4,17 @@
 #include <string.h>
 #include <time.h>
 
-#include "../../gsmcomon.h"
-#include "../../misc/coding/coding.h"
-#include "../gsmcal.h"
-#include "../gsmpbk.h"
-#include "../gsmlogo.h"
+#include <gammu-unicode.h>
+#include <gammu-debug.h>
+
+#include "gsmmulti.h"
 #include "../gsmring.h"
+#include "../gsmlogo.h"
+#include "../../misc/coding/coding.h"
+#include "../../misc/misc.h"
+#include "gsmems.h"
 #include "../gsmdata.h"
 #include "../gsmnet.h"
-#include "gsmsms.h"
-#include "gsmmulti.h"
-#include "gsmems.h"
 
 /* ----------------- Splitting SMS into parts ------------------------------ */
 
@@ -34,9 +34,9 @@ unsigned char GSM_MakeSMSIDFromTime(void)
 	return retval;
 }
 
-void GSM_Find_Free_Used_SMS2(GSM_Coding_Type Coding,GSM_SMSMessage SMS, int *UsedText, int *FreeText, int *FreeBytes)
+void GSM_Find_Free_Used_SMS2(GSM_Coding_Type Coding,GSM_SMSMessage SMS, size_t *UsedText, size_t *FreeText, size_t *FreeBytes)
 {
-	int UsedBytes;
+	size_t UsedBytes;
 
 	switch (Coding) {
 	case SMS_Coding_Default_No_Compression:
@@ -60,19 +60,19 @@ void GSM_Find_Free_Used_SMS2(GSM_Coding_Type Coding,GSM_SMSMessage SMS, int *Use
 	default:
 		break;
 	}
-	dbgprintf("UDH len %i, UsedBytes %i, FreeText %i, UsedText %i, FreeBytes %i\n",SMS.UDH.Length,UsedBytes,*FreeText,*UsedText,*FreeBytes);
+	dbgprintf("UDH len %i, UsedBytes " SIZE_T_FORMAT ", FreeText " SIZE_T_FORMAT ", UsedText " SIZE_T_FORMAT ", FreeBytes " SIZE_T_FORMAT "\n",SMS.UDH.Length,UsedBytes,*FreeText,*UsedText,*FreeBytes);
 }
 
 GSM_Error GSM_AddSMS_Text_UDH(GSM_MultiSMSMessage 	*SMS,
 		      		GSM_Coding_Type		Coding,
 		      		char 			*Buffer,
-		      		int			BufferLen,
+		      		size_t			BufferLen,
 		      		bool 			UDH,
-		      		int 			*UsedText,
-		      		int			*CopiedText,
-		      		int			*CopiedSMSText)
+		      		size_t 			*UsedText,
+		      		size_t			*CopiedText,
+		      		size_t			*CopiedSMSText)
 {
-	int FreeText,FreeBytes,Copy,i,j;
+	size_t FreeText,FreeBytes,Copy,i,j;
 
 	dbgprintf("Checking used\n");
 	GSM_Find_Free_Used_SMS2(Coding,SMS->SMS[SMS->Number], UsedText, &FreeText, &FreeBytes);
@@ -92,7 +92,7 @@ GSM_Error GSM_AddSMS_Text_UDH(GSM_MultiSMSMessage 	*SMS,
 		SMS->SMS[SMS->Number].UDH.Length  	+= BufferLen;
 		SMS->SMS[SMS->Number].UDH.Text[0] 	+= BufferLen;
 		SMS->SMS[SMS->Number].UDH.Type 		=  UDH_UserUDH;
-		dbgprintf("UDH added %i\n",BufferLen);
+		dbgprintf("UDH added " SIZE_T_FORMAT "\n",BufferLen);
 	} else {
 		dbgprintf("Adding text\n");
 		if (FreeText == 0) {
@@ -102,14 +102,14 @@ GSM_Error GSM_AddSMS_Text_UDH(GSM_MultiSMSMessage 	*SMS,
 		}
 
 		Copy = FreeText;
-		dbgprintf("copy %i\n",Copy);
+		dbgprintf("copy " SIZE_T_FORMAT "\n",Copy);
 		if (BufferLen < Copy) Copy = BufferLen;
-		dbgprintf("copy %i\n",Copy);
+		dbgprintf("copy " SIZE_T_FORMAT "\n",Copy);
 
 		switch (Coding) {
 		case SMS_Coding_Default_No_Compression:
 			FindDefaultAlphabetLen(Buffer,&i,&j,FreeText);
-			dbgprintf("def length %i %i\n",i,j);
+			dbgprintf("def length " SIZE_T_FORMAT " " SIZE_T_FORMAT "\n",i,j);
 			SMS->SMS[SMS->Number].Text[UnicodeLength(SMS->SMS[SMS->Number].Text)*2+i*2]   = 0;
 			SMS->SMS[SMS->Number].Text[UnicodeLength(SMS->SMS[SMS->Number].Text)*2+i*2+1] = 0;
 			memcpy(SMS->SMS[SMS->Number].Text+UnicodeLength(SMS->SMS[SMS->Number].Text)*2,Buffer,i*2);
@@ -143,13 +143,14 @@ GSM_Error GSM_AddSMS_Text_UDH(GSM_MultiSMSMessage 	*SMS,
 
 void GSM_MakeMultiPartSMS(GSM_MultiSMSMessage	*SMS,
 			  unsigned char		*MessageBuffer,
-			  int			MessageLength,
+			  size_t		MessageLength,
 			  GSM_UDH		UDHType,
 			  GSM_Coding_Type	Coding,
 			  int			Class,
 			  unsigned char		ReplaceMessage)
 {
-	int 		j,Len,UsedText,CopiedText,CopiedSMSText;
+	size_t 		Len,UsedText,CopiedText,CopiedSMSText;
+	int		j;
 	unsigned char 	UDHID;
 	GSM_DateTime 	Date;
 
@@ -168,9 +169,9 @@ void GSM_MakeMultiPartSMS(GSM_MultiSMSMessage	*SMS,
 			GSM_AddSMS_Text_UDH(SMS,Coding,MessageBuffer+Len*2,MessageLength - Len,false,&UsedText,&CopiedText,&CopiedSMSText);
 		}
 		Len += CopiedText;
-		dbgprintf("%i %i\n",Len,MessageLength);
+		dbgprintf(SIZE_T_FORMAT " " SIZE_T_FORMAT "\n",Len,MessageLength);
 		if (Len == MessageLength) break;
-		if (SMS->Number == MAX_MULTI_SMS) break;
+		if (SMS->Number == GSM_MAX_MULTI_SMS) break;
 		SMS->Number++;
 	}
 
@@ -190,14 +191,14 @@ void GSM_MakeMultiPartSMS(GSM_MultiSMSMessage	*SMS,
 }
 
 /* Calculates number of SMS and number of left chars in SMS */
-void GSM_SMSCounter(int 		MessageLength,
+void GSM_SMSCounter(size_t 		MessageLength,
 		    unsigned char 	*MessageBuffer,
 		    GSM_UDH	 	UDHType,
 		    GSM_Coding_Type 	Coding,
 		    int 		*SMSNum,
-		    int 		*CharsLeft)
+		    size_t 		*CharsLeft)
 {
-	int			UsedText,FreeBytes;
+	size_t			UsedText,FreeBytes;
 	GSM_MultiSMSMessage 	MultiSMS;
 
 	MultiSMS.Number = 0;
@@ -208,9 +209,9 @@ void GSM_SMSCounter(int 		MessageLength,
 
 /* Nokia Smart Messaging 3.0 */
 static void GSM_EncodeSMS30MultiPartSMS(GSM_MultiPartSMSInfo *Info,
-					char *Buffer, int *Length)
+					char *Buffer, size_t *Length)
 {
-	int len;
+	size_t len;
 
 	/*SM version. Here 3.0*/
 	Buffer[(*Length)++] = 0x30;
@@ -272,15 +273,15 @@ static void GSM_EncodeSMS30MultiPartSMS(GSM_MultiPartSMSInfo *Info,
 /* Alcatel docs from www.alcatel.com/wap/ahead */
 GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 					unsigned char 		*Data,
-					int			Len,
+					size_t			Len,
 					unsigned char		*Name,
-					int			Type)
+					size_t			Type)
 {
 	unsigned char 	buff[100],UDHID;
-	int 		p,i;
+	size_t		i, p;
 	GSM_UDHHeader	MyUDH;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		GSM_SetDefaultSMSData(&SMS->SMS[i]);
 		SMS->SMS[i].UDH.Type    = UDH_UserUDH;
 		SMS->SMS[i].UDH.Text[1] = 0x80;	/* Alcatel */
@@ -294,7 +295,7 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 		SMS->SMS[i].UDH.Text[0] = 6 + SMS->SMS[i].UDH.Text[3];
 		SMS->SMS[i].UDH.Length  = SMS->SMS[i].UDH.Text[0] + 1;
 
-		if (Len > 140 - SMS->SMS[i].UDH.Length) {
+		if (Len > (size_t)(140 - SMS->SMS[i].UDH.Length)) {
 			MyUDH.Type = UDH_ConcatenatedMessages;
 			GSM_EncodeUDHHeader(&MyUDH);
 
@@ -321,7 +322,7 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 	/* Linked sms UDH */
 	if (SMS->Number != 1) {
 		UDHID = GSM_MakeSMSIDFromTime();
-		for (i=0;i<SMS->Number;i++) {
+		for (i = 0; i < (size_t)SMS->Number; i++) {
 			SMS->SMS[i].UDH.Text[SMS->SMS[i].UDH.Length-3] = UDHID;
 			SMS->SMS[i].UDH.Text[SMS->SMS[i].UDH.Length-2] = SMS->Number;
 			SMS->SMS[i].UDH.Text[SMS->SMS[i].UDH.Length-1] = i+1;
@@ -335,9 +336,11 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 			    	 GSM_MultiSMSMessage		*SMS)
 {
-	unsigned char	Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
-	unsigned char	Buffer2[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
-	int		Length = 0,smslen,i, Class = -1, j,p;
+	unsigned char	Buffer[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
+	unsigned char	Buffer2[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
+	int		i, Class = -1, j;
+	size_t p;
+	size_t Length = 0, smslen;
 	GSM_Error	error;
 	GSM_Coding_Type Coding 	= SMS_Coding_8bit;
 	GSM_UDH		UDH	= UDH_NoUDH;
@@ -348,7 +351,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 	SMS->Number = 0;
 
 	if (Info->Entries[0].ID == SMS_AlcatelSMSTemplateName) {
-		Buffer[Length++] = 0x00; //number of elements
+		Buffer[Length++] = 0x00; /* number of elements */
 		for (i=1;i<Info->EntriesNum;i++) {
 		switch (Info->Entries[i].ID) {
 		case SMS_EMSSound10:
@@ -363,27 +366,27 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 			break;
 		case SMS_EMSPredefinedSound:
 			Buffer[0]++;
-			Buffer[Length++] 	= 0x01; 	//type of data
-			Buffer[Length++] 	= 1 % 256;	//len
-			Buffer[Length++] 	= 1 / 256;      //len
+			Buffer[Length++] 	= 0x01; 	/* type of data */
+			Buffer[Length++] 	= 1 % 256;	/* len */
+			Buffer[Length++] 	= 1 / 256;      /* len */
 			Buffer[Length++] 	= Info->Entries[i].Number;
 			break;
 		case SMS_EMSPredefinedAnimation:
 			Buffer[0]++;
-			Buffer[Length++] 	= 0x02; 	//type of data
-			Buffer[Length++] 	= 1 % 256;	//len
-			Buffer[Length++] 	= 1 / 256;      //len
+			Buffer[Length++] 	= 0x02; 	/* type of data */
+			Buffer[Length++] 	= 1 % 256;	/* len */
+			Buffer[Length++] 	= 1 / 256;      /* len */
 			Buffer[Length++] 	= Info->Entries[i].Number;
 			break;
 		case SMS_ConcatenatedTextLong:
 			Buffer[0]++;
 			p 	= UnicodeLength(Info->Entries[i].Buffer);
 			EncodeDefault(Buffer2, Info->Entries[i].Buffer, &p, true, NULL);
-			Buffer[Length++]   	= 0x00; 	//type of data
+			Buffer[Length++]   	= 0x00; 	/* type of data */
 			Length 			= Length + 2;
 			smslen			= GSM_PackSevenBitsToEight(0, Buffer2, Buffer+Length, p);
-			Buffer[Length-2] 	= smslen % 256; //len
-			Buffer[Length-1] 	= smslen / 256; //len
+			Buffer[Length-2] 	= smslen % 256; /* len */
+			Buffer[Length-1] 	= smslen / 256; /* len */
 			Length 			= Length + smslen;
 			break;
 		default:
@@ -652,7 +655,9 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		if (Info->UnicodeCoding) {
 			Coding = SMS_Coding_Unicode_No_Compression;
 			Length = UnicodeLength(Info->Entries[0].Buffer);
-			if (Length>(140-UDHHeader.Length)/2) Length = (140-UDHHeader.Length)/2;
+			if (Length > (size_t)(140 - UDHHeader.Length) / 2) {
+				Length = (140 - UDHHeader.Length) / 2;
+			}
 		} else {
 			Coding = SMS_Coding_Default_No_Compression;
 			FindDefaultAlphabetLen(Info->Entries[0].Buffer,&Length,&smslen,(GSM_MAX_8BIT_SMS_LENGTH-UDHHeader.Length)*8/7);
@@ -673,10 +678,10 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		}
 #endif
 		Info->UnicodeCoding = false;
-		for (smslen=0;smslen<(int)(UnicodeLength(Info->Entries[0].Buffer)*2);smslen++) {
+		for (smslen = 0; smslen < UnicodeLength(Info->Entries[0].Buffer) * 2; smslen++) {
 			if (Info->Entries[0].Buffer[smslen] != Buffer[smslen]) {
 				Info->UnicodeCoding = true;
-				dbgprintf("Setting to Unicode %i\n",smslen);
+				dbgprintf("Setting to Unicode " SIZE_T_FORMAT "\n",smslen);
 				break;
 			}
 		}
@@ -716,7 +721,7 @@ void GSM_ClearMultiPartSMSInfo(GSM_MultiPartSMSInfo *Info)
 {
 	int i;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		Info->Entries[i].Number		= 0;
 		Info->Entries[i].Ringtone	= NULL;
 		Info->Entries[i].Bitmap		= NULL;
@@ -753,7 +758,7 @@ void GSM_FreeMultiPartSMSInfo(GSM_MultiPartSMSInfo *Info)
 {
 	int i;
 
-	for (i=0;i<MAX_MULTI_SMS;i++) {
+	for (i=0;i<GSM_MAX_MULTI_SMS;i++) {
 		free(Info->Entries[i].Ringtone);
 		Info->Entries[i].Ringtone = NULL;
 		free(Info->Entries[i].Bitmap);
@@ -782,8 +787,8 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 			    bool			ems)
 {
 	int 			i, Length = 0;
-	unsigned int	j;
-	char			Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
+	unsigned int		j;
+	char			Buffer[GSM_MAX_SMS_LENGTH*2*GSM_MAX_MULTI_SMS];
 	bool 			emsexist = false;
 	GSM_SiemensOTASMSInfo	SiemensInfo;
 
@@ -996,7 +1001,8 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT, bool ems)
 {
 	bool			*INPUTSorted, copyit,OtherNumbers[GSM_SMS_OTHER_NUMBERS+1],wrong=false;
-	int			i,OUTPUTNum,j,z,w,m,p;
+	int			i,OUTPUTNum,z,w,m,p;
+	int			j;
 	GSM_SiemensOTASMSInfo	SiemensOTA,SiemensOTA2;
 
 	i = 0;
@@ -1068,7 +1074,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 			INPUTSorted[i]	= true;
 			j		= 1;
 			/* We're searching for other parts in sequence */
-			while (j!=SiemensOTA.PacketsNum) {
+			while (j!=(int)SiemensOTA.PacketsNum) {
 				z=0;
 				while(INPUT[z]!=NULL) {
 					/* This was sorted earlier or is not single */
@@ -1081,7 +1087,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 						continue;
 					}
 					if (SiemensOTA2.SequenceID != SiemensOTA.SequenceID ||
-					    SiemensOTA2.PacketNum != j+1 ||
+					    (int)SiemensOTA2.PacketNum != j+1 ||
 					    SiemensOTA2.PacketsNum != SiemensOTA.PacketsNum ||
 					    strcmp(SiemensOTA2.DataType,SiemensOTA.DataType) ||
 					    strcmp(SiemensOTA2.DataName,SiemensOTA.DataName)) {
@@ -1245,7 +1251,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 			INPUTSorted[i]	= true;
 			j		= 1;
 			/* We're searching for other parts in sequence */
-			while (j!=INPUT[i]->SMS[0].UDH.AllParts) {
+			while (j != INPUT[i]->SMS[0].UDH.AllParts) {
 				z=0;
 				while(INPUT[z]!=NULL) {
 					/* This was sorted earlier or is not single */
@@ -1280,9 +1286,9 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 						INPUT[z]->SMS[0].UDH.PartNumber,
 						INPUT[z]->SMS[0].UDH.AllParts);
 					if (INPUT[z]->SMS[0].UDH.ID8bit      != INPUT[i]->SMS[0].UDH.ID8bit	||
-					    INPUT[z]->SMS[0].UDH.ID16bit     != INPUT[i]->SMS[0].UDH.ID16bit	||
-					    INPUT[z]->SMS[0].UDH.AllParts    != INPUT[i]->SMS[0].UDH.AllParts 	||
-					    INPUT[z]->SMS[0].UDH.PartNumber  != j+1) {
+							INPUT[z]->SMS[0].UDH.ID16bit     != INPUT[i]->SMS[0].UDH.ID16bit	||
+							INPUT[z]->SMS[0].UDH.AllParts    != INPUT[i]->SMS[0].UDH.AllParts 	||
+							(INPUT[z]->SMS[0].UDH.PartNumber) != j + 1) {
 						z++;
 						continue;
 					}
