@@ -17,15 +17,13 @@
 #include <string.h>
 #include <time.h>
 
-#include "../../gsmcomon.h"
+#include <gammu-calendar.h>
+#include <gammu-message.h>
+#include <gammu-unicode.h>
+#include <gammu-debug.h>
+
 #include "../../misc/coding/coding.h"
-#include "../gsmcal.h"
-#include "../gsmpbk.h"
-#include "../gsmlogo.h"
-#include "../gsmring.h"
-#include "../gsmdata.h"
-#include "../gsmnet.h"
-#include "gsmsms.h"
+#include "../../misc/misc.h"
 
 /* User data headers */
 static GSM_UDHHeader UDHHeaders[] = {
@@ -99,14 +97,15 @@ static GSM_Error GSM_DecodeSMSDateTime(GSM_DateTime *DT, unsigned char *req)
 	DT->Second  = DecodeWithBCDAlphabet(req[5]);
 
 	/* Base for timezone is GMT. It's in quarters */
-	DT->Timezone=(10*(req[6]&0x07)+(req[6]>>4))/4;
+	DT->Timezone=(10*(req[6]&0x07)+(req[6]>>4))*3600/4;
 
 	if (req[6]&0x08) DT->Timezone = -DT->Timezone;
 
 	dbgprintf("Decoding date & time: ");
 	dbgprintf("%s %4d/%02d/%02d ", DayOfWeek(DT->Year, DT->Month, DT->Day),
 		DT->Year, DT->Month, DT->Day);
-	dbgprintf("%02d:%02d:%02d %02d00\n", DT->Hour, DT->Minute, DT->Second, DT->Timezone);
+	dbgprintf("%02d:%02d:%02d%+03i%02i\n", DT->Hour, DT->Minute, DT->Second,
+		DT->Timezone / 3600, abs((DT->Timezone % 3600) / 60));
 
 	return ERR_NONE;
 }
@@ -188,9 +187,53 @@ void GSM_DecodeUDHHeader(GSM_UDHHeader *UDH)
 #endif
 }
 
+GSM_Coding_Type GSM_GetMessageCoding(const char TPDCS) {
+
+	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
+	if ((TPDCS & 0xC0) == 0) {
+		/* bits 7..4 set to 00xx */
+		if ((TPDCS & 0xC) == 0xC) {
+			dbgprintf("WARNING: reserved alphabet value in TPDCS\n");
+		} else {
+			if (TPDCS == 0) 			return SMS_Coding_Default_No_Compression;
+			if ((TPDCS & 0x2C) == 0x00) 	return SMS_Coding_Default_No_Compression;
+			if ((TPDCS & 0x2C) == 0x20) 	return SMS_Coding_Default_Compression;
+			if ((TPDCS & 0x2C) == 0x08) 	return SMS_Coding_Unicode_No_Compression;
+			if ((TPDCS & 0x2C) == 0x28) 	return SMS_Coding_Unicode_Compression;
+		}
+	} else if ((TPDCS & 0xF0) >= 0x40 &&
+		   (TPDCS & 0xF0) <= 0xB0) {
+		/* bits 7..4 set to 0100 ... 1011 */
+		dbgprintf("WARNING: reserved coding group in TPDCS\n");
+	} else if (((TPDCS & 0xF0) == 0xC0) ||
+	      	   ((TPDCS & 0xF0) == 0xD0)) {
+		/* bits 7..4 set to 1100 or 1101 */
+		if ((TPDCS & 4) == 4) {
+			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
+		} else {
+			return SMS_Coding_Default_No_Compression;
+		}
+	} else if ((TPDCS & 0xF0) == 0xE0) {
+		/* bits 7..4 set to 1110 */
+		if ((TPDCS & 4) == 4) {
+			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
+		} else {
+			return SMS_Coding_Unicode_No_Compression;
+		}
+	} else if ((TPDCS & 0xF0) == 0xF0) {
+		/* bits 7..4 set to 1111 */
+		if ((TPDCS & 8) == 8) {
+			dbgprintf("WARNING: set reserved bit 3 in TPDCS\n");
+		} else {
+			if ((TPDCS & 4) == 0) return SMS_Coding_Default_No_Compression;
+		}
+	}
+	return 0;
+}
+
 GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMSMessageLayout Layout)
 {
-	int		off=0;	 	// length of the User Data Header
+	int		off=0;	 	/* length of the User Data Header */
 	int 		w,i,tmp=0;
 	unsigned char	output[161];
 
@@ -208,46 +251,6 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 		GSM_DecodeUDHHeader(&SMS->UDH);
 	}
 
-	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
-	if ((buffer[Layout.TPDCS] & 0xC0) == 0) {
-		/* bits 7..4 set to 00xx */
-		if ((buffer[Layout.TPDCS] & 0xC) == 0xC) {
-			dbgprintf("WARNING: reserved alphabet value in TPDCS\n");
-		} else {
-			if (buffer[Layout.TPDCS] == 0) 			SMS->Coding=SMS_Coding_Default_No_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x00) 	SMS->Coding=SMS_Coding_Default_No_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x20) 	SMS->Coding=SMS_Coding_Default_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x08) 	SMS->Coding=SMS_Coding_Unicode_No_Compression;
-			if ((buffer[Layout.TPDCS] & 0x2C) == 0x28) 	SMS->Coding=SMS_Coding_Unicode_Compression;
-		}
-	} else if ((buffer[Layout.TPDCS] & 0xF0) >= 0x40 &&
-		   (buffer[Layout.TPDCS] & 0xF0) <= 0xB0) {
-		/* bits 7..4 set to 0100 ... 1011 */
-		dbgprintf("WARNING: reserved coding group in TPDCS\n");
-	} else if (((buffer[Layout.TPDCS] & 0xF0) == 0xC0) ||
-	      	   ((buffer[Layout.TPDCS] & 0xF0) == 0xD0)) {
-		/* bits 7..4 set to 1100 or 1101 */
-		if ((buffer[Layout.TPDCS] & 4) == 4) {
-			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
-		} else {
-			SMS->Coding=SMS_Coding_Default_No_Compression;
-		}
-	} else if ((buffer[Layout.TPDCS] & 0xF0) == 0xE0) {
-		/* bits 7..4 set to 1110 */
-		if ((buffer[Layout.TPDCS] & 4) == 4) {
-			dbgprintf("WARNING: set reserved bit 2 in TPDCS\n");
-		} else {
-			SMS->Coding=SMS_Coding_Unicode_No_Compression;
-		}
-	} else if ((buffer[Layout.TPDCS] & 0xF0) == 0xF0) {
-		/* bits 7..4 set to 1111 */
-		if ((buffer[Layout.TPDCS] & 8) == 8) {
-			dbgprintf("WARNING: set reserved bit 3 in TPDCS\n");
-		} else {
-			if ((buffer[Layout.TPDCS] & 4) == 0) SMS->Coding=SMS_Coding_Default_No_Compression;
-		}
-	}
-
 	switch (SMS->Coding) {
 		case SMS_Coding_Default_No_Compression:
 			i = 0;
@@ -256,6 +259,11 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM
 				w=(i-off)%i;
 			} while (w<0);
 			SMS->Length=buffer[Layout.TPUDL] - (off*8 + w) / 7;
+			if (SMS->Length < 0) {
+				dbgprintf("No SMS text!\n");
+				SMS->Length = 0;
+				break;
+			}
 			tmp=GSM_UnpackEightBitsToSeven(w, buffer[Layout.TPUDL]-off, SMS->Length, buffer+(Layout.Text+off), output);
 			dbgprintf("7 bit SMS, length %i\n",SMS->Length);
 			DecodeDefault (SMS->Text, output, SMS->Length, true, NULL);
@@ -366,27 +374,13 @@ GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 	if (Layout.TPPID     != 255) dbgprintf("TPPID     : %02x %i\n",buffer[Layout.TPPID]    ,buffer[Layout.TPPID]);
 	if (Layout.TPUDL     != 255) dbgprintf("TPUDL     : %02x %i\n",buffer[Layout.TPUDL]    ,buffer[Layout.TPUDL]);
 	if (Layout.firstbyte != 255) dbgprintf("FirstByte : %02x %i\n",buffer[Layout.firstbyte],buffer[Layout.firstbyte]);
-	if (Layout.Text      != 255) dbgprintf("Text      : %02x %i\n",buffer[Layout.Text]     ,buffer[Layout.Text]);
+	if (Layout.Text      != 255 && Layout.TPUDL     != 255 && buffer[Layout.TPUDL] > 0) {
+		dbgprintf("Text      : %02x %i\n",buffer[Layout.Text]     ,buffer[Layout.Text]);
+	}
 #endif
 
-	SMS->UDH.Type 			= UDH_NoUDH;
-	SMS->Coding 			= SMS_Coding_8bit;
-	SMS->Length			= 0;
-	SMS->SMSC.Location		= 0;
-	SMS->SMSC.DefaultNumber[0]	= 0;
-	SMS->SMSC.DefaultNumber[1]	= 0;
-	SMS->SMSC.Number[0]		= 0;
-	SMS->SMSC.Number[1]		= 0;
-	SMS->SMSC.Name[0]		= 0;
-	SMS->SMSC.Name[1]		= 0;
-	SMS->SMSC.Validity.Format	= SMS_Validity_NotAvailable;
-	SMS->SMSC.Format		= SMS_FORMAT_Text;
-	SMS->Number[0]			= 0;
-	SMS->Number[1]			= 0;
-	SMS->OtherNumbersNum		= 0;
-	SMS->Name[0]			= 0;
-	SMS->Name[1]			= 0;
-	SMS->ReplyViaSameSMSC		= false;
+	GSM_SetDefaultReceivedSMSData(SMS);
+
 	if (Layout.SMSCNumber!=255) {
 		GSM_UnpackSemiOctetNumber(SMS->SMSC.Number,buffer+Layout.SMSCNumber,false);
 		dbgprintf("SMS center number : \"%s\"\n",DecodeUnicodeString(SMS->SMSC.Number));
@@ -399,7 +393,11 @@ GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 		GSM_UnpackSemiOctetNumber(SMS->Number,buffer+Layout.Number,true);
 		dbgprintf("Remote number : \"%s\"\n",DecodeUnicodeString(SMS->Number));
 	}
-	if (Layout.Text != 255 && Layout.TPDCS!=255 && Layout.TPUDL!=255) {
+	if (Layout.TPDCS != 255) {
+		/* Get message coding */
+		SMS->Coding = GSM_GetMessageCoding(buffer[Layout.TPDCS]);
+	}
+	if (Layout.Text != 255 && Layout.TPDCS!=255 && Layout.TPUDL!=255 && buffer[Layout.TPUDL] > 0) {
 		GSM_DecodeSMSFrameText(SMS, buffer, Layout);
 	}
 	if (Layout.DateTime != 255) {
@@ -482,8 +480,9 @@ static GSM_Error GSM_EncodeSMSDateTime(GSM_DateTime *DT, unsigned char *req)
 
 static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMSMessageLayout Layout)
 {
-	int	off = 0;	// length of the User Data Header
-	int	size = 0, size2 = 0, w,p;
+	int	off = 0;	/*  length of the User Data Header */
+	int	size = 0, size2 = 0, w;
+	size_t p;
 	char	buff[200];
 
 	if (SMS->UDH.Type!=UDH_NoUDH) {
@@ -500,7 +499,7 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 			/* the mask for the 8-bit data */
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
-			buffer[Layout.TPDCS] |= 0xf4;
+			buffer[Layout.TPDCS] |= (1 << 2);
 			memcpy(buffer+(Layout.Text+off), SMS->Text, SMS->Length);
 			size2 = size = SMS->Length+off;
 #ifdef DEBUG
@@ -528,7 +527,7 @@ static int GSM_EncodeSMSFrameText(GSM_SMSMessage *SMS, unsigned char *buffer, GS
 		case SMS_Coding_Unicode_No_Compression:
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
-			buffer[Layout.TPDCS] |= 0x08;
+			buffer[Layout.TPDCS] |= (1 << 3);
 			EncodeUnicodeSpecialNOKIAChars(buffer+(Layout.Text+off), SMS->Text, UnicodeLength(SMS->Text));
 			size=size2=UnicodeLength(buffer+(Layout.Text+off))*2+off;
 #ifdef DEBUG
@@ -593,7 +592,7 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 	/* Message Class*/
 	/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
 	if (Layout.TPDCS != 255) {
-		if (SMS->Class>=0 && SMS->Class<5) buffer[Layout.TPDCS] |= (240+SMS->Class);
+		if (SMS->Class>=0 && SMS->Class<5) buffer[Layout.TPDCS] |= SMS->Class | (1 << 4);
 		dbgprintf("SMS class %i\n",SMS->Class);
 	}
 
@@ -628,7 +627,7 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 
 	/* size is the length of the data in octets including UDH */
 	*length=GSM_EncodeSMSFrameText(SMS,buffer,Layout);
-//	if (*length == 0) return GE_UNKNOWN;
+/* 	if (*length == 0) return GE_UNKNOWN; */
 	*length += Layout.Text;
 
 	return ERR_NONE;
@@ -636,51 +635,59 @@ GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMS
 
 /* ----------------- Some help functions ----------------------------------- */
 
-void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
+void GSM_SetDefaultReceivedSMSData(GSM_SMSMessage *SMS)
 {
-	SMS->Class			= -1;
-	SMS->SMSC.Location		= 1;
-	SMS->SMSC.Name[0]		= 0;
-	SMS->SMSC.Name[1]		= 0;
-	SMS->SMSC.Number[0]		= 0;
-	SMS->SMSC.Number[1]		= 0;
-	SMS->SMSC.DefaultNumber[0]	= 0;
-	SMS->SMSC.DefaultNumber[1]	= 0;
-	SMS->SMSC.Format		= SMS_FORMAT_Text;
-	SMS->SMSC.Validity.Format	= SMS_Validity_RelativeFormat;
-	SMS->SMSC.Validity.Relative	= SMS_VALID_Max_Time;
-	SMS->ReplyViaSameSMSC		= false;
-	SMS->UDH.Type			= UDH_NoUDH;
+	SMS->UDH.Type 			= UDH_NoUDH;
 	SMS->UDH.Length			= 0;
 	SMS->UDH.Text[0] 		= 0;
 	SMS->UDH.ID8bit			= 0;
 	SMS->UDH.ID16bit		= 0;
 	SMS->UDH.PartNumber		= 0;
 	SMS->UDH.AllParts		= 0;
-	SMS->Coding			= SMS_Coding_8bit;
+	SMS->Coding 			= SMS_Coding_8bit;
+	SMS->Length			= 0;
+	SMS->SMSC.Location		= 0;
+	SMS->SMSC.DefaultNumber[0]	= 0;
+	SMS->SMSC.DefaultNumber[1]	= 0;
+	SMS->SMSC.Number[0]		= 0;
+	SMS->SMSC.Number[1]		= 0;
+	SMS->SMSC.Name[0]		= 0;
+	SMS->SMSC.Name[1]		= 0;
+	SMS->SMSC.Validity.Format	= SMS_Validity_NotAvailable;
+	SMS->SMSC.Validity.Relative	= SMS_VALID_Max_Time;
+	SMS->SMSC.Format		= SMS_FORMAT_Text;
+	SMS->Number[0]			= 0;
+	SMS->Number[1]			= 0;
+	SMS->OtherNumbersNum		= 0;
+	SMS->Name[0]			= 0;
+	SMS->Name[1]			= 0;
+	SMS->ReplyViaSameSMSC		= false;
+	SMS->Class			= 0;
 	SMS->Text[0] 			= 0;
 	SMS->Text[1] 			= 0;
-	SMS->PDU			= SMS_Submit;
 	SMS->RejectDuplicates		= false;
 	SMS->MessageReference		= 0;
 	SMS->ReplaceMessage		= 0;
-	SMS->Length			= 0;
 	SMS->DeliveryStatus		= 0;
-	SMS->ReplyViaSameSMSC		= 0;
-	SMS->Class			= 0;
-	SMS->MessageReference		= 0;
-	SMS->OtherNumbersNum		= 0;
+}
+
+void GSM_SetDefaultSMSData(GSM_SMSMessage *SMS)
+{
+	GSM_SetDefaultReceivedSMSData(SMS);
+
+	SMS->SMSC.Validity.Format	= SMS_Validity_RelativeFormat;
+	SMS->SMSC.Validity.Relative	= SMS_VALID_Max_Time;
+	SMS->SMSC.Location		= 1;
 
 	/* This part is required to save SMS */
 	SMS->State			= SMS_UnSent;
+	SMS->PDU			= SMS_Submit;
 	SMS->Location			= 0;
 	SMS->Memory			= 0;
 	SMS->Folder			= 0x02;	/*Outbox*/
 	SMS->InboxFolder		= false;
 	GSM_GetCurrentDateTime (&SMS->DateTime);
 	GSM_GetCurrentDateTime (&SMS->SMSCTime);
-	SMS->Name[0]			= 0;
-	SMS->Name[1]			= 0;
 }
 
 /**
@@ -748,8 +755,8 @@ bool GSM_DecodeSiemensOTASMS(GSM_SiemensOTASMSInfo	*Info,
 	if (SMS->UDH.Type != UDH_NoUDH) 	return false;
 	if (SMS->Length < 22) 			return false;
 
-	if (strncmp(SMS->Text,"//SEO",5)!=0) return false; //Siemens Exchange Object
-	if (SMS->Text[5]!=1) return false; //version 1
+	if (strncmp(SMS->Text,"//SEO",5)!=0) return false; /* Siemens Exchange Object */
+	if (SMS->Text[5]!=1) return false; /* version 1 */
 	Info->DataLen = SMS->Text[6] + SMS->Text[7]*256;
 	Info->SequenceID = SMS->Text[8] + SMS->Text[9]*256 +
 			 SMS->Text[10]*256*256 + SMS->Text[11]*256*256*256;
