@@ -16,6 +16,12 @@
 
 extern GSM_Reply_Function UserReplyFunctions4[];
 
+/**
+ * Helper define to check error code from fwrite.
+ */
+#define chk_fwrite(data, size, count, file) \
+	if (fwrite(data, size, count, file) != count) goto fail;
+
 /* ------- some usefull functions ----------------------------------------- */
 
 GSM_Error CheckDCT4Only()
@@ -24,20 +30,20 @@ GSM_Error CheckDCT4Only()
 
 	/* Checking if phone is DCT4 */
 #ifdef GSM_ENABLE_NOKIA3650
- 	if (strstr(N3650Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+ 	if (strstr(N3650Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA6510
- 	if (strstr(N6510Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+ 	if (strstr(N6510Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 #ifdef GSM_ENABLE_NOKIA3320
- 	if (strstr(N3320Phone.models, s.Phone.Data.ModelInfo->model) != NULL) found = true;
+ 	if (strstr(N3320Phone.models, gsm->Phone.Data.ModelInfo->model) != NULL) found = true;
 #endif
 	if (!found) return ERR_NOTSUPPORTED;
 
-	if (s.ConnectionType!=GCT_MBUS2	     && s.ConnectionType!=GCT_FBUS2      &&
-	    s.ConnectionType!=GCT_FBUS2DLR3  && s.ConnectionType!=GCT_PHONETBLUE &&
-	    s.ConnectionType!=GCT_IRDAPHONET && s.ConnectionType!=GCT_BLUEPHONET &&
-	    s.ConnectionType!=GCT_DKU5FBUS2  && s.ConnectionType!=GCT_DKU2PHONET) {
+	if (gsm->ConnectionType!=GCT_MBUS2	     && gsm->ConnectionType!=GCT_FBUS2      &&
+	    gsm->ConnectionType!=GCT_FBUS2DLR3  && gsm->ConnectionType!=GCT_PHONETBLUE &&
+	    gsm->ConnectionType!=GCT_IRDAPHONET && gsm->ConnectionType!=GCT_BLUEPHONET &&
+	    gsm->ConnectionType!=GCT_DKU5FBUS2  && gsm->ConnectionType!=GCT_DKU2PHONET) {
 		return ERR_OTHERCONNECTIONREQUIRED;
 	}
 	return ERR_NONE;
@@ -54,7 +60,7 @@ static void CheckDCT4()
 		break;
 	case ERR_OTHERCONNECTIONREQUIRED:
 		printf("%s\n", _("Can't do it with current phone protocol"));
-		GSM_TerminateConnection(&s);
+		GSM_TerminateConnection(gsm);
 		exit(-1);
 	default:
 		break;
@@ -248,7 +254,7 @@ static DCT4_Phone_Features DCT4PhoneFeatures[] = {
 	{"",		{{0,0}}}
 };
 
-static GSM_Error DCT4_ReplySetPPS(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplySetPPS(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf("%s\n", _("Setting done"));
 	return ERR_NONE;
@@ -260,13 +266,14 @@ void DCT4SetPhoneMenus(int argc, char *argv[])
 	unsigned char 	reqSet[200] = {
 		N7110_FRAME_HEADER,0x04,0x00,0x01,0x47,0x48,0x02,
 		0x00};		/* Number of changed features */
+	GSM_Error error;
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
 	while (DCT4PhoneFeatures[i].Model[0] != 0x00) {
-		if (!strcmp(DCT4PhoneFeatures[i].Model,s.Phone.Data.Model)) {
+		if (!strcmp(DCT4PhoneFeatures[i].Model,gsm->Phone.Data.Model)) {
 			j = 0;
 			while (DCT4PhoneFeatures[i].Features[j].Name != 0x00) {
 				z = 0;
@@ -294,31 +301,31 @@ void DCT4SetPhoneMenus(int argc, char *argv[])
 	reqSet[current++] = 0x00;
 	reqSet[current++] = 0x00;
 
-	error=GSM_WaitFor (&s, reqSet, current, 0x1b, 4, ID_User1);
+	error=GSM_WaitFor (gsm, reqSet, current, 0x1b, 4, ID_User1);
 	Print_Error(error);
 }
 
 DCT4_Phone_Tests DCT4Tests;
 
-static GSM_Error DCT4_ReplyTestsNames(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyTestsNames(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int i,pos;
 
 	DCT4Tests.Num   = msg.Buffer[5];
 	pos 		= 6;
 
-	smprintf(s,"%i names for phone tests received\n",msg.Buffer[5]);
+	smprintf(sm,"%i names for phone tests received\n",msg.Buffer[5]);
 	for (i=0;i<msg.Buffer[5];i++) {
 		strcpy(DCT4Tests.Tests[i].Name,msg.Buffer+pos+4);
 		DCT4Tests.Tests[i].ID = msg.Buffer[pos+2];
-		smprintf(s,"%x.\"%s\"\n",DCT4Tests.Tests[i].ID,DCT4Tests.Tests[i].Name);
+		smprintf(sm,"%x.\"%s\"\n",DCT4Tests.Tests[i].ID,DCT4Tests.Tests[i].Name);
 		pos+=msg.Buffer[pos+1];
 	}
 
 	return ERR_NONE;
 }
 
-static GSM_Error DCT4_ReplyTestsStartup(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyTestsStartup(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int 	 i,pos,j;
 	bool	 found;
@@ -341,13 +348,13 @@ static GSM_Error DCT4_ReplyTestsStartup(GSM_Protocol_Message msg, GSM_StateMachi
 	return ERR_NONE;
 }
 
-static GSM_Error DCT4_ReplyTestsStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyTestsStatus(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int i,pos,j;
 
 	pos = 6;
 
-	smprintf(s,"%i status entries for phone tests received\n",msg.Buffer[5]);
+	smprintf(sm,"%i status entries for phone tests received\n",msg.Buffer[5]);
 	for (i=0;i<msg.Buffer[5];i++) {
 		for (j=0;j<DCT4Tests.Num;j++) {
 			if (DCT4Tests.Tests[j].ID == msg.Buffer[pos+2]) {
@@ -374,6 +381,7 @@ static GSM_Error DCT4_ReplyTestsStatus(GSM_Protocol_Message msg, GSM_StateMachin
 void DCT4SelfTests(int argc, char *argv[])
 {
 	int 	      j;
+	GSM_Error error;
 	unsigned char GetDoneST[6]    = {0x00, 0x08, 0x01, 0x04, 0x01, 0x00};
 	unsigned char GetDoneST2[6]   = {0x00, 0x08, 0x02, 0x04, 0x02, 0x00};
 	unsigned char GetNames[6]     = {0x00, 0x08, 0x03, 0x06, 0x03, 0x00};
@@ -385,29 +393,29 @@ void DCT4SelfTests(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
 	if (answer_yes2("Run all tests now ?")) {
-		error=GSM_WaitFor (&s, RunALL, 6, 0x35, 4, ID_User1);
+		error=GSM_WaitFor (gsm, RunALL, 6, 0x35, 4, ID_User1);
 		Print_Error(error);
 	}
 
-	error=GSM_WaitFor (&s, GetNames, 6, 0x35, 4, ID_User1);
+	error=GSM_WaitFor (gsm, GetNames, 6, 0x35, 4, ID_User1);
 	Print_Error(error);
 
 	for (j=0;j<DCT4Tests.Num;j++) DCT4Tests.Tests[j].Startup = false;
 
-	error=GSM_WaitFor (&s, GetDoneST, 6, 0x35, 4, ID_User3);
+	error=GSM_WaitFor (gsm, GetDoneST, 6, 0x35, 4, ID_User3);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, GetDoneST2, 6, 0x35, 4, ID_User3);
+	error=GSM_WaitFor (gsm, GetDoneST2, 6, 0x35, 4, ID_User3);
 	Print_Error(error);
 
-	error=GSM_WaitFor (&s, GetStatus, 6, 0x35, 4, ID_User2);
+	error=GSM_WaitFor (gsm, GetStatus, 6, 0x35, 4, ID_User2);
 	Print_Error(error);
 }
 
-static GSM_Error DCT4_ReplyVibra(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyVibra(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 #ifdef DEBUG
 	switch (msg.Buffer[3]) {
@@ -418,7 +426,7 @@ static GSM_Error DCT4_ReplyVibra(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	return ERR_NONE;
 }
 
-static GSM_Error DCT4EnableVibra(GSM_StateMachine *s, bool enable)
+static GSM_Error DCT4EnableVibra(GSM_StateMachine *sm, bool enable)
 {
 	/* Enables or disables vibra */
 	unsigned char 	Control[6] = {N7110_FRAME_HEADER,0x0C,
@@ -426,13 +434,14 @@ static GSM_Error DCT4EnableVibra(GSM_StateMachine *s, bool enable)
 		0x00};
 
 	if (!enable) Control[4] = 0x00;
-	return GSM_WaitFor (s, Control, 6, 0x1C, 4, ID_User3);
+	return GSM_WaitFor (sm, Control, 6, 0x1C, 4, ID_User3);
 }
 
 void DCT4SetVibraLevel(int argc, char *argv[])
 {
 	GSM_DateTime	Date;
 	unsigned int	i,j;
+	GSM_Error error;
 
 	/* Set vibra level */
 	unsigned char 	SetLevel[6] = {N7110_FRAME_HEADER,0x0E,
@@ -443,25 +452,25 @@ void DCT4SetVibraLevel(int argc, char *argv[])
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
 	SetLevel[4] = atoi(argv[2]);
-	error=GSM_WaitFor (&s, SetLevel, 6, 0x1C, 4, ID_User3);
+	error=GSM_WaitFor (gsm, SetLevel, 6, 0x1C, 4, ID_User3);
 	Print_Error(error);
 
-	error=DCT4EnableVibra(&s, true);
+	error=DCT4EnableVibra(gsm, true);
 	Print_Error(error);
 
 	for (i=0;i<3;i++) {
 		GSM_GetCurrentDateTime (&Date);
 		j=Date.Second;
 		while (j==Date.Second) {
-			my_sleep(10);
+			usleep(10000);
 			GSM_GetCurrentDateTime(&Date);
 		}
 	}
 
-	error=DCT4EnableVibra(&s, false);
+	error=DCT4EnableVibra(gsm, false);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -470,23 +479,24 @@ void DCT4SetVibraLevel(int argc, char *argv[])
 void DCT4VibraTest(int argc, char *argv[])
 {
 	unsigned char ans[200];
+	GSM_Error error;
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=DCT4EnableVibra(&s, true);
+	error=DCT4EnableVibra(gsm, true);
 	Print_Error(error);
 
 	printf("%s\n", _("Press any key to continue..."));
 	GetLine(stdin, ans, 99);
 
-	error=DCT4EnableVibra(&s, false);
+	error=DCT4EnableVibra(gsm, false);
 	Print_Error(error);
 }
 
 #ifdef DEBUG
-static GSM_Error DCT4_ReplyResetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyResetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	switch (msg.Buffer[3]) {
 	case 0x05:
@@ -502,6 +512,7 @@ static GSM_Error DCT4_ReplyResetSecurityCode(GSM_Protocol_Message msg, GSM_State
 void DCT4ResetSecurityCode(int argc, char *argv[])
 {
 	unsigned int	i;
+	GSM_Error error;
 	unsigned char 	ResetCode[30] = {0x00,0x06,0x03,0x04,0x01,
 		'1','2','3','4','5','6','7','8','9','0',	/* Old code */
 		0x00,
@@ -510,9 +521,9 @@ void DCT4ResetSecurityCode(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, ResetCode, 27, 0x08, 4, ID_User2);
+	error=GSM_WaitFor (gsm, ResetCode, 27, 0x08, 4, ID_User2);
 	if (error == ERR_UNKNOWN) {
 		if (answer_yes2("Try brutal force ?")) {
 			for (i=10000;i<9999999;i++) {
@@ -520,7 +531,7 @@ void DCT4ResetSecurityCode(int argc, char *argv[])
 				memset(ResetCode+6,0,22);
 				sprintf(ResetCode+5,"%i",i);
 				sprintf(ResetCode+16,"12345");
-				error=GSM_WaitFor (&s, ResetCode, 27, 0x08, 4, ID_User2);
+				error=GSM_WaitFor (gsm, ResetCode, 27, 0x08, 4, ID_User2);
 				if (error == ERR_NONE) break;
 			}
 		}
@@ -530,11 +541,11 @@ void DCT4ResetSecurityCode(int argc, char *argv[])
 
 char SecLength;
 
-static GSM_Error DCT4_ReplyGetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetSecurityCode(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	if (msg.Length > 12) {
 		SecLength = msg.Buffer[13];
-		if ((msg.Buffer[17]+18) == msg.Length) {
+		if ((size_t)(msg.Buffer[17]+18) == msg.Length) {
 			printf(_("Security code is %s\n"),msg.Buffer+18);
 //			DumpMessage(stdout, msg.Buffer, msg.Length);
 		}
@@ -549,7 +560,7 @@ void DCT4GetSecurityCode(int argc, char *argv[])
 				  0x00, 0x23, 		//ID
 				  0x00, 0x00, 		//Index
 				  0x00, 0x00};
-	unsigned char 	read[]={0x00, 0x08, 0x02, 0x04,
+	unsigned char 	readcode[]={0x00, 0x08, 0x02, 0x04,
 				0x00, 0x23, 		//ID
 				0x00, 0x00, 		//Index
 				0x00, 0x00, 0x00, 0x00, 0x00,
@@ -558,21 +569,21 @@ void DCT4GetSecurityCode(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
 	SecLength = 0;
-	error=GSM_WaitFor (&s, getlen, sizeof(getlen), 0x23, 1, ID_User1);
+	error=GSM_WaitFor (gsm, getlen, sizeof(getlen), 0x23, 1, ID_User1);
 	Print_Error(error);
 	if (SecLength != 0) {
-		read[17] = SecLength;
-		error=GSM_WaitFor (&s, read, sizeof(read), 0x23, 5, ID_User1);
+		readcode[17] = SecLength;
+		error=GSM_WaitFor (gsm, readcode, sizeof(readcode), 0x23, 5, ID_User1);
 		Print_Error(error);
 	}
 }
 
-static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
-	int 		i=18,j;
+	size_t i=18,j;
 	unsigned char	Buffer[100];
 
 	switch (msg.Buffer[3]) {
@@ -582,19 +593,19 @@ static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMac
 			dbgprintf("Empty\n");
 			return ERR_EMPTY;
 		}
- 		*s->Phone.Data.VoiceRecord = 0;
+ 		*sm->Phone.Data.VoiceRecord = 0;
 		while (i<msg.Length) {
- 			s->Phone.Data.PhoneString[(*s->Phone.Data.VoiceRecord)++] = msg.Buffer[i+1];
- 			s->Phone.Data.PhoneString[(*s->Phone.Data.VoiceRecord)++] = msg.Buffer[i];
+ 			sm->Phone.Data.PhoneString[(*sm->Phone.Data.VoiceRecord)++] = msg.Buffer[i+1];
+ 			sm->Phone.Data.PhoneString[(*sm->Phone.Data.VoiceRecord)++] = msg.Buffer[i];
 			i += 2;
 		}
 		return ERR_NONE;
 	case 0x0D:
 		dbgprintf("Last part of voice record is %02x %02x\n",msg.Buffer[11],msg.Buffer[12]);
 		dbgprintf("Token is %02x\n",msg.Buffer[13]);
- 		s->Phone.Data.PhoneString[0] = msg.Buffer[11];
- 		s->Phone.Data.PhoneString[1] = msg.Buffer[12];
- 		s->Phone.Data.PhoneString[2] = msg.Buffer[13];
+ 		sm->Phone.Data.PhoneString[0] = msg.Buffer[11];
+ 		sm->Phone.Data.PhoneString[1] = msg.Buffer[12];
+ 		sm->Phone.Data.PhoneString[2] = msg.Buffer[13];
 		return ERR_NONE;
 		break;
 	case 0x31:
@@ -604,12 +615,12 @@ static GSM_Error DCT4_ReplyGetVoiceRecord(GSM_Protocol_Message msg, GSM_StateMac
 			memcpy(Buffer,msg.Buffer+(j+1),msg.Buffer[j]);
 			Buffer[msg.Buffer[j]] 	= 0;
 			Buffer[msg.Buffer[j]+1] = 0;
-			dbgprintf("%i. \"%s\"\n",i+1,DecodeUnicodeString(Buffer));
- 			if (i==*s->Phone.Data.VoiceRecord) {
- 				sprintf(s->Phone.Data.PhoneString,"%s.wav",DecodeUnicodeString(Buffer));
+			dbgprintf(SIZE_T_FORMAT ". \"%s\"\n",i+1,DecodeUnicodeString(Buffer));
+ 			if (i==*sm->Phone.Data.VoiceRecord) {
+ 				sprintf(sm->Phone.Data.PhoneString,"%s->wav",DecodeUnicodeString(Buffer));
 				return ERR_NONE;
 			}
-			if (i != msg.Buffer[9] - 1) {
+			if (i != (size_t)msg.Buffer[9] - 1) {
 				j+=msg.Buffer[j] + 1;
 				if (msg.Buffer[j] == 0x00 && msg.Buffer[j+1]==0x00) j+=2;
 				j+=23;
@@ -661,9 +672,10 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 
 	long		wavfilesize=0;
 	unsigned char	FileName[100], Buffer[10000], Token;
-	unsigned int 	Location, size=0, CurrentLocation = 0, TokenLocation;
+	size_t 	Location, size=0, CurrentLocation = 0, TokenLocation;
 	int		i;
 	FILE		*WAVFile;
+	GSM_Error error;
 
 	Location = atoi(argv[2]);
 	if (Location == 0x00) {
@@ -676,39 +688,42 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	s.Phone.Data.VoiceRecord 	= &Location;
-	s.Phone.Data.PhoneString 	= FileName;
+	gsm->Phone.Data.VoiceRecord 	= &Location;
+	gsm->Phone.Data.PhoneString 	= FileName;
 	dbgprintf("Getting voice record name\n");
-	error=GSM_WaitFor (&s, ReqNames, 14, 0x4A, 4, ID_User4);
+	error=GSM_WaitFor (gsm, ReqNames, 14, 0x4A, 4, ID_User4);
 	Print_Error(error);
 
-	s.Phone.Data.PhoneString 	= Buffer;
+	gsm->Phone.Data.PhoneString 	= Buffer;
 	ReqToken[7] 			= Location;
 	dbgprintf("Getting voice record token\n");
-	error=GSM_WaitFor (&s, ReqToken, 10, 0x23, 4, ID_User4);
+	error=GSM_WaitFor (gsm, ReqToken, 10, 0x23, 4, ID_User4);
 	Print_Error(error);
 	TokenLocation 			= Buffer[0] * 256 + Buffer[1];
 	Token				= Buffer[2];
 
 	WAVFile = fopen(FileName, "wb");
+	if (WAVFile == NULL) {
+		goto failnofile;
+	}
 
-	fwrite(&WAV_Header,	1, sizeof(WAV_Header),	WAVFile);
-	fwrite(&FMT_Header,	1, sizeof(FMT_Header),	WAVFile);
-	fwrite(&DATA_Header,	1, sizeof(DATA_Header),	WAVFile);
+	chk_fwrite(&WAV_Header,	1, sizeof(WAV_Header),	WAVFile);
+	chk_fwrite(&FMT_Header,	1, sizeof(FMT_Header),	WAVFile);
+	chk_fwrite(&DATA_Header,	1, sizeof(DATA_Header),	WAVFile);
 
-	s.Phone.Data.VoiceRecord 	= &size;
-	s.Phone.Data.PhoneString 	= Buffer;
+	gsm->Phone.Data.VoiceRecord 	= &size;
+	gsm->Phone.Data.PhoneString 	= Buffer;
 	ReqGet[7]			= Location;
 	fprintf(stderr,"Getting voice record and saving to \"%s\": ",FileName);
 	while (1) {
 		dbgprintf("Getting next part of voice record\n");
 		fprintf(stderr,".");
-		error=GSM_WaitFor (&s, ReqGet, 18, 0x23, 4, ID_User4);
+		error=GSM_WaitFor (gsm, ReqGet, 18, 0x23, 4, ID_User4);
 		if (error == ERR_NONE) {
 			wavfilesize += size;
-			fwrite(Buffer,1,size,WAVFile);
+			chk_fwrite(Buffer,1,size,WAVFile);
 		}
 		if (error == ERR_EMPTY) break;
 		Print_Error(error);
@@ -722,10 +737,10 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 		ReqGet[16] = i;
 		ReqGet[17] = Token;
 		fprintf(stderr,".");
-		error=GSM_WaitFor (&s, ReqGet, 18, 0x23, 4, ID_User4);
+		error=GSM_WaitFor (gsm, ReqGet, 18, 0x23, 4, ID_User4);
 		if (error == ERR_NONE) {
 			wavfilesize += size;
-			fwrite(Buffer,1,size,WAVFile);
+			chk_fwrite(Buffer,1,size,WAVFile);
 			break;
 		}
 		if (error != ERR_EMPTY) Print_Error(error);
@@ -736,10 +751,10 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 		ReqGet[11] = CurrentLocation / 256;
 		ReqGet[12] = CurrentLocation % 256;
 		fprintf(stderr,".");
-		error=GSM_WaitFor (&s, ReqGet, 18, 0x23, 4, ID_User4);
+		error=GSM_WaitFor (gsm, ReqGet, 18, 0x23, 4, ID_User4);
 		if (error == ERR_NONE) {
 			wavfilesize += size;
-			fwrite(Buffer,1,size,WAVFile);
+			chk_fwrite(Buffer,1,size,WAVFile);
 		}
 		if (error == ERR_EMPTY) break;
 		Print_Error(error);
@@ -765,16 +780,17 @@ void DCT4GetVoiceRecord(int argc, char *argv[])
 	DATA_Header[7] 	= (unsigned char)(wavfilesize / (256*256*256));
 
 	fseek( WAVFile, 0, SEEK_SET);
-	fwrite(&WAV_Header,	1, sizeof(WAV_Header),	WAVFile);
-	fwrite(&FMT_Header,	1, sizeof(FMT_Header),	WAVFile);
-	fwrite(&DATA_Header,	1, sizeof(DATA_Header),	WAVFile);
+	chk_fwrite(&WAV_Header,	1, sizeof(WAV_Header),	WAVFile);
+	chk_fwrite(&FMT_Header,	1, sizeof(FMT_Header),	WAVFile);
+	chk_fwrite(&DATA_Header,	1, sizeof(DATA_Header),	WAVFile);
 
+fail:
 	fclose(WAVFile);
-
+failnofile:
 	GSM_Terminate();
 }
 
-static GSM_Error DCT4_ReplyGetBTInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetBTInfo(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	printf(_("device address %02x%02x%02x%02x%02x%02x\n"),
 		msg.Buffer[9],msg.Buffer[10],msg.Buffer[11],
@@ -782,7 +798,7 @@ static GSM_Error DCT4_ReplyGetBTInfo(GSM_Protocol_Message msg, GSM_StateMachine
 	return ERR_NONE;
 }
 
-static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int 			i;
 	unsigned char 		buff[7];
@@ -800,7 +816,10 @@ static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine
 		buff[4] = buff[3];
 		buff[3] = ' ';
 		if (strcmp(DecodeUnicodeString(GSM_GetNetworkName(buff)),"unknown")) {
-			printf(_("Old simlock   : %s (%s)\n"),DecodeUnicodeString(GSM_GetNetworkName(buff)),buff);
+			printf(LISTFORMAT, _("Old simlock"));
+			printf("%s (%s)\n",
+				DecodeUnicodeString(GSM_GetNetworkName(buff)),
+				buff);
 		}
 
 		dbgprintf("\n");
@@ -818,7 +837,7 @@ static GSM_Error DCT4_ReplyGetSimlock(GSM_Protocol_Message msg, GSM_StateMachine
 				dbgprintf("%02x",msg.Buffer[i]);
 			}
 			dbgprintf("\n");
-			printf(_("Simlock data  : "));
+			printf(LISTFORMAT, _("Simlock data"));
 			for (i=60;i<63;i++) {
 				printf("%02x",msg.Buffer[i]);
 			}
@@ -834,43 +853,46 @@ void DCT4Info(int argc, char *argv[])
 	unsigned char GetBTAddress[8] = {N6110_FRAME_HEADER, 0x09, 0x19, 0x01, 0x03, 0x06};
 	unsigned char GetSimlock[5] = {N6110_FRAME_HEADER, 0x12, 0x0D};
 	unsigned char value[10];
+	GSM_Error error;
 
         if (CheckDCT4Only()!=ERR_NONE) return;
 
-	if (IsPhoneFeatureAvailable(s.Phone.Data.ModelInfo, F_SERIES40_30)) return;
+	if (GSM_IsPhoneFeatureAvailable(gsm->Phone.Data.ModelInfo, F_SERIES40_30)) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	if (IsPhoneFeatureAvailable(s.Phone.Data.ModelInfo, F_BLUETOOTH)) {
-		printf(_("Bluetooth     : "));
+	if (GSM_IsPhoneFeatureAvailable(gsm->Phone.Data.ModelInfo, F_BLUETOOTH)) {
+		printf(LISTFORMAT, _("Bluetooth"));
 
-		error=GSM_WaitFor (&s, GetBTAddress, 8, 0xD7, 4, ID_User6);
+		error=GSM_WaitFor (gsm, GetBTAddress, 8, 0xD7, 4, ID_User6);
 		Print_Error(error);
 	}
 
-	error=GSM_WaitFor (&s, GetSimlock, 5, 0x53, 4, ID_User6);
+	error=GSM_WaitFor (gsm, GetSimlock, 5, 0x53, 4, ID_User6);
 	Print_Error(error);
 	GetSimlock[4] = 0x0E;
-	error=GSM_WaitFor (&s, GetSimlock, 5, 0x53, 4, ID_User6);
+	error=GSM_WaitFor (gsm, GetSimlock, 5, 0x53, 4, ID_User6);
 	Print_Error(error);
 	GetSimlock[3] = 0x0C;
-	error=GSM_WaitFor (&s, GetSimlock, 4, 0x53, 4, ID_User6);
+	error=GSM_WaitFor (gsm, GetSimlock, 4, 0x53, 4, ID_User6);
 	Print_Error(error);
-	error=NOKIA_GetPhoneString(&s,"\x00\x03\x02\x07\x00\x08",6,0x1b,value,ID_User6,10);
+	error=NOKIA_GetPhoneString(gsm,"\x00\x03\x02\x07\x00\x08",6,0x1b,value,ID_User6,10);
 	Print_Error(error);
-	printf(_("UEM           : %s\n"),value);
+	printf(LISTFORMAT "%s\n", _("UEM"), value);
 }
 
-static FILE 	*T9File;
-int 		T9Size;
-int 		T9FullSize;
+static FILE *T9File;
+size_t T9Size;
+size_t T9FullSize;
 
-static GSM_Error DCT4_ReplyGetT9(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetT9(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	T9FullSize 	= msg.Buffer[18] * 256 + msg.Buffer[19];
 	T9Size 		= msg.Length - 18;
-	fwrite(msg.Buffer+18,1,T9Size,T9File);
+	chk_fwrite(msg.Buffer+18,1,T9Size,T9File);
 	return ERR_NONE;
+fail:
+	return ERR_WRITING_FILE;
 }
 
 void DCT4GetT9(int argc, char *argv[])
@@ -881,13 +903,14 @@ void DCT4GetT9(int argc, char *argv[])
 			       0x00, 0x00,	/* Start position */
 			       0x00, 0x00,
 			       0x02, 0xBC};	/* How many bytes to read */
+	GSM_Error error;
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
 	T9File = fopen("T9", "w");
 	if (T9File == NULL) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
 	i = 0;
 	while (1) {
@@ -900,7 +923,7 @@ void DCT4GetT9(int argc, char *argv[])
 			}
 			if (T9Dictionary - i == 0) break;
 		}
-		error=GSM_WaitFor (&s, req, 18, 0x23, 4, ID_User3);
+		error=GSM_WaitFor (gsm, req, 18, 0x23, 4, ID_User3);
 		Print_Error(error);
 		if (i==0) {
 			T9Dictionary = T9FullSize;
@@ -914,13 +937,14 @@ void DCT4GetT9(int argc, char *argv[])
 
 #ifdef GSM_ENABLE_NOKIA6510
 
-extern GSM_Error N6510_SetLight(GSM_StateMachine *s, N6510_PHONE_LIGHTS light, bool enable);
+extern GSM_Error N6510_SetLight(GSM_StateMachine *sm, N6510_PHONE_LIGHTS light, bool enable);
 
 void DCT4SetLight(int argc, char *argv[])
 {
 	int			i;
 	N6510_PHONE_LIGHTS 	type;
 	bool			enable;
+	GSM_Error error;
 
 	if (strcasecmp(argv[2],"display") == 0) { 	type = N6510_LIGHT_DISPLAY;
 	} else if (strcasecmp(argv[2],"keypad") == 0) {	type = N6510_LIGHT_KEYPAD;
@@ -937,15 +961,16 @@ void DCT4SetLight(int argc, char *argv[])
 		exit(-1);
 	}
 
-	for (i=0;i<s.ConfigNum;i++) {
-		s.Config[i].StartInfo = "false";
+	for (i=0;i<gsm->ConfigNum;i++) {
+		free(gsm->Config[i].StartInfo);
+		gsm->Config[i].StartInfo = strdup("false");
 	}
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	error=N6510_SetLight(&s, type, enable);
+	error=N6510_SetLight(gsm, type, enable);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -962,24 +987,24 @@ void DCT4DisplayTest(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	DCT4_SetPhoneMode(&s, DCT4_MODE_TEST);
+	DCT4_SetPhoneMode(gsm, DCT4_MODE_TEST);
 
-	s.Protocol.Functions->WriteMessage(&s, req0, 6, 0x40);
+	gsm->Protocol.Functions->WriteMessage(gsm, req0, 6, 0x40);
 
 	req[8] = atoi(argv[2]);
-	s.Protocol.Functions->WriteMessage(&s, req, 10, 0x40);
+	gsm->Protocol.Functions->WriteMessage(gsm, req, 10, 0x40);
 
 	printf("%s\n", _("Press any key to continue..."));
 	GetLine(stdin, ans, 99);
 
-	DCT4_SetPhoneMode(&s, DCT4_MODE_NORMAL);
+	DCT4_SetPhoneMode(gsm, DCT4_MODE_NORMAL);
 }
 
 int ADC;
 
-static GSM_Error DCT4_ReplyGetADC(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetADC(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	if (msg.Buffer[6] == 0xff && msg.Buffer[7] == 0xff) return ERR_NONE;
 	switch (msg.Buffer[3]) {
@@ -1025,6 +1050,7 @@ static struct DCT4ADCInfo DCT4ADC[] = {
 void DCT4GetADC(int argc, char *argv[])
 {
 	int		i = 0;
+	GSM_Error error;
 	unsigned char 	GetRaw[]  = {N6110_FRAME_HEADER, 0x0F,
 				     0x00,		/* Test number */
 			             0x01};
@@ -1034,16 +1060,16 @@ void DCT4GetADC(int argc, char *argv[])
 
 	if (CheckDCT4Only()!=ERR_NONE) return;
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
 	while (1) {
 		printf(" %30s ",DCT4ADC[i].name);
 		GetRaw[4] = i;
-		error=GSM_WaitFor (&s, GetRaw, 6, 0x17, 4, ID_User3);
+		error=GSM_WaitFor (gsm, GetRaw, 6, 0x17, 4, ID_User3);
 		Print_Error(error);
 		GetUnit[4] 	= i;
 		ADC		= DCT4ADC[i].x;
-		error=GSM_WaitFor (&s, GetUnit, 6, 0x17, 4, ID_User3);
+		error=GSM_WaitFor (gsm, GetUnit, 6, 0x17, 4, ID_User3);
 		Print_Error(error);
 		printf("%s\n",DCT4ADC[i].unit);
 		i++;
@@ -1056,7 +1082,7 @@ void DCT4GetADC(int argc, char *argv[])
 static double 		RadioFreq;
 static unsigned char 	RadioName[100];
 
-static GSM_Error DCT4_ReplyTuneRadio(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyTuneRadio(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int 		length;
 	unsigned char 	name[100];
@@ -1070,16 +1096,16 @@ static GSM_Error DCT4_ReplyTuneRadio(GSM_Protocol_Message msg, GSM_StateMachine
  		name[length*2]	 = 0x00;
  		name[length*2+1] = 0x00;
  		CopyUnicodeString(RadioName,name);
-		smprintf(s,"Station name: \"%s\"\n",DecodeUnicodeString(RadioName));
+		smprintf(sm,"Station name: \"%s\"\n",DecodeUnicodeString(RadioName));
 		return ERR_NONE;
 	case 0x15:
 	case 0x16:
-		smprintf(s,"Response for enabling radio/headset status received\n");
+		smprintf(sm,"Response for enabling radio/headset status received\n");
 		if (msg.Buffer[5] == 0) {
-			smprintf(s,"Connected\n");
+			smprintf(sm,"Connected\n");
 			return ERR_NONE;
 		}
-		smprintf(s,"Probably not connected\n");
+		smprintf(sm,"Probably not connected\n");
 		return ERR_PERMISSION;
 	}
 	return ERR_UNKNOWNRESPONSE;
@@ -1088,6 +1114,7 @@ static GSM_Error DCT4_ReplyTuneRadio(GSM_Protocol_Message msg, GSM_StateMachine
 void DCT4TuneRadio(int argc, char *argv[])
 {
 	double		Freq, diff;
+	GSM_Error error;
  	GSM_FMStation 	FMStation[50],FMStat;
 	int		i, j, num;
 	bool		found;
@@ -1116,17 +1143,17 @@ void DCT4TuneRadio(int argc, char *argv[])
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
 	FMStat.Location = 1;
-	error = Phone->GetFMStation(&s,&FMStat);
+	error = GSM_GetFMStation(gsm,&FMStat);
 	if (error != ERR_NONE && error != ERR_EMPTY) {
 		printf("%s\n", _("Phone seems not to support radio"));
 		GSM_Terminate();
 		exit(-1);
 	}
 
-	error=GSM_WaitFor (&s, Enable, 6, 0x3E, 4, ID_User3);
+	error=GSM_WaitFor (gsm, Enable, 6, 0x3E, 4, ID_User3);
 	if (error == ERR_PERMISSION) {
 		printf("%s\n", _("Please connect headset. Required as antenna"));
 		GSM_Terminate();
@@ -1139,10 +1166,10 @@ void DCT4TuneRadio(int argc, char *argv[])
 		fprintf(stderr,"%cSearching: %i percent",13,(i-88)*100/(108-88));
 		Freq = i;
 		N6510_EncodeFMFrequency(Freq, SetFreq+8);
-		error=GSM_WaitFor (&s, SetFreq, 10, 0x3E, 4, ID_User3);
+		error=GSM_WaitFor (gsm, SetFreq, 10, 0x3E, 4, ID_User3);
 		Print_Error(error);
 
-		error=GSM_WaitFor (&s, Find2, 10, 0x3E, 4, ID_User3);
+		error=GSM_WaitFor (gsm, Find2, 10, 0x3E, 4, ID_User3);
 		Print_Error(error);
 		found = false;
 		for (j=0;j<num;j++) {
@@ -1189,19 +1216,19 @@ void DCT4TuneRadio(int argc, char *argv[])
 
 	if (answer_yes2("Do you want to save found stations")) {
 		fprintf(stderr,"Deleting old FM stations: ");
-		error=Phone->ClearFMStations(&s);
+		error=GSM_ClearFMStations(gsm);
 		Print_Error(error);
 		fprintf(stderr,"Done\n");
 		for (i=0;i<num;i++) {
 			FMStation[i].Location = i+1;
-			error=Phone->SetFMStation(&s,&FMStation[i]);
+			error=GSM_SetFMStation(gsm,&FMStation[i]);
 			Print_Error(error);
 			fprintf(stderr,"%cWriting: %i percent",13,(i+1)*100/num);
 		}
 		fprintf(stderr,"\n");
 	}
 
-	error=GSM_WaitFor (&s, Disable, 6, 0x3E, 4, ID_User3);
+	error=GSM_WaitFor (gsm, Disable, 6, 0x3E, 4, ID_User3);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -1218,15 +1245,16 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 				0x0A,		//volume
 				0x00,0x00,0x00,0x00,0x00,0x00,0x00};
 	GSM_AllRingtonesInfo Info;
+	GSM_Error error;
 //	int i;
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=Phone->GetRingtonesInfo(&s,&Info);
+	error=GSM_GetRingtonesInfo(gsm,&Info);
 	Print_Error(error);
 
 	if (atoi(argv[2]) > Info.Number-1) {
@@ -1237,7 +1265,7 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 	req[5] = Info.Ringtone[atoi(argv[2])-1].ID % 256;
 	req[6] = Info.Ringtone[atoi(argv[2])-1].Group;
 
-	error=GSM_WaitFor (&s, req, 18, 0x1F, 4, ID_User3);
+	error=GSM_WaitFor (gsm, req, 18, 0x1F, 4, ID_User3);
 	Print_Error(error);
 
 //	for (i=0;i<Info.Number;i++) printf(_("%i. \")%s\"\n",i,DecodeUnicodeConsole(Info.Ringtone[i].Name));
@@ -1245,7 +1273,7 @@ void DCT4PlaySavedRingtone(int argc, char *argv[])
 	GSM_Terminate();
 }
 
-static GSM_Error DCT4_ReplyMakeCameraShoot(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyMakeCameraShoot(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	return ERR_NONE;
 }
@@ -1262,27 +1290,28 @@ void DCT4MakeCameraShoot(int argc, char *argv[])
 		0x00, 0x02,	//master folder id
 		0x00, 0x14}; 	//length
 	unsigned char CameraOFF[] = {N6110_FRAME_HEADER, 0x04, 0x01, 0x00};
+	GSM_Error error;
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, SetCamera, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (gsm, SetCamera, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, CameraON, 10, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (gsm, CameraON, 10, 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, CameraON2, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (gsm, CameraON2, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
 	EncodeUnicode(MakeShot+24,"GammuShot",9);
 	MakeShot[15] = 9+9*2;
 	MakeShot[23] = 9*2;
-	error=GSM_WaitFor (&s, MakeShot, 24+MakeShot[23], 0x61, 4, ID_User3);
+	error=GSM_WaitFor (gsm, MakeShot, 24+MakeShot[23], 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, SetCamera, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (gsm, SetCamera, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
-	error=GSM_WaitFor (&s, CameraOFF, 6, 0x61, 4, ID_User3);
+	error=GSM_WaitFor (gsm, CameraOFF, 6, 0x61, 4, ID_User3);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -1290,7 +1319,7 @@ void DCT4MakeCameraShoot(int argc, char *argv[])
 
 int len;
 
-static GSM_Error DCT4_ReplyGetScreenDump(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetScreenDump(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	if (msg.Buffer[7] == 0x0C) len = 1;
 	return ERR_NONE;
@@ -1299,23 +1328,24 @@ static GSM_Error DCT4_ReplyGetScreenDump(GSM_Protocol_Message msg, GSM_StateMach
 void DCT4GetScreenDump(int argc, char *argv[])
 {
 	unsigned char req[] = {N6110_FRAME_HEADER, 0x07, 0x01, 0x00};
+	GSM_Error error;
 	//n6110_frameheader 06//screen info
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, req, 6, 0x0E, 4, ID_User3);
+	error=GSM_WaitFor (gsm, req, 6, 0x0E, 4, ID_User3);
 	Print_Error(error);
 	len = 2000;
-	while (len >= 200) GSM_ReadDevice(&s,true);
+	while (len >= 200) GSM_ReadDevice(gsm,true);
 
 	GSM_Terminate();
 }
 
-static GSM_Error DCT4_ReplyGetPBKFeatures(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error DCT4_ReplyGetPBKFeatures(GSM_Protocol_Message msg, GSM_StateMachine *sm)
 {
 	int i,pos=6;
 
@@ -1381,6 +1411,7 @@ static GSM_Error DCT4_ReplyGetPBKFeatures(GSM_Protocol_Message msg, GSM_StateMac
 void DCT4GetPBKFeatures(int argc, char *argv[])
 {
 	GSM_MemoryType	MemoryType=0;
+	GSM_Error error;
 	unsigned char 	req[] = {N6110_FRAME_HEADER, 0x25,
 				 0x05, 	// memory type
 				 0x00};
@@ -1398,16 +1429,16 @@ void DCT4GetPBKFeatures(int argc, char *argv[])
 		exit (-1);
 	}
 
-	req[4] = NOKIA_GetMemoryType(&s, MemoryType,N71_65_MEMORY_TYPES);
+	req[4] = NOKIA_GetMemoryType(gsm, MemoryType,N71_65_MEMORY_TYPES);
 	if (req[4]==0xff) exit(-1);
 
 	GSM_Init(true);
 
         CheckDCT4();
 
-	s.User.UserReplyFunctions=UserReplyFunctions4;
+	gsm->User.UserReplyFunctions=UserReplyFunctions4;
 
-	error=GSM_WaitFor (&s, req, 6, 0x03, 4, ID_User3);
+	error=GSM_WaitFor (gsm, req, 6, 0x03, 4, ID_User3);
 	Print_Error(error);
 
 	GSM_Terminate();
