@@ -4,6 +4,18 @@
 #include <signal.h>
 #include <stdarg.h>
 #include <time.h>
+#ifndef WIN32
+#include <sys/types.h>
+#include <sys/wait.h>
+#endif
+
+/* Some systems let waitpid(2) tell callers about stopped children. */
+#if !defined (WCONTINUED)
+#  define WCONTINUED 0
+#endif
+#if !defined (WIFCONTINUED)
+#  define WIFCONTINUED(s)	(0)
+#endif
 
 #include "../../common/misc/coding/coding.h"
 #include "../../common/misc/locales.h"
@@ -21,9 +33,12 @@ FILE 		 *smsd_log_file = NULL;
 static int	 TPMR;
 static GSM_Error SendingSMSStatus;
 
-static void SMSSendingSMSStatus (GSM_StateMachine *s, int status, int mr)
+void SMSSendingSMSStatus (GSM_StateMachine *sm, int status, int mr)
 {
-	dbgprintf("Incoming SMS device: \"%s\" status=%d, reference=%d\n",s->CurrentConfig->Device, status, mr);
+	WriteSMSDLog("Incoming SMS device: \"%s\" status=%d, reference=%d\n",
+			GSM_GetConfig(sm, -1)->Device,
+			status,
+			mr);
 	TPMR = mr;
 	if (status==0) {
 		SendingSMSStatus = ERR_NONE;
@@ -36,18 +51,19 @@ void GSM_Terminate_SMSD(char *msg, int error, bool exitprogram, int rc)
 {
 	int ret = ERR_NONE;
 
-	if (s.opened) {
+	if (GSM_IsConnected(gsm)) {
 		WriteSMSDLog(_("Terminating communication"));
-		ret=GSM_TerminateConnection(&s);
+		ret=GSM_TerminateConnection(gsm);
 		if (ret!=ERR_NONE) {
-			printf("%s\n",print_error(error,s.di.df));
-			if (s.opened) GSM_TerminateConnection(&s);
+			printf("%s\n",GSM_ErrorString(error));
+			if (GSM_IsConnected(gsm)) {
+				GSM_TerminateConnection(gsm);
+			}
 		}
 	}
 	if (error != 0) {
-		WriteSMSDLog(msg, error, print_error(error,s.di.df));
-		fprintf(stderr, msg, error, print_error(error,s.di.df));
-		fprintf(stderr, "\n");
+		WriteSMSDLog("%s (%s:%i)", msg, GSM_ErrorString(error), error);
+		fprintf(stderr, "%s (%s:%i)\n", msg, GSM_ErrorString(error), error);
 	}
 	if (exitprogram) {
 		if (smsd_log_file!=NULL) fclose(smsd_log_file);
@@ -55,9 +71,7 @@ void GSM_Terminate_SMSD(char *msg, int error, bool exitprogram, int rc)
 	}
 }
 
-#if defined(__GNUC__) && !defined(printf)
-__attribute__((format(printf, 1, 2)))
-#endif
+PRINTF_STYLE(1, 2)
 void WriteSMSDLog(char *format, ...)
 {
 	GSM_DateTime 	date_time;
@@ -80,10 +94,11 @@ void WriteSMSDLog(char *format, ...)
 	}
 }
 
-void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *service)
+void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool uselog, char *service)
 {
 	INI_Section 		*smsdcfgfile = NULL;
 	GSM_Config 		smsdcfg;
+	GSM_Config 		*gammucfg;
 	unsigned char		*str;
 	static unsigned char	emptyPath[1] = "\0";
 	GSM_Error		error;
@@ -109,44 +124,51 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 		}
 		fprintf(stderr, _("Log filename is \"%s\"\n"),Config->logfilename);
 	}
-	if (log) WriteSMSDLog(_("Starting GAMMU smsd"));
+	if (uselog) WriteSMSDLog(_("Starting GAMMU smsd"));
 
 	/* Include Numbers used, because we don't want create new variable */
 	Config->IncludeNumbers=INI_FindLastSectionEntry(smsdcfgfile, "gammu", false);
 	if (Config->IncludeNumbers) {
 		GSM_ReadConfig(smsdcfgfile, &smsdcfg, 0);
-		memcpy(&s.Config,&smsdcfg,sizeof(GSM_Config));
-		error=GSM_SetDebugFile(s.Config[0].DebugFile, &di);
+		gammucfg = GSM_GetConfig(gsm, 0);
+		*gammucfg = smsdcfg;
+		error = GSM_SetDebugFile(gammucfg->DebugFile, GSM_GetGlobalDebug());
 	}
 
 	Config->PINCode=INI_GetValue(smsdcfgfile, "smsd", "PIN", false);
 	if (Config->PINCode == NULL) {
- 		if (log) WriteSMSDLog(_("Warning: No PIN code in %s file"),filename);
+ 		if (uselog) WriteSMSDLog(_("Warning: No PIN code in %s file"),filename);
  		fprintf(stderr, _("Warning: No PIN code in %s file\n"),filename);
 	} else {
-		if (log) WriteSMSDLog(_("PIN code is \"%s\""),Config->PINCode);
+		if (uselog) WriteSMSDLog(_("PIN code is \"%s\""),Config->PINCode);
 	}
 
 	str = INI_GetValue(smsdcfgfile, "smsd", "commtimeout", false);
 	if (str) Config->commtimeout=atoi(str); else Config->commtimeout = 1;
+	str = INI_GetValue(smsdcfgfile, "smsd", "deliveryreportdelay", false);
+	if (str) Config->deliveryreportdelay=atoi(str); else Config->deliveryreportdelay = 10;
 	str = INI_GetValue(smsdcfgfile, "smsd", "sendtimeout", false);
 	if (str) Config->sendtimeout=atoi(str); else Config->sendtimeout = 10;
 	str = INI_GetValue(smsdcfgfile, "smsd", "receivefrequency", false);
 	if (str) Config->receivefrequency=atoi(str); else Config->receivefrequency = 0;
+	str = INI_GetValue(smsdcfgfile, "smsd", "checksecurity", false);
+	if (str) Config->checksecurity=atoi(str); else Config->checksecurity = 1;
 	str = INI_GetValue(smsdcfgfile, "smsd", "resetfrequency", false);
 	if (str) Config->resetfrequency=atoi(str); else Config->resetfrequency = 0;
-	if (log) WriteSMSDLog(_("commtimeout=%i, sendtimeout=%i, receivefrequency=%i, resetfrequency=%i"),
-			Config->commtimeout, Config->sendtimeout, Config->receivefrequency, Config->resetfrequency);
+	if (uselog) WriteSMSDLog("commtimeout=%i, sendtimeout=%i, receivefrequency=%i, resetfrequency=%i, checksecurity=%i",
+			Config->commtimeout, Config->sendtimeout, Config->receivefrequency, Config->resetfrequency, Config->checksecurity);
 
 	Config->deliveryreport = INI_GetValue(smsdcfgfile, "smsd", "deliveryreport", false);
 	if (Config->deliveryreport == NULL || (strncasecmp(Config->deliveryreport, "log", 3) != 0 && strncasecmp(Config->deliveryreport, "sms", 3) != 0)) {
 		Config->deliveryreport = "no";
 	}
-	if (log) WriteSMSDLog("deliveryreport = %s", Config->deliveryreport);
+	if (uselog) WriteSMSDLog("deliveryreport = %s", Config->deliveryreport);
 
 	Config->PhoneID = INI_GetValue(smsdcfgfile, "smsd", "phoneid", false);
 	if (Config->PhoneID == NULL) Config->PhoneID = "";
-	if (log) WriteSMSDLog("phoneid = %s", Config->PhoneID);
+	if (uselog) WriteSMSDLog("phoneid = %s", Config->PhoneID);
+
+	Config->RunOnReceive = INI_GetValue(smsdcfgfile, "smsd", "runonreceive", false);
 
 	str = INI_GetValue(smsdcfgfile, "smsd", "smsc", false);
 	if (str) {
@@ -162,7 +184,7 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 		Config->SMSC.Location     = 0;
 	}
 
-	if (!strcmp(service,"FILES")) {
+	if (!strcasecmp(service,"FILES")) {
 		Config->inboxpath=INI_GetValue(smsdcfgfile, "smsd", "inboxpath", false);
 		if (Config->inboxpath == NULL) Config->inboxpath = emptyPath;
 
@@ -170,7 +192,7 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 		if (Config->inboxformat == NULL || (strncasecmp(Config->inboxformat, "detail", 6) != 0 && strncasecmp(Config->inboxformat, "unicode", 7) != 0)) {
 			Config->inboxformat = "standard";
 		}
-		if (log) WriteSMSDLog(_("Inbox is \"%s\" with format \"%s\""), Config->inboxpath, Config->inboxformat);
+		if (uselog) WriteSMSDLog(_("Inbox is \"%s\" with format \"%s\""), Config->inboxpath, Config->inboxformat);
 
 		Config->outboxpath=INI_GetValue(smsdcfgfile, "smsd", "outboxpath", false);
 		if (Config->outboxpath == NULL) Config->outboxpath = emptyPath;
@@ -179,19 +201,19 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 		if (Config->transmitformat == NULL || (strncasecmp(Config->transmitformat, "auto", 4) != 0 && strncasecmp(Config->transmitformat, "unicode", 7) != 0)) {
 			Config->transmitformat = "7bit";
 		}
-		if (log) WriteSMSDLog(_("Outbox is \"%s\" with transmission format \"%s\""), Config->outboxpath, Config->transmitformat);
+		if (uselog) WriteSMSDLog(_("Outbox is \"%s\" with transmission format \"%s\""), Config->outboxpath, Config->transmitformat);
 
 		Config->sentsmspath=INI_GetValue(smsdcfgfile, "smsd", "sentsmspath", false);
 		if (Config->sentsmspath == NULL) Config->sentsmspath = Config->outboxpath;
-		if (log) WriteSMSDLog(_("Sent SMS moved to \"%s\""),Config->sentsmspath);
+		if (uselog) WriteSMSDLog(_("Sent SMS moved to \"%s\""),Config->sentsmspath);
 
 		Config->errorsmspath=INI_GetValue(smsdcfgfile, "smsd", "errorsmspath", false);
 		if (Config->errorsmspath == NULL) Config->errorsmspath = Config->sentsmspath;
-		if (log) WriteSMSDLog(_("SMS with errors moved to \"%s\""),Config->errorsmspath);
+		if (uselog) WriteSMSDLog(_("SMS with errors moved to \"%s\""),Config->errorsmspath);
 	}
 
 #ifdef HAVE_MYSQL_MYSQL_H
-	if (!strcmp(service,"MYSQL")) {
+	if (!strcasecmp(service,"MYSQL")) {
 		Config->skipsmscnumber = INI_GetValue(smsdcfgfile, "smsd", "skipsmscnumber", false);
 		if (Config->skipsmscnumber == NULL) Config->skipsmscnumber="";
 		Config->user = INI_GetValue(smsdcfgfile, "smsd", "user", false);
@@ -206,7 +228,7 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 #endif
 
 #ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
-	if (!strcmp(service,"PGSQL")) {
+	if (!strcasecmp(service,"PGSQL")) {
 		Config->skipsmscnumber = INI_GetValue(smsdcfgfile, "smsd", "skipsmscnumber", false);
 		if (Config->skipsmscnumber == NULL) Config->skipsmscnumber="";
 		Config->user = INI_GetValue(smsdcfgfile, "smsd", "user", false);
@@ -223,13 +245,13 @@ void SMSD_ReadConfig(char *filename, GSM_SMSDConfig *Config, bool log, char *ser
 	Config->IncludeNumbers=INI_FindLastSectionEntry(smsdcfgfile, "include_numbers", false);
 	Config->ExcludeNumbers=INI_FindLastSectionEntry(smsdcfgfile, "exclude_numbers", false);
 	if (Config->IncludeNumbers != NULL) {
-		if (log) WriteSMSDLog(_("Include numbers available"));
+		if (uselog) WriteSMSDLog(_("Include numbers available"));
 	}
 	if (Config->ExcludeNumbers != NULL) {
 		if (Config->IncludeNumbers == NULL) {
-			if (log) WriteSMSDLog(_("Exclude numbers available"));
+			if (uselog) WriteSMSDLog(_("Exclude numbers available"));
 		} else {
-			if (log) WriteSMSDLog(_("Exclude numbers available, but IGNORED"));
+			if (uselog) WriteSMSDLog(_("Exclude numbers available, but IGNORED"));
 		}
 	}
 
@@ -244,10 +266,10 @@ bool SMSD_CheckSecurity(GSM_SMSDConfig *Config)
 	GSM_Error		error;
 
 	/* Need PIN ? */
-	error=Phone->GetSecurityStatus(&s,&SecurityCode.Type);
+	error=GSM_GetSecurityStatus(gsm,&SecurityCode.Type);
 	/* Unknown error */
 	if (error != ERR_NOTSUPPORTED && error != ERR_NONE) {
-		WriteSMSDLog(_("Error getting security status (%i)"), error);
+		WriteSMSDLog(_("Error getting security status (%s:%i)"), GSM_ErrorString(error), error);
 		return false;
 	}
 	/* No supported - do not check more */
@@ -262,12 +284,12 @@ bool SMSD_CheckSecurity(GSM_SMSDConfig *Config)
 		} else {
 			WriteSMSDLog(_("Trying to enter PIN"));
 			strcpy(SecurityCode.Code,Config->PINCode);
-			error=Phone->EnterSecurityCode(&s,SecurityCode);
+			error=GSM_EnterSecurityCode(gsm,SecurityCode);
 			if (error == ERR_SECURITYERROR) {
 				GSM_Terminate_SMSD(_("ERROR: incorrect PIN"), error, true, -1);
 			}
 			if (error != ERR_NONE) {
-				WriteSMSDLog(_("Error entering PIN (%i)"), error);
+				WriteSMSDLog(_("Error entering PIN (%s:%i)"), GSM_ErrorString(error), error);
 				return false;
 		  	}
 		}
@@ -276,13 +298,104 @@ bool SMSD_CheckSecurity(GSM_SMSDConfig *Config)
 	case SEC_Pin2:
 	case SEC_Puk:
 	case SEC_Puk2:
-		GSM_Terminate_SMSD(_("ERROR: phone requires not supported code type"), 0, true, -1);
+	case SEC_Phone:
+		GSM_Terminate_SMSD(_("ERROR: phone requires not supported code type"), ERR_UNKNOWN, true, -1);
 	case SEC_None:
 		break;
 	}
 	return true;
 }
 
+#ifdef WIN32
+bool SMSD_RunOnReceive(GSM_MultiSMSMessage sms UNUSED, GSM_SMSDConfig *Config)
+{
+	BOOL ret;
+	STARTUPINFO si;
+	PROCESS_INFORMATION pi;
+
+	ZeroMemory(&si, sizeof(si));
+	si.cb = sizeof(si);
+	ZeroMemory(&pi, sizeof(pi));
+
+	ret = CreateProcess(NULL,     /* No module name (use command line) */
+			Config->RunOnReceive, /* Command line */
+			NULL,           /* Process handle not inheritable*/
+			NULL,           /* Thread handle not inheritable*/
+			FALSE,          /* Set handle inheritance to FALSE*/
+			0,              /* No creation flags*/
+			NULL,           /* Use parent's environment block*/
+			NULL,           /* Use parent's starting directory */
+			&si,            /* Pointer to STARTUPINFO structure*/
+			&pi );           /* Pointer to PROCESS_INFORMATION structure*/
+	if (! ret) {
+		WriteSMSDLog("CreateProcess failed (%d)\n", (int)GetLastError());
+	} else {
+		/* We don't need handles at all */
+		CloseHandle(pi.hProcess);
+		CloseHandle(pi.hThread);
+	}
+	return ret;
+}
+#else
+bool SMSD_RunOnReceive(GSM_MultiSMSMessage sms UNUSED, GSM_SMSDConfig *Config)
+{
+	int pid;
+	int i;
+	pid_t w;
+	int status;
+
+	if (Config->RunOnReceive == NULL) {
+		return false;
+	}
+
+	pid = fork();
+
+	if (pid == -1) {
+		WriteSMSDLog("Error spawning new process");
+		return false;
+	}
+
+	if (pid != 0) {
+		/* We are the parent, wait for child */
+		i = 0;
+		do {
+			w = waitpid(pid, &status, WUNTRACED | WCONTINUED);
+			if (w == -1) {
+				WriteSMSDLog("Failed to wait for process");
+				return false;
+			}
+
+			if (WIFEXITED(status)) {
+				WriteSMSDLog("Process exited, status=%d\n", WEXITSTATUS(status));
+				return (WEXITSTATUS(status) == 0);
+			} else if (WIFSIGNALED(status)) {
+				WriteSMSDLog("Process killed by signal %d\n", WTERMSIG(status));
+				return false;
+			} else if (WIFSTOPPED(status)) {
+				WriteSMSDLog("Process stopped by signal %d\n", WSTOPSIG(status));
+			} else if (WIFCONTINUED(status)) {
+				WriteSMSDLog("Process continued\n");
+			}
+			usleep(100000);
+			if (i++ > 1200) {
+				WriteSMSDLog("Waited two minutes for child process, giving up\n");
+				return true;
+			}
+		} while (!WIFEXITED(status) && !WIFSIGNALED(status));
+
+		return true;
+	}
+
+	/* we are the child */
+	for(i = 0; i < 255; i++) {
+		close(i);
+	}
+
+	execlp(Config->RunOnReceive, Config->RunOnReceive, (char*)NULL);
+	exit(2);
+}
+#endif
+
 bool SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config, GSM_SMSDService *Service)
 {
 	bool			start,process;
@@ -295,7 +408,7 @@ bool SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config, GSM_SMSDService *Service)
 	start=true;
 	while (error == ERR_NONE && !gshutdown) {
 		sms.SMS[0].Folder=0x00;
-		error=Phone->GetNextSMS(&s, &sms, start);
+		error=GSM_GetNextSMS(gsm, &sms, start);
 		switch (error) {
 		case ERR_EMPTY:
 			break;
@@ -328,25 +441,28 @@ bool SMSD_ReadDeleteSMS(GSM_SMSDConfig *Config, GSM_SMSDService *Service)
 				}
 			}
 			if (process) {
-	 			Service->SaveInboxSMS(sms, Config);
+	 			Service->SaveInboxSMS(&sms, Config);
+				if (Config->RunOnReceive != NULL) {
+					SMSD_RunOnReceive(sms,Config);
+				}
 			} else {
 				WriteSMSDLog(_("Excluded %s"), buffer);
 			}
 			break;
 		default:
-	 		WriteSMSDLog(_("Error getting SMS (%i)"), error);
+	 		WriteSMSDLog(_("Error getting SMS (%s:%i)"), GSM_ErrorString(error), error);
 			return false;
 		}
 		if (error == ERR_NONE && sms.SMS[0].InboxFolder) {
 			for (i=0;i<sms.Number;i++) {
 				sms.SMS[i].Folder=0;
-				error=Phone->DeleteSMS(&s,&sms.SMS[i]);
+				error=GSM_DeleteSMS(gsm,&sms.SMS[i]);
 				switch (error) {
 				case ERR_NONE:
 				case ERR_EMPTY:
 					break;
 				default:
-					WriteSMSDLog(_("Error deleting SMS (%i)"), error);
+					WriteSMSDLog(_("Error deleting SMS (%s:%i)"), GSM_ErrorString(error), error);
 					return false;
 				}
 			}
@@ -362,9 +478,9 @@ bool SMSD_CheckSMSStatus(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 	GSM_Error		error;
 
 	/* Do we have any SMS in phone ? */
-	error=Phone->GetSMSStatus(&s,&SMSStatus);
+	error=GSM_GetSMSStatus(gsm,&SMSStatus);
 	if (error != ERR_NONE) {
-		WriteSMSDLog(_("Error getting SMS status (%i)"), error);
+		WriteSMSDLog(_("Error getting SMS status (%s:%i)"), GSM_ErrorString(error), error);
 		return false;
 	}
 	/* Yes. We have SMS in phone */
@@ -379,7 +495,16 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 	GSM_MultiSMSMessage  	sms;
 	GSM_DateTime         	Date;
 	GSM_Error            	error;
-	unsigned int         	i, j, z;
+	unsigned int         	j, z;
+	int			i;
+	unsigned int		second;
+	GSM_BatteryCharge  charge;
+	GSM_SignalQuality  network;
+
+	/* Clean structure before use */
+	for (i = 0; i < GSM_MAX_MULTI_SMS; i++) {
+		GSM_SetDefaultSMSData(&sms.SMS[i]);
+	}
 
 	error = Service->FindOutboxSMS(&sms, Config, Config->SMSID);
 
@@ -387,12 +512,14 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 		/* No outbox sms - wait few seconds and escape */
 		for (j=0;j<Config->commtimeout && !gshutdown;j++) {
 			GSM_GetCurrentDateTime (&Date);
-			i=Date.Second;
-	 		while (i==Date.Second && !gshutdown) {
-				my_sleep(10);
+			second = Date.Second;
+	 		while (second == Date.Second && !gshutdown) {
+				usleep(10000);
 				GSM_GetCurrentDateTime(&Date);
 			}
-			Service->RefreshPhoneStatus(Config);
+			GSM_GetBatteryCharge(gsm, &charge);
+			GSM_GetSignalQuality(gsm, &network);
+			Service->RefreshPhoneStatus(Config, &charge, &network);
 		}
 		return true;
 	}
@@ -427,7 +554,7 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 			if (sms.SMS[i].SMSC.Location == 1) {
 			    	if (Config->SMSC.Location == 0) {
 					Config->SMSC.Location = 1;
-					error = Phone->GetSMSC(&s,&Config->SMSC);
+					error = GSM_GetSMSC(gsm,&Config->SMSC);
 					if (error!=ERR_NONE) {
 						WriteSMSDLog(_("Error getting SMSC from phone"));
 						return false;
@@ -448,13 +575,15 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 				if ((strcmp(Config->deliveryreport, "no") != 0 && (Config->currdeliveryreport == -1))) sms.SMS[i].PDU = SMS_Status_Report;
 			}
 
-			error=Phone->SendSMS(&s, &sms.SMS[i]);
+			GSM_GetBatteryCharge(gsm, &charge);
+			GSM_GetSignalQuality(gsm, &network);
+			error=GSM_SendSMS(gsm, &sms.SMS[i]);
 			if (error!=ERR_NONE) {
 				Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_SENDING_ERROR, -1);
-				WriteSMSDLog(_("Error sending SMS %s (%i): %s"), Config->SMSID, error,print_error(error,s.di.df));
+				WriteSMSDLog(_("Error sending SMS %s (%i): %s"), Config->SMSID, error,GSM_ErrorString(error));
 				return false;
 			}
-			Service->RefreshPhoneStatus(Config);
+			Service->RefreshPhoneStatus(Config, &charge, &network);
 			j    = 0;
 			TPMR = -1;
 			SendingSMSStatus = ERR_TIMEOUT;
@@ -462,20 +591,20 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 				GSM_GetCurrentDateTime (&Date);
 				z=Date.Second;
 				while (z==Date.Second) {
-					my_sleep(10);
+					usleep(10000);
 					GSM_GetCurrentDateTime(&Date);
-					GSM_ReadDevice(&s,true);
+					GSM_ReadDevice(gsm,true);
 					if (SendingSMSStatus != ERR_TIMEOUT) break;
 				}
 				Service->RefreshSendStatus(Config, Config->SMSID);
-				Service->RefreshPhoneStatus(Config);
+				Service->RefreshPhoneStatus(Config, &charge, &network);
 				if (SendingSMSStatus != ERR_TIMEOUT) break;
 				j++;
 				if (j>Config->sendtimeout) break;
 			}
 			if (SendingSMSStatus != ERR_NONE) {
 				Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_SENDING_ERROR, TPMR);
-				WriteSMSDLog(_("Error getting send status of %s (%i): %s"), Config->SMSID, SendingSMSStatus,print_error(SendingSMSStatus,s.di.df));
+				WriteSMSDLog(_("Error getting send status of %s (%i): %s"), Config->SMSID, SendingSMSStatus,GSM_ErrorString(SendingSMSStatus));
 				return false;
 			}
 			error = Service->AddSentSMSInfo(&sms, Config, Config->SMSID, i+1, SMSD_SEND_OK, TPMR);
@@ -491,26 +620,26 @@ bool SMSD_SendSMS(GSM_SMSDConfig *Config,GSM_SMSDService *Service)
 	return true;
 }
 
-void SMSDaemon(int argc, char *argv[])
+void SMSDaemon(int argc UNUSED, char *argv[])
 {
-	int                     errors = 255, initerrors=0;
+	int                     errors = -1, initerrors=0;
 	GSM_SMSDService		*Service;
 	GSM_Error		error;
  	time_t			lastreceive, lastreset = 0;
 	GSM_SMSDConfig		Config;
 
-	if (!strcmp(argv[2],"FILES")) {
+	if (!strcasecmp(argv[2],"FILES")) {
 		Service = &SMSDFiles;
 #ifdef HAVE_MYSQL_MYSQL_H
-	} else if (!strcmp(argv[2],"MYSQL")) {
+	} else if (!strcasecmp(argv[2],"MYSQL")) {
 		Service = &SMSDMySQL;
 #endif
 #ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
-	} else if (!strcmp(argv[2],"PGSQL")) {
+	} else if (!strcasecmp(argv[2],"PGSQL")) {
 		Service = &SMSDPgSQL;
 #endif
 	} else {
-		fprintf(stderr,"Unknown service type (\"%s\")\n",argv[2]);
+		fprintf(stderr, _("Unknown SMSD service type (\"%s\")\n"), argv[2]);
 		exit(-1);
 	}
 
@@ -518,7 +647,7 @@ void SMSDaemon(int argc, char *argv[])
 
 	error = Service->Init(&Config);
 	if (error!=ERR_NONE) {
-		GSM_Terminate_SMSD(_("Stop GAMMU smsd (%i)"), error, true, -1);
+		GSM_Terminate_SMSD(_("Initialisation failed, stopping Gammu smsd"), error, true, -1);
 	}
 
 	signal(SIGINT, interrupt);
@@ -531,90 +660,104 @@ void SMSDaemon(int argc, char *argv[])
 
 	while (!gshutdown) {
 		/* There were errors in communication - try to recover */
-		if (errors > 2) {
-			if (errors != 255) {
-				WriteSMSDLog(_("Terminating communication (%i,%i)"), error, errors);
-				error=GSM_TerminateConnection(&s);
+		if (errors > 2 || errors == -1) {
+			if (errors != -1) {
+				WriteSMSDLog(_("Terminating communication %s, (%i, %i times)"),
+						GSM_ErrorString(error), error, errors);
+				error=GSM_TerminateConnection(gsm);
 			}
-			if (initerrors++ > 3) my_sleep(30000);
+			if (initerrors++ > 3) sleep(30);
 			WriteSMSDLog(_("Starting communication"));
-			error=GSM_InitConnection(&s,2);
+			error=GSM_InitConnection(gsm,2);
 			switch (error) {
 			case ERR_NONE:
-				s.User.SendSMSStatus 	= SMSSendingSMSStatus;
-				Phone			= s.Phone.Functions;
-				if (errors == 255) {
+				GSM_SetSendSMSStatusCallback(gsm, SMSSendingSMSStatus);
+				if (errors == -1) {
 					errors = 0;
-					s.Phone.Data.IMEI[0] = 0;
-					if (!(Phone->GetIMEI(&s))) {
+					if (GSM_GetIMEI(gsm, Config.IMEI) != ERR_NONE) {
 						errors++;
 					} else {
 						error = Service->InitAfterConnect(&Config);
 						if (error!=ERR_NONE) {
-							GSM_Terminate_SMSD(_("Stop GAMMU smsd (%i)"), error, true, -1);
+							GSM_Terminate_SMSD(_("Post initialisation failed, stopping Gammu smsd"), error, true, -1);
 						}
-						Phone->SetFastSMSSending(&s,true);
+						GSM_SetFastSMSSending(gsm, true);
 					}
 				} else {
 					errors = 0;
 				}
 				if (initerrors > 3 || initerrors < 0) {
-					error=Phone->Reset(&s, false); /* soft reset */
-					WriteSMSDLog(_("Reset return code: %s (%i) "), error == ERR_NONE? "OK":"ERROR", error);
+					error = GSM_Reset(gsm, false); /* soft reset */
+					WriteSMSDLog(_("Reset return code: %s (%i) "),
+							GSM_ErrorString(error),
+							error);
 					lastreset = time(NULL);
-					my_sleep(5000);
+					sleep(5);
 				}
-				/* Marcin Wiacek: FIXME. To check */
-//				di 			= s.di;
 				break;
 			case ERR_DEVICEOPENERROR:
-				GSM_Terminate_SMSD(_("Can't open device (%i)"), error, true, -1);
+				GSM_Terminate_SMSD(_("Can't open device"),
+						error, true, -1);
+				break;
 			default:
-				WriteSMSDLog(_("Error at init connection (%i)"), error);
+				WriteSMSDLog(_("Error at init connection %s (%i)"),
+						GSM_ErrorString(error), error);
 				errors = 250;
+				break;
 			}
 			continue;
 		}
 		if ((difftime(time(NULL), lastreceive) >= Config.receivefrequency) || (SendingSMSStatus != ERR_NONE)) {
 	 		lastreceive = time(NULL);
 
-			if (!SMSD_CheckSecurity(&Config)) {
+
+			if (Config.checksecurity && !SMSD_CheckSecurity(&Config)) {
 				errors++;
 				initerrors++;
 				continue;
-			} else errors=0;
+			} else {
+				errors=0;
+			}
 
 			initerrors = 0;
 
-			if (!SMSD_CheckSMSStatus(&Config,Service)) { /* read all incoming SMS */
+			/* read all incoming SMS */
+			if (!SMSD_CheckSMSStatus(&Config,Service)) {
 				errors++;
 				continue;
-			} else errors=0;
+			} else {
+				errors=0;
+			}
 
-			if (Config.resetfrequency > 0 && difftime(time(NULL), lastreset) >= Config.resetfrequency) { /* time for preventive reset */
-				errors = 254; initerrors = -2;
+			/* time for preventive reset */
+			if (Config.resetfrequency > 0 && difftime(time(NULL), lastreset) >= Config.resetfrequency) {
+				errors = 254;
+				initerrors = -2;
 				continue;
 			}
 		}
-		if (!SMSD_SendSMS(&Config,Service)) continue;
+		if (!SMSD_SendSMS(&Config,Service)) {
+			continue;
+		}
 	}
-	Phone->SetFastSMSSending(&s,false);
-	GSM_Terminate_SMSD(_("Stop GAMMU smsd"), 0, false, 0);
+	GSM_SetFastSMSSending(gsm,false);
+	GSM_Terminate_SMSD(_("Stopping Gammu smsd"), 0, false, 0);
 }
 
 GSM_Error SMSDaemonSendSMS(char *service, char *filename, GSM_MultiSMSMessage *sms)
 {
 	GSM_SMSDService		*Service;
 	GSM_SMSDConfig		Config;
+	GSM_Error error;
 
-	if (!strcmp(service,"FILES")) {
+	if (!strcasecmp(service,"FILES")) {
 		Service = &SMSDFiles;
 #ifdef HAVE_MYSQL_MYSQL_H
-	} else if (!strcmp(service,"MYSQL")) {
+	} else if (!strcasecmp(service,"MYSQL")) {
 		Service = &SMSDMySQL;
 #endif
 #ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
-	} else if (!strcmp(service,"PGSQL")) {
+	} else if (!strcasecmp(service,"PGSQL")) {
 		Service = &SMSDPgSQL;
 #endif
 	} else {
@@ -630,6 +773,20 @@ GSM_Error SMSDaemonSendSMS(char *service, char *filename, GSM_MultiSMSMessage *s
 	return Service->CreateOutboxSMS(sms,&Config);
 }
 
+GSM_Error SMSD_NoneFunction(void)
+{
+	return ERR_NONE;
+}
+
+GSM_Error SMSD_NotImplementedFunction(void)
+{
+	return ERR_NOTIMPLEMENTED;
+}
+
+GSM_Error SMSD_NotSupportedFunction(void)
+{
+	return ERR_NOTSUPPORTED;
+}
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
